<?php

class ConfCenter
{

    private static $_instance = null ;
    public $_confType ; //'serv','admin'
    public $_serv ;
    public $_curOne ; //vh or tp
    public $_info ;
    private $_disp ;
    private $_validator ;

    // prevent an object from being constructed
    private function __construct()
    {

    }

    public static function singleton()
    {

        if ( ! isset(self::$_instance) ) {
            $c = __CLASS__ ;
            self::$_instance = new $c ;
            self::$_instance->init() ;
        }

        return self::$_instance ;
    }

    private function init()
    {
        $mid = DUtil::grab_input("request", 'm') ;
        $this->_confType = ( $mid != null && $mid[0] == 'a' ) ? 'admin' : 'serv' ;

        $this->_disp = new DispInfo('confMgr.php?') ;

        $this->loadConfig() ;

        if ( CLIENT::singleton()->getTimeout() == 0 ) {
            $this->setSessionTimeout() ;
        }
    }

    public function GetValidator()
    {
        if ( $this->_validator == null )
            $this->_validator = new ConfValidation() ;
        return $this->_validator ;
    }

    function GetDispInfo()
    {
        return $this->_disp ;
    }

    public function &ExportConf()
    {
        $data = [] ;
        $listeners = [] ;

        if ( isset($this->_serv->_data['listeners']) ) {
            $lnames = array_keys($this->_serv->_data['listeners']) ;
            foreach ( $lnames as $ln ) {
                $listeners[$ln] = DUtil::getDataVal($this->_serv->_data['listeners'][$ln], 'address') ;
            }
        }

        $data['listener'] = $listeners ;
        $vhnames = [] ;
        if ( isset($this->_serv->_data['vhTop']) )
            $vhnames = array_keys($this->_serv->_data['vhTop']) ;
        if ( isset($this->_serv->_data['tpTop']) ) {
            foreach ( $this->_serv->_data['tpTop'] as $tp ) {
                if ( isset($tp['member']) ) {
                    $vhnames = array_merge($vhnames, array_keys($tp['member'])) ;
                }
            }
        }

        $data['vhost'] = $vhnames ;
        $data['serv'] = $this->_serv->_id ;
        $data['servLog'] = $this->getAbsFile(DUtil::getDataVal($this->_serv->_data['general']['log'], 'fileName'), 'SR') ;

        return $data ;
    }

    function GetAdminEmails()
    {
        return DUtil::getDataVal($this->_serv->_data['general'], 'adminEmails') ;
    }

    function setSessionTimeout()
    {
        $timeout = 600 ;
        if ( $this->_confType == 'admin' ) {
            $t = DUtil::getDataVal($this->_serv->_data['general'], 'sessionTimeout') ;
        }
        else {
            $adminFile = $this->GetConfFilePath('admin') ;
            $t = ConfigFileEx::grepTagValue($adminFile, 'sessionTimeout') ;
        }
        if ( ($t != null) && is_numeric($t) && ($t > 60) ) {
            $timeout = $t ;
        }


        CLIENT::singleton()->setTimeout($timeout) ;
    }

    function GetConfFilePath( $type, $name = '' )
    {
        $path = null ;
        if ( $type == 'serv' ) {
            $path = $_SERVER['LS_SERVER_ROOT'] . "conf/lslbd_config.xml" ; //fixed location
        }
        elseif ( $type == 'ha' ) {
            $path = $_SERVER['LS_SERVER_ROOT'] . "conf/ha_config.xml" ; //fixed location
        }
        elseif ( $type == 'admin' ) {
            $servconf = $this->GetConfFilePath('serv') ;
            $adminRoot = $this->getAbsFile('$SERVER_ROOT/admin/', 'SR') ; //fixed loc

            if ( $name == '' ) {
                $path = $adminRoot . 'conf/admin_config.xml' ;
            }
            elseif ( $name == 'key' ) {
                $path = $adminRoot . 'conf/jcryption_keypair' ;
            }
        }
        elseif ( $type == 'vh' ) {
            $path = $this->getAbsFile(DUtil::getDataVal($this->_serv->_data['vhTop'][$name], 'configFile'), 'SR', $name) ;
        }
        else {
            $path = $this->getAbsFile(DUtil::getDataVal($this->_serv->_data['tpTop'][$name], 'templateFile'), 'SR', $name) ;
        }
        return $path ;
    }

    function loadConfig()
    {
        $confpath = $this->GetConfFilePath($this->_confType) ;
        $this->_serv = new ConfData($this->_confType, $confpath) ;

        $config = new ConfigFile() ;

        $err = '' ;
        $res = $config->load($this->_serv, $err) ;
        if ( ! $res ) {
            $this->_info['CONF_FILE_ERR'] = $err ;
            return false ;
        }

        if ( $this->_confType == 'serv' ) {
            $serverName = DUtil::getDataVal($this->_serv->_data['general'], 'serverName') ;
            if ( $serverName == '$HOSTNAME' || $serverName == '' ) {
                $serverName = php_uname('n') ;
            }
            $this->_serv->setId($serverName) ;
            $this->_disp->init($this->_serv->_id) ;
        }
        $this->genMenuInfo() ;

        if ( in_array($this->_disp->_type, array( 'vh', 'tp', 'ha' )) ) {
            $vhpath = $this->GetConfFilePath($this->_disp->_type, $this->_disp->_name) ;
            $this->_curOne = new ConfData($this->_disp->_type, $vhpath, $this->_disp->_name) ;
            $res = $config->load($this->_curOne, $err) ;

            if ( ! $res ) {
                $this->_info['CONF_FILE_ERR'] = $err ;
                return false ;
            }

            if ( $this->_disp->_type == 'vh' ) {
                $this->_info['VH_NAME'] = $this->_disp->_name ;
                //$this->getAbsFile('$VH_ROOT/', 'VR', $this->_disp->_name); //set $this->_info['VH_ROOT']
            }
        }

        $this->loadSpecial() ;
        $viewActions = array( 'C' ) ;
        if ( ! in_array($this->_disp->_act, $viewActions) ) {
            $this->refreshInfo() ;
        }

        return true ;
    }

    function genMenuInfo()
    {
        if ( $this->_confType == 'serv' ) {
            $this->_info['LNS'] = [] ;
            $this->_info['LNS_menu'] = [] ;
            if ( isset($this->_serv->_data['listeners']) ) {
                $keys = array_keys($this->_serv->_data['listeners']) ;
                sort($keys) ;
                foreach ( $keys as $name ) {
                    $this->_info['LNS'][$name] = $name ;
                    $type = (DUtil::getDataVal($this->_serv->_data['listeners'][$name], 'type') == 'tcp') ? 'sl4' : 'sl' ;
                    $this->_info['LNS_menu'][$name] = $type ;
                }
            }

            $this->_info['CLS'] = [] ;
            $this->_info['CLS_L4'] = [] ;
            $this->_info['CLS_menu'] = [] ;
            if ( isset($this->_serv->_data['clusters']) ) {
                $keys = array_keys($this->_serv->_data['clusters']) ;
                sort($keys) ;
                foreach ( $keys as $name ) {
                    if ( 'layer7' == DUtil::getDataVal($this->_serv->_data['clusters'][$name], 'type')) {
                        $this->_info['CLS'][$name] = $name ;
                        $this->_info['CLS_menu'][$name] = 'lb' ;
                    }
                    else {
                        $this->_info['CLS_L4'][$name] = $name ;
                        $this->_info['CLS_menu'][$name] = 'lb4' ;
                    }
                }
            }

            $this->_info['VHS'] = [] ;

            if ( isset($this->_serv->_data['vhTop']) ) {
                $keys = array_keys($this->_serv->_data['vhTop']) ;
                sort($keys) ;
                foreach ( $keys as $name ) {
                    $this->_info['VHS'][$name] = $name ;
                }
            }
        }
    }

    function getAbsFile( $filename, $type, $vn = null )
    {
        // type = 'SR'
        if ( $vn != null && strpos($filename, '$VH_NAME') !== false ) {
            $filename = str_replace('$VH_NAME', $vn, $filename) ;
        }
        $s = substr($filename, 0, 1) ;
        if ( $s == '$' ) {
            if ( strncasecmp('$SERVER_ROOT', $filename, 12) == 0 ) {
                $filename = $_SERVER['LS_SERVER_ROOT'] . substr($filename, 13) ;
            }
        }
        elseif ( $s == '/' ) {
            if ( isset($_SERVER['LS_CHROOT']) ) {
                $root = $_SERVER['LS_CHROOT'] ;
                $len = strlen($root) ;
                if ( strncmp($filename, $root, $len) == 0 )
                    $filename = substr($filename, $len) ;
            }
        }
        else { // treat relative path to SERVER_ROOT
            $filename = $_SERVER['LS_SERVER_ROOT'] . $filename;
        }
        return $filename ;
    }

    function savePost( $extractData, $disp, $tbl, &$cur_ref )
    {
        $oldConf = &$this->getConfData($disp) ;
        $data = &DUtil::locateData($oldConf, $tbl->_dataLoc, $disp->_ref) ;
        $data0 = &$data ;

        $oldRef = $cur_ref ;
        $newRef = null ;

        if ( $tbl->_holderIndex != null ) {
            $newRef = DUtil::getDataVal($extractData, $tbl->_holderIndex) ;
            if ( $cur_ref != null && $cur_ref != $newRef && isset($data[$cur_ref]) ) {
                $data[$newRef] = $data[$cur_ref] ;
                unset($data[$cur_ref]) ;
            }
            $cur_ref = $newRef ;
            $data = &$data[$cur_ref] ;
        }

        if ( isset($tbl->_defaultExtract) ) {
            foreach ( $tbl->_defaultExtract as $k => $v ) {
                if ( $v == '$$' )
                    $data[$k] = $extractData[$k] ;
                else
                    $data[$k] = new CVal($v) ;
            }
        }

        $index = array_keys($tbl->_dattrs) ;
        foreach ( $index as $i ) {
            $attr = &$tbl->_dattrs[$i] ;
            if ( $attr == null || $attr->bypassSavePost() )
                continue ;
            $data[$attr->_key] = $extractData[$attr->_key] ;
        }

        if ( strncmp('VH_CTX', $tbl->_id, 6) == 0 && $oldRef == null ) {
            $data['order'] = new CVal(count($data0)) ;
        }
        elseif ( in_array($tbl->_id, array( 'VH_BASE', 'TP', 'L_GENERAL', 'ADMIN_L_GENERAL', 'LB_GENERAL', 'LB4_GENERAL', 'LT_GENERAL' )) ) {
            $oldRef = $disp->_name ;
            $newRef = DUtil::getDataVal($data, 'name') ;
        }
        $this->checkRefresh($tbl->_id, 's', $oldRef, $newRef, $disp) ;
    }

    function saveFile( $type, $name, $tid = null, $ref = null )
    {
        $config = new ConfigFile() ;
        $res = false ;
        $restart_required = true ;

        if ( $tid == 'SERV_MIME' ) {
            $res = ConfigFileEx::saveMime($this->_info['MIME_FILE'], $this->_serv->_data['MIME']) ;
        }
        elseif ( $tid == 'VH_UDB' ) {
            DUtil::trimLastId($ref) ;
            $ref = DUtil::getLastId($ref) ;
            $res = ConfigFileEx::saveUserDB($this->_info['UDB_FILE'], $this->_curOne->_data['UDB'][$ref]) ;
            $restart_required = false ;
        }
        elseif ( $tid == 'VH_GDB' ) {
            DUtil::trimLastId($ref) ;
            $ref = DUtil::getLastId($ref) ;
            $res = ConfigFileEx::saveGroupDB($this->_info['GDB_FILE'], $this->_curOne->_data['GDB'][$ref]) ;
            $restart_required = false ;
        }
        elseif ( $tid == 'ADMIN_USR' || $tid == 'ADMIN_USR_NEW' ) {
            $file = $_SERVER['LS_SERVER_ROOT'] . 'admin/conf/htpasswd' ;
            $res = ConfigFileEx::saveUserDB($file, $this->_serv->_data['ADM']) ;
            $restart_required = false ;
        }
        elseif ( $type == 'serv' || $type == 'sltop' || $type == 'sl' || $type == 'sl4'
                || $type == 'vhtop' || $type == 'tptop' || $type == 'lbtop' || $type == 'lb'
                || $type == 'lb4' || ($type == 'tp' && ($tid == 'TP_MEMBER' || $tid == 'TP' || $tid == 'TP_ZCONF')) 
                || ($type == 'vh' && stristr($tid, 'VH_BASE')) ) {
            $res = $config->save($this->_serv) ;
        }
        elseif ( $type == 'admin' || $type == 'altop' || $type == 'al' ) {
            $res = $config->save($this->_serv) ;
            $this->setSessionTimeout() ;
        }
        elseif ( $type == 'vh' || $type == 'tp' ) {
            if ( ! isset($this->_curOne) ) {
                $curpath = $this->GetConfFilePath($type, $name) ;
                $this->_curOne = new ConfData($type, $curpath, $name) ;
            }
            $res = $config->save($this->_curOne) ;
        }
        elseif ( $type == 'ha' ) {
            if ( ! isset($this->_curOne) ) {
                $curpath = $this->getConfFilePath($type, $name) ;
                $this->_curOne = new ConfData($type, $curpath, $name) ;
            }
            $res = $config->save($this->_curOne) ;
            $restart_required = false ;
        }

        if ( $restart_required && $res ) {
            CLIENT::singleton()->setChanged() ;
        }
        return $res ;
    }

    function &loadKeys( $list )
    {
        if ( isset($list) && count($list) > 0 ) {
            $temp = array_keys($list) ;
        }
        else {
            $temp = null ;
        }
        return $temp ;
    }

    function refreshInfo()
    {
        $handler = [] ;

        if ( isset($this->_serv->_data['ext']) ) {
            foreach ( $this->_serv->_data['ext'] as $name => $e ) {
                $handler[$e['type']->GetVal()][$name] = '[Server Level]: ' . $name ;
            }
        }
        $handler['cgi']['cgi'] = 'CGI Daemon' ;

        if ( isset($this->_curOne->_data['ext']) ) {
            foreach ( $this->_curOne->_data['ext'] as $name => $e ) {
                $handler[$e['type']->GetVal()][$name] = '[VHost Level]: ' . $name ;
            }
        }
        $this->_info['ext'] = $handler ;

        if ( isset($this->_curOne) && isset($this->_curOne->_data['sec']['realm']) ) {
            $realms = [] ;
            $rn = $this->loadKeys($this->_curOne->_data['sec']['realm']) ;
            if ( $rn != null ) {
                foreach ( $rn as $name ) {
                    $realms[$name] = $name ;
                }
            }
            $this->_info['realms'] = $realms ;
        }
    }

    private function checkRefresh( $tid, $act, $oldRef, $newRef, $disp )
    {
        if ( $tid == 'SERV_PROCESS' ) {
            $serverName = DUtil::getDataVal($this->_serv->_data['general'], 'serverName') ;
            if ( $serverName != $this->_serv->_id ) {
                $this->_serv->setId($serverName) ;
                $this->_disp->init($serverName) ;
            }
            return ;
        }

        $isRealm = (strncmp($tid, 'VH_REALM', 8) == 0) || (strncmp($tid, 'TP_REALM', 8) == 0) ;

        // for edit change name
        if ( $act == 's' && $oldRef != null && $oldRef != $newRef ) {
            if ( $isRealm ) {
                $this->changeName_REALM($oldRef, $newRef) ;
            }
            else {
                $this->changeName($oldRef, $newRef, $tid) ;
            }
        }

        // for add/delete, check if regnerate menu info
        $menuTids = array( 'L_GENERAL_NEW', 'LT_GENERAL_NEW', 'L_GENERAL_DEL',
            'LB4_GENERAL_NEW', 'LB_GENERAL_NEW', 'LB_GENERAL_DEL',
            'VH_TOP_D' ) ;
        // TP will always pick up from original data, so OK.
        if ( in_array($tid, $menuTids) ) {
            $this->genMenuInfo() ;
        }
        elseif ( $isRealm ) {
            $this->refreshInfo() ;
        }
    }

    private function changeName( $oldName, $newName, $tid )
    {
        $midPrefix = '' ;

        if ( $tid == 'VH_BASE' ) {
            $l = &$this->_serv->_data['listeners'] ;
            $lnames = array_keys($l) ;
            foreach ( $lnames as $ln ) {
                if ( isset($l[$ln]['vhMap'][$oldName]) ) {
                    $map = $l[$ln]['vhMap'][$oldName] ;
                    $map['vhost']->SetVal($newName) ;
                    unset($l[$ln]['vhMap'][$oldName]) ;
                    $l[$ln]['vhMap'][$newName] = $map ;
                }
            }
            $this->_serv->_data['vhTop'][$newName] = $this->_serv->_data['vhTop'][$oldName] ;
            unset($this->_serv->_data['vhTop'][$oldName]);
            $midPrefix = 'vh_' ;
            $this->_curOne->_id = $newName ;
        }
        elseif ( $tid == 'TP' ) {
            $this->_serv->_data['tpTop'][$newName] = $this->_serv->_data['tpTop'][$oldName] ;
            unset($this->_serv->_data['tpTop'][$oldName]) ;
            $midPrefix = 'tp_' ;
            $this->_curOne->_id = $newName ;
        }
        elseif ( $tid == 'L_GENERAL' ) {
            if ( !empty($this->_serv->_data['tpTop']) ) {
                $tns = array_keys($this->_serv->_data['tpTop']) ;
                foreach ( $tns as $tn ) {
                    $tl = &$this->_serv->_data['tpTop'][$tn]['listeners'] ;
                    $vals = DUtil::splitMultiple($tl->GetVal()) ;
                    foreach ( $vals as $i => $v ) {
                        if ( $v == $oldName )
                            $vals[$i] = $newName ;
                    }
                    $tl->SetVal(implode(', ', $vals)) ;
                }
            }
            $this->_serv->_data['listeners'][$newName] = $this->_serv->_data['listeners'][$oldName] ;
            unset($this->_serv->_data['listeners'][$oldName]);
            $midPrefix = 'sl_' ;
        }
        elseif ($tid == 'LT_GENERAL') {
            $this->_serv->_data['listeners'][$newName] = $this->_serv->_data['listeners'][$oldName] ;
            unset($this->_serv->_data['listeners'][$oldName]);
            $midPrefix = 'sl4_' ;
        }
        elseif ($tid == 'ADMIN_L_GENERAL' ) {
            $this->_serv->_data['listeners'][$newName] = $this->_serv->_data['listeners'][$oldName] ;
            unset($this->_serv->_data['listeners'][$oldName]);
            $midPrefix = 'al_' ;
        }
        elseif ( $tid == 'LB_GENERAL' ) {
            // default cluster for vh|tp
            if ( !empty($this->_serv->_data['tpTop']) ) {
                $tns = array_keys($this->_serv->_data['tpTop']) ;
                foreach ( $tns as $tn ) {
                    $tp = &$this->_serv->_data['tpTop'][$tn] ;
                    if (!empty($tp['member'])) {
                        $tmvn = array_keys($tp['member']);
                        foreach ($tmvn as $vn) {
                            if (isset($tp['member'][$vn]['defaultCluster']) && ($tp['member'][$vn]['defaultCluster']->GetVal() == $oldName)) {
                                $tp['member'][$vn]['defaultCluster']->SetVal($newName);
                            }
                        }
                    }
                }
            }
            if ( !empty($this->_serv->_data['vhTop']) ) {
                $vns = array_keys($this->_serv->_data['vhTop']) ;
                foreach ( $vns as $vn ) {
                    $vh = &$this->_serv->_data['vhTop'][$vn] ;
                    if ( isset($vh['defaultCluster']) && ($vh['defaultCluster']->GetVal() == $oldName)) {
                        $vh['defaultCluster']->SetVal($newName);
                    }
                }
            }

            $this->_serv->_data['clusters'][$newName] = $this->_serv->_data['clusters'][$oldName] ;
            unset($this->_serv->_data['clusters'][$oldName]);
            // name is referenced in VH/TP Context, but user has to change it mannually
            $midPrefix = 'lb_' ;
        }
        elseif ( $tid == 'LB4_GENERAL' ) {
            // layer 4 cluster, maybe referenced in layer 4 listener
            $l = &$this->_serv->_data['listeners'] ;
            $lnames = array_keys($l) ;
            foreach ( $lnames as $ln ) {
                if ( isset($l[$ln]['loadBalancer']) && ($l[$ln]['loadBalancer']->GetVal() == $oldName) ) {
                    $l[$ln]['loadBalancer']->SetVal($newName) ;
                }
            }
            $this->_serv->_data['clusters'][$newName] = $this->_serv->_data['clusters'][$oldName] ;
            unset($this->_serv->_data['clusters'][$oldName]);
            $midPrefix = 'lb4_' ;
        }

        if ( $midPrefix ) {
            $this->_disp->_mid = $midPrefix . $newName ;
            $this->_disp->_name = $newName ;
            $this->_disp->init() ;
        }
    }

    function changeName_REALM( $oldName, $newName )
    {
        $data = &$this->_curOne->_data ;
        if ( $data['context'] != null ) {
            $keys = array_keys($data['context']) ;
            foreach ( $keys as $key ) {
                $ctx = &$data['context'][$key] ;
                if ( isset($ctx['realm']) && $ctx['realm']->GetVal() == $oldName )
                    $ctx['realm']->SetVal($newName) ;
            }
        }
    }

    function &getConfData( $disp )
    {
        $data = null ;
        if ( $disp->_type == 'vh' ) {
            if ( $disp->_pid == 'base' ) {
                $data = &$this->_serv->_data['vhTop'][$disp->_name] ;
            }
            else {
                $data = &$this->_curOne->_data ;
            }
        }
        elseif ( $disp->_type == 'tp' ) {
            if ( $disp->_pid == 'member' ) {
                $data = &$this->_serv->_data['tpTop'][$disp->_name] ;
            }
            else {
                $data = &$this->_curOne->_data ;
            }
        }
        elseif ( $disp->_type[0] == 'a' ) {
            if ( $disp->_type == 'al' ) {
                $data = &$this->_serv->_data['listeners'][$disp->_name] ;
            }
            else {
                $data = &$this->_serv->_data ;
            }
        }
        elseif ( $disp->_type == 'ha' ) {
            $data = &$this->_curOne->_data ;
        }
        else {
            if ( $disp->_type == 'sl' || $disp->_type == 'sl4' ) {
                $data = &$this->_serv->_data['listeners'][$disp->_name] ;
            }
            elseif ( $disp->_type == 'lb' || $disp->_type == 'lb4' ) {
                $data = &$this->_serv->_data['clusters'][$disp->_name] ;
            }
            elseif ( $disp->_type == 'tptop' && $disp->_pid == 'tpTop1' ) {
                $data = &$this->_serv->_data['tpTop'][$disp->_ref] ;
            }
            else {
                $data = &$this->_serv->_data ;
            }
        }
        return $data ;
    }

    function loadSpecial()
    {
        $tid = DUtil::getLastId($this->_disp->_tid) ;

        if ( $this->_disp->_type == 'serv' ) {

            if ( $tid == 'SERV_MIME_TOP' || $tid == 'SERV_MIME' ) {
				$file = $this->getAbsFile(DUtil::getDataVal($this->_serv->_data['general'], 'mime'), 'SR') ;
                $this->_serv->_data['MIME'] = &ConfigFileEx::loadMime($file) ;
                $this->_info['MIME_FILE'] = $file ;
            }
        }
        elseif (strpos($this->_disp->_type, 'sl') !== false || $tid == 'LB_WORKERGRP' || $tid == 'LB4_WORKERGRP') {
            if ($this->_disp->_act == 'n' || $this->_disp->_act == 'e' || $this->_disp->_act == 's') { // for new & edit & save only
                $file = $this->GetConfFilePath('ha');
                $vips = ConfigFileEx::loadHAVip($file);
                if ($vips != null) {
                    $attr = DTblDef::GetInstance()->GetCommonAttr('l_ip') ;
                    $attr->_maxVal = array_merge($attr->_maxVal, $vips);
                }
            }
        }
        elseif ( $this->_disp->_type == 'admin' ) {

            if ( $this->_disp->_pid == 'security' ) {
                $file = $_SERVER['LS_SERVER_ROOT'] . 'admin/conf/htpasswd' ;
                $this->_serv->_data['ADM'] = ConfigFileEx::loadUserDB($file) ;
            }
        }
        elseif ( $this->_disp->_type == 'vh' ) {
            if ( $tid == 'VH_UDB' || $tid == 'VH_GDB' ) {
                $temp = explode('`', $this->_disp->_ref) ;
                $ref = $temp[0] ;
            }
            else {
                $ref = DUtil::getLastId($this->_disp->_ref) ;
            }

            if ( $tid == 'VH_UDB_TOP' || $tid == 'VH_UDB' ) {
                $loc = DUtil::getDataVal($this->_curOne->_data['sec']['realm'][$ref], 'userDB:location') ;
                $file = $this->getAbsFile($loc, 'VR', $this->_info['VH_NAME']) ;
                $this->_info['UDB_FILE'] = $file ;
                $this->_curOne->_data['UDB'][$ref] = ConfigFileEx::loadUserDB($file) ;
            }
            elseif ( $tid == 'VH_GDB_TOP' || $tid == 'VH_GDB' ) {
                $loc = DUtil::getDataVal($this->_curOne->_data['sec']['realm'][$ref], 'groupDB:location') ;
                $file = $this->getAbsFile($loc, 'VR', $this->_info['VH_NAME']) ;
                $this->_info['GDB_FILE'] = $file ;
                $this->_curOne->_data['GDB'][$ref] = ConfigFileEx::loadGroupDB($file) ;
            }
        }
    }

    private function &doAction( $disp )
    {
        if ( $disp->_act != 'v' ) {
            //if (strpos($_SERVER['HTTP_REFERER'], "{$_SERVER['HTTP_HOST']}/config/confMgr.php") === false) {
            //	$disp->_err = 'illegal entry!';
            //	return null;
            //}
            if ( $disp->_token != $disp->_tokenInput ) {
                $disp->_err = "illegal entry point!" ;
                return null ;
            }
        }

        $disp->_info = &$this->_info ;

        //init here
        $data = &$this->getConfData($disp) ;

        $tid = DUtil::getLastId($disp->_tid) ;
        $ref = DUtil::getLastId($disp->_ref) ;
        $needTrim = false ;
        $tblDef = DTblDef::GetInstance() ;

        if ( $disp->_act == 's' ) {
            $tbl = $tblDef->GetTblDef($tid) ;
            $extractedData = null ;
            if ( $tbl->_holderIndex != null ) {
                $curdata = DUtil::locateData($data, $tbl->_dataLoc) ;
                $disp->_info['holderIndex'] = array_keys($curdata) ;
                $disp->_info['holderIndex_cur'] = $ref ;
            }
            else
                $disp->_info['holderIndex'] = null ;

            $go = $this->GetValidator()->ExtractPost($tbl, $extractedData, $disp) ;
            if ( $go > 0 ) {
                $this->savePost($extractedData, $disp, $tbl, $ref) ;
                $this->saveFile($disp->_type, $disp->_name, $tbl->_id, $disp->_ref) ;
                $needTrim = true ;
            }
            else {
                $disp->_act = 'S' ;
                $disp->_err = 'Input error detected. Please resolve the error(s). ' ;
                return $extractedData ;
            }
        }
        else if ( $disp->_act == 'c' || $disp->_act == 'n' ) {
            $tbl = $tblDef->GetTblDef($tid) ;
            $extractedData = [] ;
            $this->GetValidator()->ExtractPost($tbl, $extractedData, $disp) ;
            if ( $disp->_act == 'n' ) {
                $tid = DUtil::getSubTid($tbl->_subTbls, $extractedData) ;
                if ( $tid != null ) {
                    DUtil::switchLastId($disp->_tid, $tid) ;
                    $tbl = $tblDef->GetTblDef($tid) ;
                }
            }
            return $extractedData ;
        }
        else if ( $disp->_act == 'D' ) {
            $tbl = $tblDef->GetTblDef($tid) ;
            $oldConf = &$this->getConfData($disp) ;
            $data = &DUtil::locateData($oldConf, $tbl->_dataLoc, $disp->_ref) ;
            if ( ($disp->_tid == 'ADMIN_L_GENERAL1') || ($disp->_pid == 'security' && $disp->_tid == 'ADMIN_USR') ) {
                // for admin listener, do not allow to delete all
                // for admin user, do not allow to delete all
                if ( count($data) == 1 ) {
                    DUtil::trimLastId($disp->_tid) ;
                    DUtil::trimLastId($disp->_ref) ;
                    return $oldConf ;
                }
            }
            unset($data[$ref]) ;

            if ( strncmp('VH_CTX', $tbl->_id, 6) == 0 )
                $this->resetCtxSeq($data) ;

            $this->saveFile($disp->_type, $disp->_name, $tid, $disp->_ref) ;
            $data = &$this->getConfData($disp) ; //reload everything
            $this->checkRefresh($tid, 'D', $ref, null, $disp) ;
            $needTrim = true ;
        }
        else if ( $disp->_act == 'I' ) {
            if ( $this->instantiateTempl($disp) ) {
                $tbl = $tblDef->GetTblDef($tid) ;
                $oldConf = &$this->getConfData($disp) ;
                $data = &DUtil::locateData($oldConf, $tbl->_dataLoc) ;
                unset($data[$ref]) ;
                $this->saveFile($disp->_type, $disp->_name, $tid) ;

                $this->genMenuInfo('serv') ;
                $data = &$this->getConfData($disp) ; //reload everything
                $needTrim = true ;
            }
        }
        else if ( $disp->_act == 'B' ) {
            $needTrim = true ;
        }

        if ( $tid == 'HA_STATUS' && in_array($disp->_act, array( 'a', 'y', 'z', 'd', 'r' )) ) {
            $res = $this->haControlCommand($disp->_act, $ref, $data) ;
            if ( $res != 0 ) {
                $disp->_err = 'HA Control Operation failed: ' . $res ;
            }
            $this->loadSpecial() ; // looks like not needed
            $needTrim = true ;
            $disp->_act = 'v' ;
        }

        if ( isset($_GET['ctxseq']) ) {
            if ( $this->changeCtxSeq($disp, DUtil::getGoodVal1($_GET['ctxseq'])) )
                $this->saveFile($disp->_type, $disp->_name, $tid) ;
        }


        if ( $needTrim ) {
            DUtil::trimLastId($disp->_tid) ;
            DUtil::trimLastId($disp->_ref) ;
        }
        return $data ;
    }

    public function haControlCommand( $act, $ref, $data )
    {
        $cmd = 'HA_CMD: ' ;
        if ( $act == 'r' ) {
            $cmd .= 'refresh' ;
        }
        elseif ( $act == 'd' ) {
            $cmd .= 'delete ' . $ref ;
            // before has z for disable
        }
        else {
            if ( $act == 'y' )
                $cmd .= 'edit' ;
            elseif ( $act == 'a' )
                $cmd .= 'add' ;

# Parameter index:
# $1 Command (add, etc)
# --idx vrrp instance index
# --ip Comma delimited list of virtual IPs for the LOCAL MACHINE.
# --vid Virtual Router ID. Used to identify the local machine.
# --pass The authentication password.
# --priority Priority value to select master. Higher number indicates higher priority.
# --nic VIP Interface Device. (VIP output)
# --int_dev Internal Interface device. (unicast/multicast communication)
# --unicastip [OPTIONAL] Unicast Src IP. This is the local machine's virtual IP.
# --unicastpeers [OPTIONAL] Comma delimited list of virtual IPs for the peers.
#
# For --ip and --unicastpeers, if using a list, should surround the parameter with quotes.

            $d = $data['vipconf'][$ref] ;
            $vip = DUtil::getDataVal($d, 'vip') ;
            $vipId = DUtil::getDataVal($d, 'vipId') ;
            $vipToken = DUtil::getDataVal($d, 'vipToken') ;
            $priority = DUtil::getDataVal($d, 'priority') ;
            $nic = DUtil::getDataVal($d, 'nic') ;

            $cmd .= " --idx $ref --ip '$vip' --vid $vipId --pass '$vipToken' --priority $priority --nic $nic" ;

            $internal_nic = DUtil::getDataVal($d, 'internal_dev') ;
            if ($internal_nic)
            $cmd .= " --int_dev $internal_nic";

            $unisrc = DUtil::getDataVal($d, 'unicastSrcIp') ;
            if ($unisrc) {
                $unipeer = DUtil::getDataVal($d, 'unicastPeerIps') ;
                $cmd .= " --unicastip '$unisrc' --unicastpeers '$unipeer'";
            }
        }
        error_log("hactrlcmd = $cmd \n") ;
        $result = Service::retrieveCommandData($cmd, 5) ;
        $res = substr($result, strpos($result, 'RESULT: ') + 8) ;
        return $res ;
        //echo ("result = $result \n <br/>");
    }

    function resetCtxSeq( &$data )
    {
        $keys = array_keys($data) ;
        $i = 1 ;
        foreach ( $keys as $k ) {
            $data[$k]['order']->SetVal($i) ;
            ++ $i ;
        }
    }

    function changeCtxSeq( $disp, $seq )
    {
        $plus = true ;
        if ( $seq < 0 ) {
            $plus = false ;
            $seq = - $seq ;
        }

        $ctxs = &$this->_curOne->_data['context'] ;

        $c = count($ctxs) ;
        if ( ( ! $plus && $seq == 1) || ($plus && $seq == $c) )
            return false ;

        $newCtxs = [] ;
        $i = 1 ;
        foreach ( $ctxs as $uri => $ctx ) {
            if ( $plus ) {
                if ( $i == $seq ) {
                    $savedUri = $uri ;
                    $saved = $ctx ;
                }
                else if ( $i == $seq + 1 ) {
                    $ctx['order']->SetVal($seq) ;
                    $newCtxs[$uri] = $ctx ;
                    $saved['order']->SetVal($seq + 1) ;
                    $newCtxs[$savedUri] = $saved ;
                }
                else
                    $newCtxs[$uri] = $ctx ;
            }
            else {
                if ( $i == $seq - 1 ) {
                    $savedUri = $uri ;
                    $saved = $ctx ;
                }
                else if ( $i == $seq ) {
                    $ctx['order']->SetVal($seq - 1) ;
                    $newCtxs[$uri] = $ctx ;
                    $saved['order']->SetVal($seq) ;
                    $newCtxs[$savedUri] = $saved ;
                }
                else
                    $newCtxs[$uri] = $ctx ;
            }
            ++ $i ;
        }

        $this->_curOne->_data['context'] = &$newCtxs ;
        return true ;
    }

    function instantiateTempl( $disp )
    {
        $tn = $disp->_name ;
        $tp = &$this->_serv->_data['tpTop'][$tn] ;
        if ( $tp['listeners']->HasErr() ) {
            $disp->_err = 'Please fix error in Mapped Listeners first.' ;
            return false ;
        }

        $vn = $disp->_ref ;
        $d = $this->_curOne->_data ;

        $configFile = $d['general']['configFile'] ;
        if ( empty($configFile) || $configFile->HasErr() ) {
            $disp->_err = 'Please fix error in General->Config File first.' ;
            return false ;
        }

        // replace $VH_NAME
        $configFile->SetVal(str_replace('$VH_NAME', $vn, $d['general']['configFile']->GetVal())) ;

        $path = $configFile->GetVal() ;
        $path = $this->getAbsFile($path, 'SR', $vn) ;
        if ( ! file_exists($path) && ! PathTool::createFile($path, $disp->_err) ) {
            return false ;
        }
        $confData = new ConfData('vh', $path, $vn) ;
        $confData->_data = &$this->_curOne->_data ;

        $config = new ConfigFile() ;
        $res = $config->save($confData) ;
        if ( ! $res ) {
            $disp->_err = 'failed to save to file ' . $path ;
            return false ;
        }

        $v = [] ;
        $v['name'] = new CVal($vn) ;
        $v['configFile'] = new CVal($path) ;
        $v['defaultCluster'] = $tp['member'][$vn]['defaultCluster'] ;

        $this->_serv->_data['vhTop'][$vn] = $v ;

        $lns = preg_split("/, /", $tp['listeners']->GetVal(), -1, PREG_SPLIT_NO_EMPTY) ;
        $vhmap['vhost'] = new CVal($vn) ;
        $domains = DUtil::getDataVal($tp['member'][$vn], 'vhDomain') ;
		$vhalias = DUtil::getDataVal($tp['member'][$vn], 'vhAliases');
        if ( $vhalias != '' ) {
            if ( $domains != '' )
                $domains .= ', ' ;
            $domains .= $vhalias ;
        }
        $vhmap['domain'] = new CVal($domains) ;
        foreach ( $lns as $ln ) {
            $l = &$this->_serv->_data['listeners'][$ln] ;
            $l['vhMap'][$vn] = $vhmap ;
        }

        return true ;
    }

    function enableDisableVh( $act, $actId )
    {
        $haschanged = false ;
        $cur_disabled = [] ;
        if ( isset($this->_serv->_data['service']['suspendedVhosts']) ) {
            $cur_disabled = preg_split("/[,;]+/", DUtil::getDataVal($this->_serv->_data['service'], 'suspendedVhosts'), -1, PREG_SPLIT_NO_EMPTY) ;
        }
        $found = in_array($actId, $cur_disabled) ;
        if ( $act == 'disable' ) {
            if ( ! $found ) {
                $cur_disabled[] = $actId ;
                $haschanged = true ;
            }
        }
        else if ( $act == 'enable' ) {
            if ( $found ) {
                $key = array_search($actId, $cur_disabled) ;
                unset($cur_disabled[$key]) ;
                $haschanged = true ;
            }
        }
        if ( $haschanged ) {
            $vals = implode(',', $cur_disabled) ;
            $this->_serv->_data['service']['suspendedVhosts'] = new CVal($vals) ;
            $this->saveFile('serv', '') ;
        }
    }

    public function GetPageData( $disp, &$confErr )
    {
        $disp->_err = null ;

        $data = &$this->doAction($disp) ;

        if ( isset($this->_info['CONF_FILE_ERR']) ) {
            $confErr = $this->_info['CONF_FILE_ERR'] ;
        }
        return $data ;
    }

}
