<?php

class ConfValidation extends CValidation
{
	// to hold special validation
	protected function validatePostTbl($tbl, &$d)
	{
		$isValid = parent::validatePostTbl($tbl, $d);
        $extra_tables = ['HA_CONF', 'SERV_QUIC', 'ZCONF_GENERAL'];
        if (in_array($tbl->_id, $extra_tables)) {
            $func = 'chkPostTbl_' . $tbl->_id;
            $res = $this->$func($d);
            $this->setValid($isValid, $res);
        }
		return $isValid;
	}

	protected function chkPostTbl_HA_CONF(&$d)
	{
		$isValid = 1;

        $confCenter = ConfCenter::singleton();
        $vipconf = $confCenter->_curOne->_data['vipconf'];

        $oldId = $this->_info['holderIndex_cur'];
        $newvipId = $d['vipId']->GetVal();
        $newvip = $d['vip']->GetVal();
        foreach($vipconf as $id => $c) {
            if ($oldId != $id) {
                if ($c['vipId']->GetVal() == $newvipId) {
                    $d['vipId']->SetErr('This value has been used. Please enter a unique one.');
                    $isValid = -1;
                }
                if ($c['vip']->GetVal() == $newvip) {
                    $d['vip']->SetErr('This value has been used. Please enter a unique one.');
                    $isValid = -1;
                }
            }
        }
        $unisrc = $d['unicastSrcIp']->GetVal();
        $unipeer = $d['unicastPeerIps']->GetVal();
        if ($unisrc != '' && $unipeer == '') {
            $d['unicastPeerIps']->SetErr('This has to be set together with Unicast Source IP.');
            $isValid = -1;
        } elseif ($unisrc == '' && $unipeer != '') {
            $d['unicastSrcIp']->SetErr('This has to be set together with Unicast Peer IPs.');
            $isValid = -1;
        }

		return $isValid;
	}

	protected function chkPostTbl_SERV_QUIC(&$d)
	{
		$isValid = 1;

        $rates = ['quicBadPacketRate', 'quicHskNewRate', 'quicHskTimeoutRate'];
        foreach( $rates as $rate) {
            $v = $d[$rate]->GetVal();
            if ($v !== '') {
                $vs = preg_split("/\//", $v, -1, PREG_SPLIT_NO_EMPTY);
                if ((count($vs) != 2)
                        || (intval($vs[0]) != $vs[0])
                        || (intval($vs[1]) != $vs[1])) {
                    $d[$rate]->SetErr('Invalid format.');
                    $isValid = -1;
                }
                elseif ($vs[0] < 0 || $vs[1] < 0 || $vs[1] > 30) {
                    $d[$rate]->SetErr('Number out of range.');
                    $isValid = -1;
                }
                else {
                    $d[$rate]->SetVal(sprintf("%d/%d", $vs[0], $vs[1]));
                }
            }
        }

		return $isValid;
	}
    
    protected function chkPostTbl_ZCONF_GENERAL(&$d)
    {
        $isValid = 1;
        if ($d['logPerDomain']->GetVal() > 0 ) {
            if (!$d['domLogDir']->HasVal()) {
                $d['domLogDir']->SetErr('Value must be set.');
                $isValid = -1;
            }
            $path = $d['domLogDir']->getVal();
            if (substr($path, 0, 1) !== '/' && (strncasecmp('$SERVER_ROOT', $path, 12) != 0)) {
                $d['domLogDir']->SetErr('path needs to start with / or $SERVER_ROOT');
                $isValid = -1;
            }
        }
        return $isValid;
    }

}
