<?php

class DTblDef
{
	private $_tblDef = [];
	private $_options;
	private $_attrs;

	public static function GetInstance()
	{
        if ( !isset($GLOBALS['_DTblDef_']) )
			$GLOBALS['_DTblDef_'] = new DTblDef();
		return $GLOBALS['_DTblDef_'];
	}

	private function __construct()
	{
		$this->loadCommonOptions();
		$this->loadCommonAttrs();
	}

	public function GetTblDef($tblId)
	{
		if (!isset( $this->_tblDef[$tblId] ))
		{
			$funcname = 'add_' . $tblId;
			if (!method_exists($this, $funcname))
				die("invalid tid $tblId");
			$this->$funcname($tblId);
		}
		return $this->_tblDef[$tblId];
	}

	private function DupTblDef($tblId, $newId, $newTitle=NULL)
	{
		$tbl = $this->GetTblDef($tblId);
		$newtbl = $tbl->dup($newId, $newTitle);
		return $newtbl;
	}

	private function loadCommonOptions()
	{
		$this->_options = [];
		$this->_options['text_size'] = 'size="95"';
		$this->_options['tp_vname'] = array( '/\$VH_NAME/', 'requiring variable $VH_NAME');
		$this->_options['tp_vroot'] = array( '/(\$VH_NAME)|(\$VH_ROOT)/',
											  'requiring variable $VH_NAME or $VH_ROOT');

		$this->_options['logHeaders'] = array('1'=>'Referrer', '2'=>'UserAgent', '4'=>'Host', '0'=>'None');
        $this->_options['recaptcha'] = array('0' => 'Not Set', '1' => 'Checkbox', '2' => 'Invisible', '3' => 'hCaptcha');


		$this->_options['workerType'] = array(
				'fcgi'=>'Fast CGI App',
				'lsapi'=>'LSAPI App',
				'servlet'=>'Servlet Engine', 'proxy'=>'HTTP Proxy', 'https' => 'HTTPS Proxy');

		$this->_options['ext_respBuffer'] = array('0'=>'No', '1'=>'Yes',
												  '2'=>'No for Non-Parsed-Header (NPH)');

		$this->_options['chrootmode'] = array( 0=>'Same as Server', 1=>'Virtual Host Root', 2=>'Customized Chroot Path');

		$this->_options['ctxType'] = array(
				'loadbalancer'=> 'Load Balancer',
				'redirect'=>'Redirect');

		$this->_options['ctxTbl'] = array(
				0=>'type', 1=>'VH_CTXB',
				'loadbalancer'=>'VH_CTXB',
				'redirect'=>'VH_CTXR');

        $this->_options['lsnTbl'] = array(
            0=>'type', 1=>array('L_GENERAL', 'L_VHMAP', 'L_SSL_CERT', 'L_SSL', 'L_SSL_FEATURE', 'L_SSL_OCSP', 'L_SSL_CLIENT_VERIFY'),
            'http' => array('L_GENERAL', 'L_VHMAP', 'L_SSL_CERT', 'L_SSL', 'L_SSL_FEATURE', 'L_SSL_OCSP', 'L_SSL_CLIENT_VERIFY'),
            'tcp' => array('LT_GENERAL', 'LT_CLMAP')
        );

        $this->_options['lsnTblNew'] = array( // for new
            0=>'type', 1=>'L_GENERAL_NEW',
            'http' => 'L_GENERAL_NEW',
            'tcp' => 'LT_GENERAL_NEW'
        );

        $this->_options['clusterTbl'] = array(
            0=>'type',
            'layer7' => array('LB_GENERAL', 'LB_SESSION', 'LB_ERRMSG', 'LB_WORKERGRP'),
            'layer4' => array('LB4_GENERAL', 'LB4_WORKERGRP')
        );

        $this->_options['clusterTblNew'] = array( // for new
            0=>'type',
            'layer7' => 'LB_GENERAL_NEW',
            'layer4' => 'LB4_GENERAL_NEW'
        );

		$this->_options['on_off'] = array('off'=>'Off', 'on'=>'On');

		$this->_options['realmType'] = array('file' => 'Password File', 'LDAP' => 'LDAP' );
		$this->_options['realmTbl'] = array( 0=>'type', 1=>'VH_REALM_LDAP', 'file' => 'VH_REALM_FILE', 'LDAP' => 'VH_REALM_LDAP' );
		$this->_options['tp_realmTbl'] = array( 0=>'type', 1=>'VH_REALM_LDAP', 'file' => 'TP_REALM_FILE', 'LDAP' => 'VH_REALM_LDAP' );

		$this->_options['lbType'] = array('layer7' => 'Layer 7', 'layer4' => 'Layer 4');
        $this->_options['l_type'] = array('http' => 'HTTP (Layer 7)', 'tcp' => 'TCP (Layer 4)');

		$this->_options['logLevel'] = array('ERROR'=>'ERROR', 'WARN'=>'WARNING',
									'NOTICE'=>'NOTICE', 'INFO'=>'INFO', 'DEBUG'=>'DEBUG');
		$this->_options['updateInterval'] = array( 86400=>'Daily', 604800=>'Weekly', 2592000=>'Monthly', 0=>'Disable' );

		$this->_options['ha_policy'] = array('policy1' => 'Fail Over Policy 1', 'policy2' => 'Fail Over Policy 2', 'policy3' => 'Fail Over Policy 3');

		$this->_options['respBuffer'] = ['0' => 'No', '1' => 'Yes',
			'2' => 'No for Non-Parsed-Header (NPH)'];


		$ipv6 = [];
        $nic = [];
        $label = 'LS' . strtoupper($_SERVER['LS_PRODUCT']);
		if ( $_SERVER[$label . '_IPV6_ADDRS'] != '' )
		{
			$ipv6['[ANY]'] = '[ANY] IPv6';
			$ips = explode( ',', $_SERVER[$label . '_IPV6_ADDRS'] );
			foreach( $ips as $ip )
				{
					$pos = strpos( $ip, ':' );
					if ( $pos )
					{
						$aip = substr( $ip, $pos+1 );
						$ipv6[$aip] = $aip;
                        $nicname = substr($ip, 0, $pos);
                        if (!isset($nic[$nicname]))
                            $nic[$nicname] = $nicname;
					}
				}
		}
		$ipo = [];
		$ipo['ANY'] = 'ANY IPv4';
		$ips = explode(',', $_SERVER[$label . '_IPV4_ADDRS']);
		foreach( $ips as $ip )
		{
			$pos = strpos($ip, ':');
			if ( $pos )
			{
				$aip = substr($ip, $pos+1);
                $nicname = substr($ip, 0, $pos);
                if (!isset($nic[$nicname])) {
                    $nic[$nicname] = $nicname;
				}
				$ipo[$aip] = $aip;
				if ( $aip != '127.0.0.1' ) {
					$ipv6["[::FFFF:$aip]"] = "[::FFFF:$aip]";
				}
			}
		}
		if ( $_SERVER[$label . '_IPV6_ADDRS'] != '' ) {
			$this->_options['ip'] = $ipo + $ipv6;
		} else {
			$this->_options['ip'] = $ipo;
		}
        $this->_options['nic'] = $nic;

	}


	private function loadCommonAttrs()
	{
		$ctxOrder = new DAttr('order', 'cust', 'Order');
		$ctxOrder->_FDE = 'NNN';

        $l_addr = new DAttr('address', 'cust', 'Address', NULL, false);
        $l_addr->_FDE = 'YNN';

        $l_ip = new DAttr('ip', 'sel', 'IP Address', 'select', false, 0, $this->_options['ip'], NULL, 0, 'listenerIP');
		$l_ip->_FDE = 'NYY';

   		$l_port = new DAttr('port', 'uint', 'Port', 'text', false, 0, 65535, NULL, 0, 'listenerPort');
		$l_port->_FDE = 'NYY';

		$nodeAddr = new DAttr('nodeAddresses', 'parse', 'Node Address List', 'textarea', false, "/^(\(.+\))?([^)(]+\->)?[^\->)(]+(:[0-9]+)?$/", "[(NodeTag)][SourceIP->]DestinationAddress[:DestinationPort]", 'rows="4" cols="50"', 1);
		$nodeAddr->_note = 'comma separated list of [(NodeTag)][SourceIP->]DestinationAddress[:DestinationPort]. Note: IPv6 must be surrounded by [ ]. Example: 192.168.0.11, (node1)192.168.0.11, (node1)192.168.0.10->192.168.0.11:80, [2a0b:4342:4000:bd::d627]';

		$l_type = new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['l_type'], NULL, 0, 'listenerType');
        $l_type_file = new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['l_type'], NULL, 0, 'listenerType');
        $l_type_file->_FDE = 'YYN';

        $lb_type = new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['lbType'], NULL, 0, 'lb_type');
        $lb_type_file = new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['lbType'], NULL, 0, 'lb_type');
        $lb_type_file->_FDE = 'YYN';

		$processes = isset($_SERVER['LSLB_CHILDREN']) ? $_SERVER['LSLB_CHILDREN'] : 1;
		if ($pos = strpos($processes, ':')) {
			$processes = substr($processes, $pos+1);
		}
        $processes = (int)$processes;
		for( $i = 1; $i <= $processes; ++$i )
			$bindoptions[1<<($i-1)] = "Process $i";

		$binding = new DAttr('binding', 'checkboxOr', 'Binding', 'checkboxgroup', true, NULL, $bindoptions);
		$binding->_version = 3; //3: LSLB 2CPU +
		$binding->_helpKey = 'listenerBinding';
        
		$this->_attrs = array(
			'indexFiles' => new DAttr('indexFiles', 'fname', 'Index Files', 'textarea', true, NULL, NULL, 'rows="2" cols="60"', 1),
			'autoIndex' => new DAttr('autoIndex', 'bool', 'Auto Index', 'radio'),
			'suffix' => new DAttr('suffix', 'parse', 'Suffixes', 'text', false,
								  "/^[A-z0-9_\-]+$/", 'comma-separated list, allowed character[A-z0-9_\-]', $this->_options['text_size'], 1),
			'fileName2' => new DAttr('fileName', 'file0', 'File Name', 'text', false, 2, 'r', $this->_options['text_size']),
			'fileName3' => new DAttr('fileName', 'file0', 'File Name', 'text', true, 3, 'r', $this->_options['text_size']),

			'rollingSize' => new DAttr('rollingSize', 'uint', 'Rolling Size (bytes)', 'text', true, NULL, NULL, NULL, NULL, 'log_rollingSize'),
			'keepDays' => new DAttr('keepDays', 'uint', 'Keep Days', 'text', true, 0, NULL, NULL, 0, 'log_keepDays'),
			'logFormat' => new DAttr('logFormat', 'cust', 'Log Format', 'text', true, NULL, NULL, $this->_options['text_size'], 0, 'accessLog_logFormat'),
			'logHeaders' => new DAttr('logHeaders',	'checkboxOr', 'Log Headers', 'checkboxgroup', true, NULL, $this->_options['logHeaders'], NULL, 0, 'accessLog_logHeader'),
			'compressArchive' => new DAttr('compressArchive', 'bool', 'Compress Archive', 'radio', true, NULL, NULL, NULL, 0, 'log_compressArchive'),
			'enableExpires' => new DAttr('enableExpires', 'bool', 'Enable Expires', 'radio'),
			'expiresDefault' => new DAttr('expiresDefault', 'parse', 'Expires Default', 'text', true,
										  "/^[AaMm]\d+$/",
										  '[A|M]###, A means client access time, M means file modified time, ### is number of seconds'),
			'expiresByType' => new DAttr(
				'expiresByType', 'parse', 'Expires By Type', 'textarea', true,
				"/^(\*\/\*)|([A-z0-9_\-\.\+]+\/\*)|([A-z0-9_\-\.\+]+\/[A-z0-9_\-\.\+]+)=[AaMm]\d+$/",
				'MIME_type=A|Mseconds, MIME type can be like */*, text/*, text/html',
				'rows="2" cols="50"', 1),

			'extraHeaders' => new DAttr(
				'extraHeaders', 'cust', 'Header Operations', 'textarea1', true,
				NULL, 'Extra headers will be inserted into HTTP response Header, one header per line, put "NONE" to disable headers inherited from parent content.',
			    'rows="2" cols="60" wrap=off'),


			'realm_type' => new DAttr('type', 'sel', 'DB Type', 'select', false, NULL, $this->_options['realmType'], NULL, 0, 'realmType'),
			'realm_name' => new DAttr('name', 'name', 'Realm Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'realmName'),
			'realm_udb_maxCacheSize' => new DAttr('userDB:maxCacheSize', 'uint', 'User DB Max Cache Size',
												  'text', true, 0, '100K', NULL, 0, 'userDBMaxCacheSize'),
			'realm_udb_cacheTimeout' => new DAttr('userDB:cacheTimeout', 'uint',
												  'User DB Cache Timeout (secs)', 'text', true, 0, 3600, NULL, 0, 'userDBCacheTimeout'),
			'realm_gdb_maxCacheSize' => new DAttr('groupDB:maxCacheSize', 'uint',
												  'Group DB Max Cache Size', 'text', true, 0, '100K', NULL, 0, 'groupDBMaxCacheSize'),
			'realm_gdb_cacheTimeout' => new DAttr('groupDB:cacheTimeout', 'uint',
												  'Group DB Cache Timeout (secs)', 'text', true, 0, 3600, NULL, 0, 'groupDBCacheTimeout'),


			'lb_mode' => new DAttr('mode', 'sel', 'Mode', 'select', false, 0, ['0'=>'Stateless', '1'=>'Stateful'], NULL, 0, 'lb_mode'),
			'lb_strategy' => new DAttr('strategy', 'sel', 'Strategy', 'select', false, 0, ['0'=>'Least Load', '1'=>'Round Robin', '2'=>'Least Session', '3' => 'Fastest Response', '4' => 'Failover'] ),
			'lb_type' => $lb_type,
            'lb_type_file' => $lb_type_file,
			'lb_insertCookie' => new DAttr('insertCookie', 'bool', 'Insert Tracking Cookie', 'radio', true),
			'lb_cookieName' => new DAttr('cookieName', 'cust', 'Tracking Cookie Name', 'text', true, NULL, NULL, $this->_options['text_size']),
            'lb_showBackend' => new DAttr('showBackend', 'bool', 'Show Backend in Header', 'radio', true),
			'lb_forwardIpHeader' => new DAttr('forwardIpHeader', 'parse', 'Forward IP Header', 'text', true,
								 "/^[A-z_][A-z0-9_\-]+$/", 'Allowed character: A-z, 0-9, hypen(-) and underscore(_).', $this->_options['text_size'], 1),
			'lb_forwardByHeader' => new DAttr('forwardByHeader', 'parse', 'Forwarded By Header', 'text', true,
								 "/^[A-z_][A-z0-9_\-]+$/", 'Allowed character: A-z, 0-9, hypen(-) and underscore(_).', $this->_options['text_size'], 1),
            
			'lb_sessionExMethod' => new DAttr('sessionExMethod','checkboxOr', 'Session Extraction', 'checkboxgroup',true, NULL,
				array('1'=>'IP Address', '2'=>'Basic Authentication', '4'=>'Query String', '8'=>'Cookies',
					'16'=>'SSL Session', '32'=>'JVM Route', '64'=>'URL Path Parameter')),

			'lb_sessionId' => new DAttr('sessionId', 'cust', 'Session ID String', 'text', true, NULL, NULL, $this->_options['text_size']),
			'lb_sessionTimeout' => new DAttr('sessionTimeout', 'uint', 'Session Timeout', 'text', true, 0),
			'lb_errMessage' => new DAttr('errMessage', 'cust', 'Custom Error Messages', 'textarea', true, NULL, NULL,  'rows="6" cols="50"'),

			'worker_type' => new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['workerType'], NULL, 0, 'extAppType'),
			'worker_protocol' => new DAttr('protocol','checkboxOr', 'Transport Protocols', 'checkboxgroup',true, NULL,
				array('1'=>'HTTP/1.1', '4' => 'HTTP/2', '2'=>'HTTP/3 | QUIC', ''=>'Not Set'), NULL, 0, 'workerProtocol'),

			'workerGroupEnabled' => new DAttr('workerGroupEnabled', 'bool', 'Enabled', 'radio', false ),
			'sourceIP' => new DAttr('sourceIP', 'sel', 'Source IP', 'select', true, 0, $this->_options['ip']),
			'workergroup_priority' => new DAttr('priority', 'uint', 'Priority', 'text', true, 0, 255, NULL, 0, 'workergroup_priority'),
			'defaultTargetPort' => new DAttr('defaultTargetPort', 'uint', 'Default Target Port', 'text', true, 1, 65535),
			'maxConns' => new DAttr('maxConns', 'uint', 'Max Connections', 'text', false, 0, 2000),
			'pingInterval' => new DAttr('pingInterval', 'uint', 'Ping Interval (secs)', 'text', true, 0),
			'pingSmartFactor' => new DAttr('pingSmartFactor', 'uint', 'Smart Ping Factor', 'text', true, 0, 255),
			'initTimeout' => new DAttr('initTimeout', 'uint', 'Initial Request Timeout (secs)', 'text', false, 1),
			'retryTimeout' => new DAttr('retryTimeout', 'uint', 'Retry Timeout (secs)', 'text', false, 0),
			'persistConn' => new DAttr('persistConn', 'bool', 'Persistent Connection', 'radio', true),
			'pcKeepAliveTimeout' => new DAttr('pcKeepAliveTimeout', 'uint', 'Connection Keepalive Timeout', 'text', true, -1, 10000),
			'respBuffer' => new DAttr('respBuffer', 'sel', 'Response Buffering', 'select', false, 0, $this->_options['respBuffer']),
            'nodeAddr' => $nodeAddr,
			'name'=> new DAttr('name', 'name', 'Name', 'text', false, NULL, NULL, $this->_options['text_size']),

            'l_type' => $l_type,
            'l_type_file' => $l_type_file,
            'l_name' => new DAttr('name', 'name', 'Listener Name', 'text', false, NULL, NULL, NULL, 0, 'listenerName'),
            'l_addr' => $l_addr,
            'l_port' => $l_port,
            'l_ip' => $l_ip,
			'l_binding' => $binding,
            'l_secure' => new DAttr('secure', 'bool', 'Secure', 'radio', false, NULL, NULL, NULL, 0, 'listenerSecure'),
            'l_zconfEnabled' => new DAttr('zconfEnabled', 'bool', 'Enable ZeroConf', 'radio'),
		    'ssl_renegProtection' => new DAttr('renegProtection', 'bool', 'SSL Renegotiation Protection', 'radio'),
			'sslSessionCache' => new DAttr('sslSessionCache', 'bool', 'Enable Session Cache', 'radio'),
            'sslSessionTickets' => new DAttr('sslSessionTickets', 'bool', 'Enable Session Tickets', 'radio'),

            'l_vhost' => new DAttr('vhost', 'sel1', 'Virtual Host', 'select', false, array('VHS'), NULL, NULL, 0, 'virtualHostName' ),
			'l_domain' => new DAttr('domain', 'domain', 'Domains', 'text', false, NULL, NULL, $this->_options['text_size'], 1, 'domainName'),
			'tp_templateFile' => new DAttr('templateFile', 'filetp', 'Template File', 'text', false, 2, 'rwc', $this->_options['text_size']),
			'tp_listeners' => new DAttr('listeners', 'sel2', 'Mapped Listeners', 'text', false, array('LNS'), NULL, $this->_options['text_size'], 1, 'mappedListeners'),

			'tp_vhName' => new DAttr('vhName', 'vhname', 'Virtual Host Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'templateVHName'),
			'tp_vhDomain' => new DAttr('vhDomain', 'domain', 'Domain', 'text', true, NULL, NULL, $this->_options['text_size'], 0, 'templateVHDomain'),

			'tp_vrFile' => new DAttr('fileName', 'parse', 'File Name', 'text', true,
									   $this->_options['tp_vroot'][0], $this->_options['tp_vroot'][1],
										 $this->_options['text_size'], 0, 'templateFileRef'),

			'tp_name' => new DAttr('name', 'parse', 'Name', 'text', false,
								   $this->_options['tp_vname'][0], $this->_options['tp_vname'][1]),

			'defaultCluster' => new DAttr('defaultCluster', 'sel1', 'Default Cluster', 'select', false, array('CLS')),

			'vh_maxKeepAliveReq' => new DAttr('maxKeepAliveReq', 'uint', 'Max Keep-Alive Requests', 'text', true, 0, 32767, NULL, 0, 'vhMaxKeepAliveReq'),
            'vh_enableGzip' => new DAttr('enableGzip', 'bool', 'Enable Compression', 'radio', true, NULL, NULL, NULL, 0, 'vhEnableGzip'),
			'dynReqPerSec' => new DAttr('dynReqPerSec', 'uint', 'Requests/second', 'text', true, 0),
			'outBandwidth' => new DAttr('outBandwidth', 'uint', 'Outbound Bandwidth (bytes/sec)', 'text', true, 0),
			'inBandwidth' => new DAttr('inBandwidth', 'uint', 'Inbound Bandwidth (bytes/sec)', 'text', true, 0),

			'ctx_order' => $ctxOrder,
			'ctx_type' => new DAttr('type', 'sel', 'Type', 'select', false, NULL, $this->_options['ctxType'], NULL, 0, 'ctxType'),
			'ctx_uri' => new DAttr('uri', 'expuri', 'URI', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'expuri'),

			'geoipDBFile' => new DAttr( 'geoipDBFile', 'filep', 'DB File Path', 'text', false, 2, 'r', $this->_options['text_size']),

			'enableIpGeo' => new DAttr( 'enableIpGeo', 'bool', 'Enable GeoLocation Lookup', 'radio'),
			'note' => new DAttr('note', 'note', 'Notes', 'textarea', true, NULL, NULL, 'rows="4" cols="60" wrap=off'),
			'enable_pagespeed' => new DAttr('enabled', 'bool', 'Enable PageSpeed', 'radio', true, NULL, NULL, NULL, 0, 'pagespeedEnabled'),
			'pagespeed_setting' => new DAttr('param', 'cust', 'PageSpeed Settings', 'textarea1', true, NULL, NULL, 'rows="25" cols="60" wrap=off', 0, 'pagespeedParams'),

			);

	}

    public function GetCommonAttr($key)
    {
        if (isset($this->_attrs[$key])) {
            return $this->_attrs[$key];
        } else {
            return null;
        }

    }


	//	DAttr($key, $type, $label,  $inputType, $allowNull,$min, $max, $inputAttr, $multiInd)

	private function add_SERV_PROCESS($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Server Process');

		$workerProcesses = new DAttr('workerProcesses', 'uint', 'Number of Processes', 'text', false, 0, 128);
		$workerProcesses->_version = 4; // XCPU version
		$workerProcesses->_helpKey = 'lslbdProcesses';

		$enableChroot = new DAttr('enableChroot', 'bool', 'Enable chroot', 'radio', false);

		$chrootPath = new DAttr('chrootPath', 'cust', 'chroot Path', 'text', true);
		$chrootPath->_FDE = 'YYN';
		$chrootPath->_helpKey = 'servChrootPath';

		$runningAs = new DAttr('runningAs', 'cust', 'Running As', 'text', false, NULL, NULL, $this->_options['text_size'] . ' readonly');
		$runningAs->_FDE = 'NYN';

		$cpuaffinity = new DAttr('cpuAffinity', 'uint', 'CPU Affinity', 'text', true, -2, 64);
		$cpuaffinity->_FDE = 'YNN';

		$user = new DAttr('user', 'cust', NULL, 'text', false);
		$user->_FDE = 'YNN';
		$group = new DAttr('group', 'cust', NULL, 'text', false);
		$group->_FDE = 'YNN';

		$attrs = array(
			new DAttr('serverName', 'name', 'Server Name', 'text', false),
			$workerProcesses,
			$runningAs, $user, $group,
			new DAttr('priority', 'uint', 'Priority', 'text', true, -20, 20, NULL, 0, 'serverPriority'),
			$cpuaffinity,
			$chrootPath, $enableChroot,
			new DAttr('inMemBufSize', 'uint', 'Memory I/O Buffer', 'text', false, 0, NULL),
			new DAttr('swappingDir', 'cust', 'Swapping Directory', 'text', false, 1, 'rw', $this->_options['text_size']),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general', NULL, 'servProcess');
	}

	private function add_SERV_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'General Settings');

		$attrs = array(
            new DAttr('disableInitLogRotation', 'bool', 'Disable Initial Log Rotation', 'radio', true),
            new DAttr('showVersionNumber', 'sel', 'Server Signature', 'select', false, NULL, array('0'=>'Hide Version', '1'=>'Show Version', '2'=>'Hide Full Header')),
            new DAttr('hideSigInErrorPage', 'bool', 'Hide Error Page Signature', 'radio'),
            $this->_attrs['enableIpGeo'],
            new DAttr('useIpInProxyHeader', 'sel', 'Use Client IP in Header', 'select', true, NULL, array('0'=>'No', '1'=>'Yes', '2'=>'Trusted IP Only', '3' => 'Keep Header from Trusted IP', '4' => 'Use Last IP (for AWS ELB)') ),
			$this->_attrs['extraHeaders'],
            new DAttr('autoUpdateInterval', 'sel', 'Check For Update', 'select', false, NULL, $this->_options['updateInterval'] ),
            new DAttr('autoUpdateDownloadPkg', 'bool', 'Download Updates', 'radio', false),
            new DAttr('adminEmails', 'email', 'Administrator Email', 'textarea', true, NULL, NULL, 'rows="3" cols="60"', 1),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general', NULL, 'servGeneral');
	}

	private function add_SERV_LOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Server Log');

		$dbg_options = ['10'=>'HIGH', '5'=>'MEDIUM', '2'=>'LOW', '0'=>'NONE'];
		$attrs = [
			$this->_attrs['fileName2']->dup(NULL, NULL, 'log_fileName'),
			new DAttr('logLevel', 'sel', 'Log Level', 'select', false, 0, $this->_options['logLevel'], NULL, 0, 'log_logLevel'),
			new DAttr('debugLevel', 'sel', 'Debug Level', 'select', false, 0, $dbg_options, NULL, 0, 'log_debugLevel'),
			$this->_attrs['rollingSize'],
            $this->_attrs['keepDays'],
			$this->_attrs['compressArchive'],
			new DAttr('enableStderrLog', 'bool', 'Enable stderr Log', 'radio', true, NULL, NULL, NULL, 0, 'log_enableStderrLog'),
            new DAttr('enableAioLog', 'bool', 'Enable AIO Logging', 'radio', true, NULL, NULL, NULL, 0, 'log_enableAioLog')
			];
		$this->_tblDef[$id]->setAttr($attrs, 'general:log', 'logging:log', 'servLog');
	}

	private function add_SERV_ACLOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Access Log');
		$aclog_fileName = $this->_attrs['fileName2'];
		$aclog_fileName->_helpKey = 'accessLog_fileName';
		$attrs = array(
			$aclog_fileName,
			new DAttr('pipedLogger', 'cust', 'Piped Logger', 'text', true,
					NULL, NULL, $this->_options['text_size'], 0, 'accessLog_pipedLogger'),
			$this->_attrs['logFormat'],
			$this->_attrs['logHeaders'],
			$this->_attrs['rollingSize'],
			$this->_attrs['keepDays'],
			$this->_attrs['compressArchive'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general:aclog', 'logging:accessLog', 'servAcLog');
	}

	private function add_ZCONF_LOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf Listener Log');

		$dbg_options = ['10'=>'HIGH', '5'=>'MEDIUM', '2'=>'LOW', '0'=>'NONE'];
		$attrs = [
			$this->_attrs['fileName2']->dup(NULL, NULL, 'log_fileName'),
			new DAttr('logLevel', 'sel', 'Log Level', 'select', false, 0, $this->_options['logLevel'], NULL, 0, 'log_logLevel'),
			new DAttr('debugLevel', 'sel', 'Debug Level', 'select', false, 0, $dbg_options, NULL, 0, 'log_debugLevel'),
			$this->_attrs['rollingSize'],
            $this->_attrs['keepDays'],
			$this->_attrs['compressArchive'],
			];
		$this->_tblDef[$id]->setAttr($attrs, 'zconf:log', 'zeroConfig:logging:log', 'zconfLog');
	}

	private function add_ZCONF_ACLOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf Access Log');
		$aclog_fileName = $this->_attrs['fileName2'];
		$aclog_fileName->_helpKey = 'accessLog_fileName';
		$attrs = array(
			$aclog_fileName,
			new DAttr('pipedLogger', 'cust', 'Piped Logger', 'text', true,
					NULL, NULL, $this->_options['text_size'], 0, 'accessLog_pipedLogger'),
			$this->_attrs['logFormat'],
			$this->_attrs['logHeaders'],
			$this->_attrs['rollingSize'],
			$this->_attrs['keepDays'],
			$this->_attrs['compressArchive']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'zconf:aclog', 'zeroConfig:logging:accessLog', 'zconfAcLog');
	}


	private function add_SERV_TUNING_OS($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'General');
		$secureck_options = array( '1'=>'secure', '2'=>'SameSite=Strict', '4'=>'SameSite=Lax', '8'=>'SameSite=None', ''=>'Not Set');


		$attrs = array(
			new DAttr('ramDiskPath', 'path', 'RAM Disk Path', 'text', true, 1, 'rw', $this->_options['text_size']),
            new DAttr('shmDefaultDir', 'cust', 'Default SHM Directory', 'text', true, NULL, '', $this->_options['text_size']),
            new DAttr('bindIPv6Only', 'bool', 'Bind IPv6 Only', 'radio'),
            new DAttr('reverseDnsLookup', 'bool', 'Reverse DNS Lookup', 'radio'),
            new DAttr('verifyGoogleBot', 'bool', 'Verify Google Bot', 'radio'),
            new DAttr('forceSecureCookie', 'checkboxOr', 'Secure Cookie Control', 'checkboxgroup', true, '', $secureck_options),

			);

		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning');
	}

	private function add_SERV_TUNING_CONN($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Connection');

		$attrs = array(
			new DAttr('maxConnections', 'uint', 'Max Connections', 'text', false, 1),
			new DAttr('maxSSLConnections', 'uint', 'Max SSL Connections', 'text', false, 0),
			new DAttr('connTimeout', 'uint', 'Connection Timeout (secs)', 'text', false, 10, 1000000),
			new DAttr('maxKeepAliveReq', 'uint', 'Max Keep-Alive Requests', 'text', false, 0, 32767),
			new DAttr('keepAliveTimeout', 'uint', 'Keep-Alive Timeout (secs)', 'text', false, 0, 60),
			new DAttr('sndBufSize', 'uint', 'Send Buffer Size (bytes)', 'text', true, 0, '512K'),
			new DAttr('rcvBufSize', 'uint', 'Receive Buffer Size (bytes)', 'text', true, 0, '512K')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning', 'connTuning');
	}

	private function add_SERV_TUNING_REQ($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Request/Response');

		$attrs = array(
            new DAttr('maxReqURLLen', 'uint', 'Max Request URL Length (bytes)', 'text', false, 200, 65530),
			new DAttr('maxReqHeaderSize', 'uint', 'Max Request Header Size (bytes)', 'text', false, 1024, 65530),
			new DAttr('maxReqBodySize', 'uint', 'Max Request Body Size (bytes)', 'text', false, '1M', NULL),
			new DAttr('maxDynRespHeaderSize', 'uint', 'Max Response Header Size (bytes)', 'text', false, 200, '64K'),
			new DAttr('maxDynRespSize', 'uint', 'Max Response Body Size (bytes)', 'text', false, '1M')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning', 'reqTuning');
	}

	private function add_SERV_TUNING_GZIP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'GZIP Compression');

		$parseFormat = "/^(\!)?(\*\/\*)|([A-z0-9_\-\.\+]+\/\*)|([A-z0-9_\-\.\+]+\/[A-z0-9_\-\.\+]+)|default$/";
		$parseHelp = 'default, (!)MIME types, like */*, text/*, text/html, !text/css';

		$attrs = array(
			new DAttr('enableGzipCompress', 'bool', 'Enable Compression', 'radio', false),
			new DAttr('compressibleTypes', 'parse', 'Compressible Types', 'textarea', true, $parseFormat, $parseHelp, 'rows="5" cols="50"', 1),
			new DAttr('gzipCompressLevel', 'uint', 'GZIP Compression Level', 'text', true, 1, 9),
			new DAttr('enableBrCompress', 'uint', 'Brotli Compression Level', 'text', true, 0, 6),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning', 'gzipTuning');
	}

	private function add_SERV_TUNING_SSL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'SSL Global Settings');
		$parseFormat = '/^((http|https):\/\/)?([A-z0-9._\-]+|\[[[:xdigit:]:]+\])(:\d+)$/';
		$parseHelp = '"[http|https://]IPV4|IPV6_address:port"';

		$attrs = [
            new DAttr('sslDefaultCiphers', 'cust', 'Default Cipher Suite', 'text', true, NULL, '', $this->_options['text_size']),
	        new DAttr('sslStrongDhKey', 'bool', 'SSL Strong DH Key', 'radio', true),
            new DAttr('sslEnableMultiCerts', 'bool', 'Enable Multiple SSL Certificates', 'radio'),
            $this->_attrs['sslSessionCache'],
			new DAttr('sslSessionCacheSize', 'uint', 'Session Cache Size (bytes)', 'text', true, 512),
            new DAttr('sslSessionCacheTimeout', 'uint', 'Session Cache Timeout (secs)', 'text', true, 10, 1000000),
            $this->_attrs['sslSessionTickets'],
            new DAttr('sslSessionTicketLifetime', 'uint', 'SSL Session Ticket Lifetime (secs)', 'text', true, 10, 1000000),
            new DAttr('sslSessionTicketKeyFile', 'cust', 'SSL Session Ticket Key File', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('sslOcspProxy', 'parse', 'OCSP Proxy', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size']),
			new DAttr('sslStrictSni', 'bool', 'Strict SNI Certificate', 'radio', true),
			];
		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning', 'sslGlobal');
	}

	private function add_SERV_TUNING_SSLSNI($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'SSL Dynamic SNI');

		$attrs = array(
			// dyn cert is lslb only
            new DAttr('sslDynCertsCiphers', 'cust', 'Certificates Cipher Suite', 'text', true, NULL, '', $this->_options['text_size']),
            new DAttr('sslDynCertsBaseDir', 'cust', 'Certificates Base Dir', 'text', true, NULL, '', $this->_options['text_size']),
            new DAttr('sslDynCertsNumLevels', 'uint', 'Certificates Levels', 'text', true, 0, 4),
            new DAttr('sslDynCertsExpire', 'uint', 'Certificates Expire Time (secs)', 'text', true, 10, 1000000),
            new DAttr('sslDynCertsRefresh', 'uint', 'Certificates Refresh Time (secs)', 'text', true, 10, 1000000), // check refresh <= expire
			);
		$this->_tblDef[$id]->setAttr($attrs, 'tuning', 'tuning', 'sslDynSNI');
	}

	private function add_SERV_QUIC($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'HTTP3/QUIC');

		$parseFormat = "/^\d+\s*\/\s*\d+$/";
		$parseHelp = 'X/Y where X is the number of events, Y is the interval in seconds (0-30).';
        $congest_options = ['' => 'Default', '1' => 'Cubic', '2' => 'BBR'];

		$attrs = [
            new DAttr('quicEnable', 'bool', 'Enable HTTP3/QUIC', 'radio'),
            new DAttr('quicShmDir', 'cust', 'QUIC SHM Directory', 'text', true, NULL, '', $this->_options['text_size']),
            new DAttr('quicVersions', 'cust', 'HTTP3/QUIC Versions', 'text', true, NULL, '', $this->_options['text_size']),
            new DAttr('quicCongestionCtrl', 'sel', 'Congestion Control', 'select', true, NULL, $congest_options),
            new DAttr('quicCfcw', 'uint', 'Connection Flow Control Window', 'text', true, '64K', '512M'),
            new DAttr('quicMaxCfcw', 'uint', 'Max Connection Flow Control Window', 'text', true, '64K', '512M'),
            new DAttr('quicSfcw', 'uint', 'Stream Flow Control Window', 'text', true, '64K', '128M'),
            new DAttr('quicMaxSfcw', 'uint', 'Max Stream Flow Control Window', 'text', true, '64K', '128M'),
			new DAttr('quicInitMaxIncomingStreamsBidi', 'uint', 'HTTP/3 Initial Incoming Streams', 'text', true, 10, 1000),
            new DAttr('quicMaxStreams', 'uint', 'Max Concurrent Streams Per Connection', 'text', true, 10, 1000),
            new DAttr('quicHandshakeTimeout', 'uint', 'Handshake Timeout (secs)', 'text', true, 1, 15),
            new DAttr('quicIdleTimeout', 'uint', 'Idle Timeout (secs)', 'text', true, 10, 30),
            new DAttr('quicDosMitigationDuration', 'uint', 'QUIC DoS Mitigation Duration (secs)', 'text', true, 0, 86400),
            new DAttr('quicBadPacketRate', 'parse', 'Bad Packet Rate', 'text', true, $parseFormat, $parseHelp),
            new DAttr('quicHskNewRate', 'parse', 'New Handshake Rate', 'text', true, $parseFormat, $parseHelp),
            new DAttr('quicHskTimeoutRate', 'parse', 'Timed Out Handshake Rate', 'text', true, $parseFormat, $parseHelp),
            new DAttr('quicEnableDPLPMTUD', 'bool', 'Enable DPLPMTUD', 'radio'),
            new DAttr('quicBasePLPMTU', 'uint', 'PLPMTU Base Value', 'text', true, 0, 65527), //0 default, 1200-65527
            new DAttr('quicMaxPLPMTU', 'uint', 'PLPMTU Max Value', 'text', true, 0, 65527), //0 default, 1200-65527
        ];
		$this->_tblDef[$id]->setAttr($attrs, 'quic', 'quic');
	}

    private function add_SERV_SEC_CONN($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Per Client Throttling');

		$attrs = array(
			$this->_attrs['dynReqPerSec'],
			$this->_attrs['outBandwidth'],
			$this->_attrs['inBandwidth'],
			new DAttr('softLimit', 'uint', 'Connection Soft Limit',  'text', true, 0),
			new DAttr('hardLimit', 'uint', 'Connection Hard Limit', 'text', true, 0),
			new DAttr('blockBadReq', 'bool', 'Block Bad Request', 'radio', true),
			new DAttr('gracePeriod', 'uint', 'Grace Period (sec)', 'text', true, 1, 3600),
			new DAttr('banPeriod', 'uint', 'Banned Period (sec)', 'text', true, 0)

			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:conn', 'security:perClientConnLimit', 'perClientConnLimit');
	}

	function add_LB_TOP($id)
	{
		$align = array( 'left', 'center', 'center', 'center' ) ;
        $this->_tblDef[$id] = new DTbl($id, 'Clusters List', 1, 'LB_SEL', $align, "network") ;
        $attrs = array(
			$this->_attrs['name']->dup(NULL, NULL, 'lb_name'),
            $this->_attrs['lb_type'],
            $this->_attrs['lb_mode'],
            $this->_attrs['lb_strategy'],
            new DAttr('action', 'action', 'Action', NULL, false, 'LB_GENERAL_DEL', 'Xd') ) ;
        $this->_tblDef[$id]->setAttr($attrs, 'clusters') ;
        $this->_tblDef[$id]->setRepeated('name') ;
    }

	function add_LB_SEL_FILE($id)
	{
        // for file
		$this->_tblDef[$id] = new DTbl($id, 'Cluster');
		$this->_tblDef[$id]->_subTbls = $this->_options['clusterTbl'];

		$attrs = array($this->_attrs['lb_type']);
		$this->_tblDef[$id]->setAttr($attrs);
	}

	function add_LB_SEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Cluster - Select Type');
		$this->_tblDef[$id]->_subTbls = $this->_options['clusterTblNew'];

		$attrs = array($this->_attrs['lb_type']);
		$this->_tblDef[$id]->setAttr($attrs);
		$this->_tblDef[$id]->setRepeated('name');
	}


	function add_LB_GENERAL_NEW($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Layer 7 Cluster', 2);

		$attrs = array(
			$this->_attrs['name']->dup(NULL, 'Cluster Name', 'lb_name'),
			$this->_attrs['lb_mode'],
			$this->_attrs['lb_strategy'],
			$this->_attrs['lb_sessionExMethod'],
			$this->_attrs['lb_sessionId'],
			$this->_attrs['lb_sessionTimeout'],
			$this->_attrs['lb_insertCookie'],
			$this->_attrs['lb_cookieName'],
            $this->_attrs['lb_showBackend'],
            $this->_attrs['lb_forwardIpHeader'],
            $this->_attrs['lb_forwardByHeader'],
			$this->_attrs['lb_errMessage'],
			$this->_attrs['note']);
		$this->_tblDef[$id]->setAttr($attrs, 'clusters', 'loadBalancerList:loadBalancer');
		$this->_tblDef[$id]->setRepeated('name');
        $this->_tblDef[$id]->_defaultExtract = array('type'=>'layer7');
	}

	function add_LB4_GENERAL_NEW($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Layer 4 Cluster', 2);

		$attrs = array(
			$this->_attrs['name']->dup(NULL, 'Cluster Name', 'lb_name'),
			$this->_attrs['lb_mode'],
			$this->_attrs['lb_strategy'],
			$this->_attrs['lb_sessionTimeout'],
			$this->_attrs['note']);
		$this->_tblDef[$id]->setAttr($attrs, 'clusters', 'loadBalancerList:loadBalancer');
		$this->_tblDef[$id]->setRepeated('name');
        $this->_tblDef[$id]->_defaultExtract = array('type'=>'layer4');
	}

	function add_LB_GENERAL_DEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Cluster Settings', 2);

		$attrs = array(
			$this->_attrs['name']->dup(NULL, 'Cluster Name', 'lb_name'),
            $this->_attrs['lb_type'],
			$this->_attrs['lb_mode'],
			$this->_attrs['lb_strategy'],
			$this->_attrs['lb_sessionTimeout'],
			$this->_attrs['note']);
		$this->_tblDef[$id]->setAttr($attrs, 'clusters', 'loadBalancerList:loadBalancer');
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_LB_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Load Balancer Definition', 2);

		$attrs = array(
			$this->_attrs['name']->dup(NULL, NULL, 'lb_name'),
			$this->_attrs['lb_type_file'],
			$this->_attrs['lb_mode'],
			$this->_attrs['lb_strategy'],
			$this->_attrs['note']);
		$this->_tblDef[$id]->setAttr($attrs);
        $this->_tblDef[$id]->_defaultExtract = array('type'=>'layer7');
	}

	function add_LB4_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Load Balancer Definition', 2);

		$attrs = array(
			$this->_attrs['name']->dup(NULL, NULL, 'lb_name'),
			$this->_attrs['lb_type_file'],
			$this->_attrs['lb_mode'],
			$this->_attrs['lb_strategy'],
			$this->_attrs['note']);
		$this->_tblDef[$id]->setAttr($attrs);
        $this->_tblDef[$id]->_defaultExtract = array('type'=>'layer4');
	}

	private function add_LB_SESSION($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Session Management', 2);

		$attrs = array($this->_attrs['lb_sessionExMethod'],
			$this->_attrs['lb_sessionId'],
			$this->_attrs['lb_sessionTimeout'],
			$this->_attrs['lb_insertCookie'],
			$this->_attrs['lb_cookieName'],
            $this->_attrs['lb_showBackend'],
            $this->_attrs['lb_forwardIpHeader'],
            $this->_attrs['lb_forwardByHeader'],
        );
		$this->_tblDef[$id]->setAttr($attrs);
	}

	private function add_LB_ERRMSG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Custom Error Messages', 2);

		$attrs = array(
				new DAttr('errMessage', 'cust', null, 'textarea', true, NULL, NULL,  'rows="15" cols="70"')
		);
		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'ClusterErrMsg');
	}

	function add_LB_WORKERGRP_TOP($id)
	{
		$align = array('left', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Layer 7 Worker Groups', 1, 'LB_WORKERGRP', $align, 'application');

		$attrs = [
			new DAttr('name', 'cust', 'Name'),
			$this->_attrs['worker_type'],
			$this->_attrs['workerGroupEnabled'],
			$this->_attrs['sourceIP'],
			$this->_attrs['workergroup_priority'],
			$this->_attrs['defaultTargetPort'],
			$this->_attrs['maxConns'],
			new DAttr('action', 'action', 'Action', NULL, false, 'LB_WORKERGRP', 'vEd'),
		];
		$this->_tblDef[$id]->setAttr($attrs, 'workers', 'workerGroupList:workerGroup');
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_LB_WORKERGRP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Layer 7 Worker Group', 2);

		$attrs = [
			$this->_attrs['name']->dup(NULL, NULL, 'workerName'),
			$this->_attrs['worker_type'],
			$this->_attrs['worker_protocol'],
			$this->_attrs['workerGroupEnabled'],
			$this->_attrs['sourceIP'],
			$this->_attrs['workergroup_priority'],
			$this->_attrs['defaultTargetPort'],
			$this->_attrs['maxConns'],
			new DAttr('env', 'parse', 'Environment', 'textarea', true, "/\S+=\S+/", 'PROPERTY=VALUE', 'rows="5" cols="50" wrap=off', 2),
			$this->_attrs['initTimeout'], 
			$this->_attrs['retryTimeout'], 
			$this->_attrs['persistConn'], 
			$this->_attrs['pcKeepAliveTimeout'], 
			$this->_attrs['respBuffer'],
			new DAttr('pingUrl', 'url', 'Ping URL', 'text', true, NULL, NULL, $this->_options['text_size']),
			$this->_attrs['pingInterval'],
			$this->_attrs['pingSmartFactor'],
			$this->_attrs['nodeAddr'],
		];
		$this->_tblDef[$id]->setAttr($attrs, 'workers', 'workerGroupList:workerGroup', 'workerGroup');
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_LB4_WORKERGRP_TOP($id)
	{
		$align = ['left', 'center', 'center'];
		$this->_tblDef[$id] = new DTbl($id, 'Layer 4 Worker Groups', 1, 'LB4_WORKERGRP', $align, 'application');

		$attrs = [
			new DAttr('name', 'cust', 'Name'),
			$this->_attrs['workerGroupEnabled'],
			$this->_attrs['sourceIP'],
			$this->_attrs['workergroup_priority'],
			$this->_attrs['defaultTargetPort'],
			$this->_attrs['maxConns'],
			new DAttr('action', 'action', 'Action', NULL, false, 'LB4_WORKERGRP', 'vEd')
		];
		$this->_tblDef[$id]->setAttr($attrs, 'workers', 'workerGroupList:workerGroup');
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_LB4_WORKERGRP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Layer 4 Worker Group', 2);

		$attrs = [
			$this->_attrs['name']->dup(NULL, NULL, 'workerName'),
			$this->_attrs['workerGroupEnabled'],
			$this->_attrs['sourceIP'],
			$this->_attrs['workergroup_priority'],
			$this->_attrs['defaultTargetPort'],
			$this->_attrs['maxConns'],
			$this->_attrs['initTimeout'],
			$this->_attrs['retryTimeout'],
			$this->_attrs['pingInterval'],
			$this->_attrs['pingSmartFactor'],
			$this->_attrs['nodeAddr'],
		];
		$this->_tblDef[$id]->setAttr($attrs, 'workers', 'workerGroupList:workerGroup');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_A_SECAC($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Access Control');

		$attrs = array(
			new DAttr('allow', 'subnet', 'Allowed List', 'textarea', true, NULL, NULL, 'rows="5" cols="60"',1, 'accessControl_allow'),
			new DAttr('deny', 'subnet', 'Denied List', 'textarea', true, NULL, NULL, 'rows="5" cols="60"', 1, 'accessControl_deny'));
		$this->_tblDef[$id]->setAttr($attrs, 'sec:ac', 'security:accessControl', 'accessControl');
		$this->_tblDef[$id]->_cols = 1;
	}

    private function add_SERV_SEC_WP($id)
    {
        $this->_tblDef[$id] = new DTbl($id, 'WordPress Brute Force Attack Protection');
        $wp_options = array('' => '', '2' => 'Throttle', '3' => 'Deny', '4' => 'Drop', '5' => 'CAPTCHA or Drop',
               '6' => 'WP Login CAPTCHA Full Protection', '0' => 'Disable');

        $attrs = array(
            new DAttr('wpProtectAction', 'sel', 'Protection Mode', 'select', true, 0, $wp_options),
            new DAttr('wpProtectLimit', 'uint', 'Allowed Login Attempts', 'text', true, 1, 1000, NULL, 0),
			new DAttr('trustJetpack', 'bool', 'Trust Jetpack', 'radio', true),
        );
        $this->_tblDef[$id]->setAttr($attrs, 'sec:wp', 'security');
    }

    private function add_VH_SEC_WP($id)
    {
        $this->_tblDef[$id] = new DTbl($id, 'WordPress Brute Force Attacks Protection');
		$wp_options = array('' => '', '1' => 'Use Server Level Setting', '2' => 'Throttle', '3' => 'Deny', '4' => 'Drop', 
            '5' => 'Captcha or Drop', '6' => 'WP Login CAPTCHA Full Protection', '0' => 'Disable');

        $attrs = array(
            new DAttr('wpProtectAction', 'sel', 'Protection Mode', 'select', true, 0, $wp_options),
            new DAttr('wpProtectLimit', 'uint', 'Allowed Login Attempts', 'text', true, 5, 1000, NULL, 0),
			new DAttr('trustJetpack', 'bool', 'Trust Jetpack', 'radio', true),
        );
        $this->_tblDef[$id]->setAttr($attrs, 'sec:wp', 'security');
    }

    private function add_SERV_SEC_DOS($id)
    {
        $this->_tblDef[$id] = new DTbl($id, 'Anti-DDoS Protection');

        $attrs = array(
            new DAttr('enableAntiddos', 'bool', 'Enable Anti-DDoS Protection', 'radio'),
            new DAttr('firewallEnable', 'bool', 'Enable Firewall Modifications', 'radio'),
        );
        $this->_tblDef[$id]->setAttr($attrs, 'sec:dos', 'security');
    }

	private function add_SERV_SEC_CC($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Web Application Firewall (WAF)');

		$parseFormat = "/^0?[0-7]{3}$/";
		$parseHelp = '3 digits octet number, Default value is 666';
		$auditlog_options = ['' => '', '0' => 'Disable', '1' => 'Native Audit Log', '2' => 'Json Audit Log', '3' => 'Pretty Json Audit Log'];

		$attrs = array(
			new DAttr('enableCensorship', 'bool', 'Enable WAF', 'radio'),
			new DAttr('logLevel', 'uint', 'Log Level', 'text', true, 0, 9, NULL, 0, 'censorLogLevel'),
			new DAttr('defaultAction', 'cust', 'Default Action', 'text', true, NULL, NULL, $this->_options['text_size']),
			new DAttr('scanPOST', 'bool', 'Scan Request Body', 'radio'),
			new DAttr('uploadTmpDir', 'path', 'Temporary File Path', 'text', false, 2, 'r', $this->_options['text_size']),
			new DAttr('uploadTmpFilePermission', 'parse', 'Temporary File Permission', 'text', true, $parseFormat, $parseHelp),
			new DAttr('secAuditLogEngine', 'sel', 'Enable Security Audit Log', 'select', true, 0, $auditlog_options),
			new DAttr('secAuditLog', 'file0', 'Security Audit Log', 'text', false, 2, 'r', $this->_options['text_size']),
			new DAttr('useRe2', 'bool', 'Use RE2 regex engine', 'radio'));

		$this->_tblDef[$id]->setAttr($attrs, 'sec:cc', 'security:censorshipControl' );
	}

	private function add_VH_SEC_CC($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Web Application Firewall (WAF)');

		$attrs = array(
			new DAttr('enableCensorship', 'bool', 'Enable WAF', 'radio'),
			new DAttr('logLevel', 'uint', 'Log Level', 'text', true, 0, 9, NULL, 0, 'censorLogLevel'),
			new DAttr('defaultAction', 'cust', 'Default Action', 'text', true, NULL, NULL, $this->_options['text_size']),
			new DAttr('scanPOST', 'bool', 'Scan Request Body', 'radio'));

		$this->_tblDef[$id]->setAttr($attrs, 'sec:cc', 'security:censorshipControl' );
	}

	private function add_A_SEC_CR_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'WAF Rule Set', 1, 'A_SEC_CR', $align, "filter");

		$attrs = array(
			new DAttr('name', 'cust', 'Name', NULL, true, NULL, NULL, NULL, 0, 'censorRuleSetName'),
			new DAttr('ruleSetAction', 'cust', 'Action'),
			new DAttr('enabled', 'bool', 'Enabled', 'radio', false, NULL, NULL, NULL, 0, 'censorRuleSetEnabled'),
			new DAttr('action', 'action', 'Action', NULL, false, 'A_SEC_CR', 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:cr', 'security:censorshipRuleSet');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_A_SEC_CR($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'WAF Rule Set');

		$attrs = array(
			new DAttr('name', 'cust', 'Name', 'text', false, null, null, $this->_options['text_size'], 0, 'censorRuleSetName'),
			new DAttr('ruleSetAction', 'cust', 'Action', 'text', true, null, null, $this->_options['text_size']),
			new DAttr('enabled', 'bool', 'Enabled', 'radio', false, NULL, NULL, NULL, 0, 'censorRuleSetEnabled'),
			new DAttr('ruleSet', 'cust', 'Rules Definition', 'textarea1', true, NULL, NULL, 'rows="15" cols="60" wrap=off', 0, 'censorRuleSet'),
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:cr', 'security:censorshipRuleSet');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_SERV_SEC_RECAP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'reCAPTCHA Protection');

		$parseFormat = "/^[A-z0-9\-_]{20,100}$/";
		$parseHelp = 'key string assigned from your google reCAPTCHA account';

		$attrs = array(
			new DAttr('enabled', 'bool', 'Enable reCAPTCHA', 'radio', true, NULL, NULL, NULL, 0, 'enableRecaptcha'),
			new DAttr('siteKey', 'parse', 'Site Key', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'recaptchaSiteKey'),
			new DAttr('secretKey', 'parse', 'Secret Key', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'recaptchaSecretKey'),
            new DAttr('type', 'sel', 'reCAPTCHA Type', 'select', true, 0, $this->_options['recaptcha'], NULL, 0, 'recaptchaType'),
			new DAttr('maxTries', 'uint', 'Max Tries', 'text', true, 0, 65535, NULL, 0, 'recaptchaMaxTries'),
			new DAttr('verifyExpires', 'uint', 'Verification Expires (secs)', 'text', true, 30, 31536000, 'recaptchaVerifyExpires'), // 30 secs to 1 yer
            new DAttr('allowedRobotHits', 'uint', 'Allowed Robot Hits', 'text', true, 0, 65535, NULL, 0, 'recaptchaAllowedRobotHits'),
            new DAttr('botWhiteList', 'cust', 'Bot White List', 'textarea1', true, NULL, NULL, 'rows="5" cols="60" wrap=off', 0, 'recaptchaBotWhiteList'),
            new DAttr('regConnLimit', 'uint', 'Connection Limit', 'text', true, 0, NULL, NULL, 0, 'recaptchaRegConnLimit'),
            new DAttr('sslConnLimit', 'uint', 'SSL Connection Limit', 'text', true, 0, NULL, NULL, 0, 'recaptchaSslConnLimit'),
        );
		$this->_tblDef[$id]->setAttr($attrs, 'lsrecaptcha', 'lsrecaptcha', 'lsrecaptcha');
	}

	private function add_VH_SEC_RECAP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'reCAPTCHA Protection');

		$parseFormat = "/^[A-z0-9\-_]{20,100}$/";
		$parseHelp = 'key string assigned from your google reCAPTCHA account';

		$attrs = array(
			new DAttr('enabled', 'bool', 'Enable reCAPTCHA', 'radio', true, NULL, NULL, NULL, 0, 'enableRecaptcha'),
			new DAttr('siteKey', 'parse', 'Site Key', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'recaptchaSiteKey'),
			new DAttr('secretKey', 'parse', 'Secret Key', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'recaptchaSecretKey'),
            new DAttr('type', 'sel', 'reCAPTCHA Type', 'select', true, 0, $this->_options['recaptcha'], NULL, 0, 'recaptchaType'),
			new DAttr('maxTries', 'uint', 'Max Tries', 'text', true, 0, 65535, NULL, 0, 'recaptchaMaxTries'),
			new DAttr('verifyExpires', 'uint', 'Verification Expires (secs)', 'text', false, 30, 31536000, 'recaptchaVerifyExpires'), // 30 secs to 1 yer
            new DAttr('regConnLimit', 'uint', 'Concurrent Request Limit', 'text', true, 0, NULL, NULL, 0, 'recaptchaVhReqLimit'),
        );
		$this->_tblDef[$id]->setAttr($attrs, 'lsrecaptcha', 'lsrecaptcha', 'lsrecaptcha');
	}

	private function add_SERV_CACHE_POLICY($id)
	{
	    $this->_tblDef[$id] = new DTbl($id, 'Cache Policy');
	    $attrs = $this->get_ctx_attrs('serv_cache');
	    $this->_tblDef[$id]->setAttr($attrs, 'cache', 'cache' );
	}

	private function add_VH_CACHE_POLICY($id)
	{
	    $this->_tblDef[$id] = new DTbl($id, 'Cache Policy');
	    $attrs = $this->get_ctx_attrs('cache');
	    $this->_tblDef[$id]->setAttr($attrs, 'cache', 'cache' );
	}

	private function add_A_CACHE_NCURL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Do-Not-Cache URL');
		$ncurl = new DAttr('noCacheUrl', 'cust', NULL, 'textarea1', true, NULL, NULL, 'rows="10" cols="60" wrap=off', 0, 'noCacheUrl');
		$attrs = array($ncurl);
		$this->_tblDef[$id]->setAttr($attrs, 'cache', 'cache');
		$this->_tblDef[$id]->_cols = 1;
	}

	private function add_SERV_CACHE_STORAGE($id)
	{
	    $this->_tblDef[$id] = new DTbl($id, 'Cache Storage Settings');
	    $cacheStorePath = new DAttr('cacheStorePath', 'cust', 'Storage Path', 'text', true, NULL, NULL, $this->_options['text_size']);
		$cacheMgrPath = new DAttr('cacheMgrStorePath', 'cust', 'Cache Manager Path', 'text', true, NULL, NULL, $this->_options['text_size']);
		$storeageExpires = new DAttr('pubStoreExpireMinutes', 'uint', 'Public Storage Expire (minutes)', 'text', true, 60);
        $purgeNoHitTimeout = new DAttr('purgeNoHitTimeout', 'uint', 'No Hit Expire (minutes)', 'text', true, 0);
	    $attrs = [$cacheStorePath, $cacheMgrPath, $storeageExpires, $purgeNoHitTimeout];

	    $this->_tblDef[$id]->setAttr($attrs, 'cache:storage', 'cache:storage' );
	}

	private function add_VH_CACHE_STORAGE($id)
	{
	    $this->_tblDef[$id] = new DTbl($id, 'Cache Storage Settings');
	    $cacheStorePath = new DAttr('cacheStorePath', 'cust', 'Storage Path', 'text', true, NULL, NULL, $this->_options['text_size']);
		$litemage_options = array(0=>'Off', 1=>'On', 2=>'Safe');
		$litemage = new DAttr('litemage', 'sel', 'Enable LiteMage', 'select', true, 0, $litemage_options);
		$litemage->_feature = 10; // 1010 bit for litemage
	    $attrs = array(	$cacheStorePath, $litemage );

	    $this->_tblDef[$id]->setAttr($attrs, 'cache:storage', 'cache:storage' );
	}

	private function add_SERV_CACHE_NCDOMAIN($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Do-Not-Cache Domains');
		$ncdomain = new DAttr('noCacheDomain', 'cust', NULL, 'textarea1', true, NULL, NULL, 'rows="10" cols="60" wrap=off', 0, 'noCacheDomain');
		$attrs = array($ncdomain);
		$this->_tblDef[$id]->setAttr($attrs, 'cache', 'cache');
		$this->_tblDef[$id]->_cols = 1;
	}

	private function add_SERV_PAGESPEED($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'PageSpeed Module Settings');

		$attrs = array(
			new DAttr('activate', 'bool', 'Load Module', 'radio', true, NULL, NULL, NULL, 0, 'pagespeedActivate'),
   			$this->_attrs['enable_pagespeed'],
			$this->_attrs['pagespeed_setting']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'modpagespeed', 'modpagespeed');
	}

	private function add_VH_PAGESPEED($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'PageSpeed Module Settings');

        $attrs = array(
			$this->_attrs['enable_pagespeed'],
			$this->_attrs['pagespeed_setting']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'modpagespeed', 'modpagespeed');
	}

	private function add_A_GEOIP_TOP($id)
	{
		$align = array('center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'MaxMind GeoIP DB', 1, 'A_GEOIP', $align, "database");

		$attrs = array(
			$this->_attrs['geoipDBFile'],
            new DAttr('geoipDBName', 'cust', 'DB Name'),
			new DAttr('action', 'action', 'Action', NULL, false, 'A_GEOIP', 'Ed'));
		$this->_tblDef[$id]->setAttr($attrs, 'geoipDB', NULL, 'geolocationDB');
		$this->_tblDef[$id]->setRepeated('geoipDBFile');
	}

	private function add_A_GEOIP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'MaxMind GeoIP DB configuration', 2);

		$attrs = array(
			$this->_attrs['geoipDBFile'],
			new DAttr('geoipDBName', 'dbname', 'DB Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0),
			new DAttr('maxMindDBEnv', 'parse', 'Environment Variables', 'textarea', true,
								   "/^\S+[ \t]+\S+$/", 'Variable_Name mapped_DB_data', 'rows="5" cols="50" wrap=off', 2),
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'geoipDB', 'ipToGeo:geoipDB', 'geolocationDB');
		$this->_tblDef[$id]->setRepeated('geoipDBFile');
	}

	private function add_SERV_IP2LOCATION($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'IP2Location DB');

		$attrs = array(
				new DAttr( 'ip2locDBFile', 'filep', 'IP2Location DB File Path', 'text', true, 2, 'r', $this->_options['text_size']),
                new DAttr( 'ip2locDBCache', 'sel',
							'DB Cache Type', 'select', true, 0,
							array( ''=>'', 'FileIo'=>'File System',
								   'MemoryCache' => 'Memory',
								   'SharedMemoryCache' => 'Shared Memory') ),
		);
		$this->_tblDef[$id]->setAttr($attrs, 'ip2locDB', 'ip2locDB');
	}

	private function add_VH_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host List', 1, 'VH_TOP_D', $align, "web");

		$attrs = array(
			new DAttr('name', 'cust', 'Name'),
			new DAttr('defaultCluster', 'cust', 'Default Cluster'),
			new DAttr('action', 'action', 'Action', NULL, false, 'VH_TOP_D', 'Xd'));
		$this->_tblDef[$id]->setAttr($attrs, 'vhTop');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_TOP_D($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Hosts', 2);

		$attrs = array(
			new DAttr('name', 'vhname', 'Virtual Host Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'vhName'),
			$this->_attrs['defaultCluster'],
			new DAttr('configFile', 'file', 'Configure File', 'text', false, 3, 'rwc', $this->_options['text_size']),
			$this->_attrs['note']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'vhTop', 'virtualHostList:virtualHost', 'vhTop');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_BASE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Base');

		$attrs = array(
            new DAttr('name', 'vhname', 'Virtual Host Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'vhName'),
			$this->_attrs['defaultCluster'],
			new DAttr('configFile', 'file', 'Configure File', 'text', false, 3, 'rwc', $this->_options['text_size']),
			$this->_attrs['note']
			);
		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'vhTop');

	}

	private function add_VH_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'General');

		$attrs = array(
			$this->_attrs['vh_maxKeepAliveReq'],
			$this->_attrs['vh_enableGzip'],
			$this->_attrs['enableIpGeo']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general');
	}

	private function add_VH_LOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host Log');

		$attrs = [
			new DAttr('useServer', 'bool', 'Use Server\'s Log', 'radio', false, NULL, NULL, NULL, 0, 'logUseServer'),
			$this->_attrs['fileName3']->dup(NULL, NULL, 'vhlog_fileName'),
			new DAttr('logLevel', 'sel', 'Log Level', 'select', true, 0, $this->_options['logLevel'], NULL, 0, 'vhlog_logLevel'),
			$this->_attrs['rollingSize'],
            $this->_attrs['keepDays'],
			$this->_attrs['compressArchive'],
			];
		$this->_tblDef[$id]->setAttr($attrs, 'general:log', 'logging:log', 'vhLog');
	}

    private function add_VH_ACLOG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Access Log');
		$use_options = array(0=>'Own Log File', 1=>'Server\'s Log File', 2=>'Disabled');
		$log_filename = $this->_attrs['fileName3'];
		$log_filename->_helpKey = 'vhaccessLog_fileName';

		$attrs = array(
			new DAttr('useServer', 'sel', 'Log Control', 'select', false, 0, $use_options, NULL, 0, 'aclogUseServer'),
			$log_filename,
			new DAttr('pipedLogger', 'cust', 'Piped Logger', 'text', true,
						NULL, NULL, $this->_options['text_size'], 0, 'accessLog_pipedLogger'),
			//new DAttr('pipedLogger', 'sel1', 'Piped Logger', 'select', true,
				//						  array( 'ext:logger'), NULL, NULL, 0, 'accessLog_pipedLogger'),
			$this->_attrs['logFormat'],
			$this->_attrs['logHeaders'],
			$this->_attrs['rollingSize'],
			$this->_attrs['keepDays'],
			$this->_attrs['compressArchive'],
			new DAttr('bytesLog', 'file0', 'Bytes log', 'text', true, 3, 'r', $this->_options['text_size'], 0, 'accessLog_bytesLog'),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general:aclog', 'logging:accessLog', 'vhAcLog');
	}


	private function add_VH_ERRPG_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Customized Error Pages', 1, 'VH_ERRPG', $align, "file");
		$errCodeOptions = &CustStatusCode::Get();
		$attrs = array(
			new DAttr('errCode', 'sel', 'Error Code', NULL, false, 0, $errCodeOptions),
			new DAttr('action', 'action', 'Action', NULL, false, 'VH_ERRPG', 'Ed'));
		$this->_tblDef[$id]->setAttr($attrs, 'general:errorPage', NULL, 'errPage');
		$this->_tblDef[$id]->setRepeated('errCode');
	}

	private function add_VH_ERRPG($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Customized Error Pages', 2);
		$errCodeOptions = &CustStatusCode::Get();
		$attrs = array(
			new DAttr('errCode', 'sel', 'Error Code', 'select', false, 0, $errCodeOptions),
  		    $this->_attrs['lb_errMessage'],
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general:errorPage', 'customErrorPages:errorPage', 'errPage');
		$this->_tblDef[$id]->setRepeated('errCode');
	}

	private function add_VH_REALM_TOP($id)
	{
		$align = array('center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Authorization Realms', 1, 'VH_REALM_SEL', $align, "shield");

		$attrs = array(
			new DAttr('name', 'cust', 'Realm Name'),
			$this->_attrs['realm_type'],
			new DAttr('action', 'action', 'Action', NULL, false, $this->_options['realmTbl'], 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm', NULL, 'realms');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_REALM_SEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Realm - Select Type');
		$this->_tblDef[$id]->_subTbls = $this->_options['realmTbl'];

		$attrs[] = $this->_attrs['realm_type'];
 		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm', 'security:realmList:realm', 'realms');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_REALM_FILE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Password File Realm Definition', 2);
		$udbLoc = new DAttr('userDB:location', 'file', 'User DB Location', 'text', false, 3, 'rc', $this->_options['text_size'], 0, 'userDBLocation');
		$udbLoc->_href = '&t1=VH_UDB_TOP&r1=$R';
		$gdbLoc = new DAttr('groupDB:location', 'file', 'Group DB Location', 'text', true, 3, 'rc', $this->_options['text_size'], 0, 'GroupDBLocation');
		$gdbLoc->_href = '&t1=VH_GDB_TOP&r1=$R';

		$attrs = array(
			$this->_attrs['realm_name'],
			$udbLoc,
			$this->_attrs['note'],
			$this->_attrs['realm_udb_maxCacheSize'],
			$this->_attrs['realm_udb_cacheTimeout'],
			$gdbLoc,
			$this->_attrs['realm_gdb_maxCacheSize'],
			$this->_attrs['realm_gdb_cacheTimeout']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('type'=>'file');
	}

	private function add_VH_REALM_LDAP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'LDAP Realm Definition', 2);

		$parseFormat = "/^(ldap|ldaps):\/\//";
		$parseHelp = 'start with ldap:// or ldaps://';

		$attrs = array(
			$this->_attrs['realm_name'],
			new DAttr('userDB:location', 'parse', 'User DB Location', 'text', false, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'userDBLocation'),
			new DAttr('userDB:attrPasswd', 'name', 'Password Attribute', 'text', true, NULL, NULL, NULL, 0, 'userDB_attrPasswd'),
			new DAttr('userDB:attrMemberOf', 'name', 'Member-of Attribute', 'text', true, NULL, NULL, NULL, 0, 'userDB_attrMemberOf'),
			$this->_attrs['realm_udb_maxCacheSize'],
			$this->_attrs['realm_udb_cacheTimeout'],
			new DAttr('groupDB:location', 'parse', 'Group DB Location', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'GroupDBLocation'),
			new DAttr('groupDB:attrGroupMember', 'name', 'Group Member Attribute', 'text', true, NULL, NULL, NULL, 0, 'groupDB_attrGroupMember'),
			$this->_attrs['realm_gdb_maxCacheSize'],
			$this->_attrs['realm_gdb_cacheTimeout'],
			new DAttr('LDAPBindDN', 'cust', 'LDAP Bind DN', 'text', true, NULL, NULL, $this->_options['text_size']),
			new DAttr('LDAPBindPasswd', 'cust', 'LDAP Bind Password', 'text'),
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('type'=>'LDAP');
	}

	private function add_VH_UDB_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'User DB Entries', 1, 'VH_UDB', $align);

		$attrs = array(
			new DAttr('name', 'cust', 'User Name'),
			new DAttr('group', 'cust', 'Groups'),
			new DAttr('action', 'action', 'Action', NULL, false, 'VH_UDB', 'Ed')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'UDB!$0');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_hasB = true;
		$this->_tblDef[$id]->_showParentRef = true;
	}

	private function add_VH_UDB($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'User DB Entry', 2);

		$attrs = array(
			new DAttr('name', 'name', 'User Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'UDBusername'),
			new DAttr('group', 'name', 'Groups', 'text', true, NULL, NULL, $this->_options['text_size'], 1, 'UDBgroup'),
			new DAttr('pass', 'cust', 'New Password', 'password', true, NULL, NULL, $this->_options['text_size'], 0, 'UDBpass'),
			new DAttr('pass1', 'cust', 'Retype Password', 'password', true, NULL, NULL, $this->_options['text_size'], 0, 'UDBpass1'),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'UDB!$0');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('passwd'=>'$$');
		$this->_tblDef[$id]->_hasB = true;
		$this->_tblDef[$id]->_showParentRef = true;
	}

	private function add_VH_GDB_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Group DB Entries', 1, 'VH_GDB', $align);

		$attrs = array(
			new DAttr('name', 'cust', 'Group Name'),
			new DAttr('users', 'cust', 'Users'),
			new DAttr('action', 'action', 'Action', NULL, false, 'VH_GDB', 'Ed')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'GDB!$0');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_hasB = true;
		$this->_tblDef[$id]->_showParentRef = true;
	}

	private function add_VH_GDB($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Group DB Entry', 2);
		$users = new DAttr('users', 'name', 'Users', 'textarea', true, NULL, NULL, 'rows="15" cols="50"', 1, 'gdb_users');
		$users->_glue = ' ';

		$attrs = array(
			new DAttr('name', 'name', 'Group Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'gdb_groupname'),
			$users,
			);
		$this->_tblDef[$id]->setAttr($attrs, 'GDB!$0');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_hasB = true;
		$this->_tblDef[$id]->_showParentRef = true;
	}

	private function add_VH_REWRITE_CTRL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Rewrite Control');

		$attrs = array(
			new DAttr('enable', 'bool', 'Enable Rewrite', 'radio', true, NULL, NULL, NULL, 0, 'enableRewrite'),
			new DAttr('logLevel', 'uint', 'Log Level', 'text', true, 0, 9, NULL, 0, 'rewriteLogLevel'));

		$this->_tblDef[$id]->setAttr($attrs, 'rewrite', 'rewrite');

	}

	private function add_VH_REWRITE_RULE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Rewrite Rules');

		$attrs = array(
			new DAttr('rules', 'cust', NULL, 'textarea1', true, NULL, NULL, 'rows="15" cols="60" wrap=off'));

		$this->_tblDef[$id]->setAttr($attrs, 'rewrite', 'rewrite', 'rewriteRules');
		$this->_tblDef[$id]->_cols = 1;
	}

	private function add_VH_REWRITE_MAP_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Rewrite Map', 1, 'VH_REWRITE_MAP', $align, "redirect");

		$attrs = array(
			new DAttr('name', 'cust', 'Name'),
			new DAttr('location', 'cust', 'Location'),
			new DAttr('action', 'action', 'Action', NULL, false, 'VH_REWRITE_MAP', 'Ed')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'rewrite:map', 'rewrite:map');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_REWRITE_MAP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Rewrite Map');

		$parseFormat = "/^((txt|rnd):\/*)|(int:(toupper|tolower|escape|unescape))$/";
		$parseHelp = 'txt:/location or rnd:/location or int:(toupper|tolower|escape|unescape)';

		$attrs = array(
			new DAttr('name', 'name',  'Name', 'text', false, NULL, NULL, NULL, 0, 'rewriteMapName'),
			new DAttr('location', 'parse', 'Location', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size'], 0, 'rewriteMapLocation'),
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'rewrite:map', 'rewrite:map');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_VH_CTX_TOP($id)
	{
		$align = array('center', 'left', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Context List', 1, 'VH_CTX_SEL', $align, array("NULL" => "file", "proxy" => "network", "redirect" => "redirect"));
		$o = new DAttr('order', 'ctxseq', 'Order');
		$o->_href = '&ctxseq=';
		$o->_FDE = 'NYN';

		$attrs = array(
			$this->_attrs['ctx_type'],
			new DAttr('uri', 'cust', 'URI'),
			$o,
			new DAttr('action', 'action', 'Action', NULL, false, $this->_options['ctxTbl'], 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'context');
		$this->_tblDef[$id]->setRepeated('uri');
	}

	private function add_VH_CTX_SEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Context');
		$this->_tblDef[$id]->_subTbls = $this->_options['ctxTbl'];

		$attrs[] = $this->_attrs['ctx_type'];
		$this->_tblDef[$id]->setAttr($attrs, 'context', 'contextList:context');
		$this->_tblDef[$id]->setRepeated('uri');
	}

	private function get_ctx_attrs($type)
	{
		switch ($type) {
			case 'auth':
				return [
					new DAttr('realm', 'sel1', 'Realm', 'select', true, array('realms')),
					new DAttr('authName', 'name', 'Authentication Name', 'text'),
					new DAttr('required', 'cust', 'Require (Authorized Users/Groups)', 'text'),
					new DAttr('accessControl:allow', 'subnet', 'Access Allowed', 'textarea', true, NULL, NULL, 'rows="3" cols="60"', 1, 'accessAllowed'),
					new DAttr('accessControl:deny', 'subnet', 'Access Denied', 'textarea', true, NULL, NULL, 'rows="3" cols="60"', 1, 'accessDenied'),
				];
			case 'rewrite':
				return [
					new DAttr('rewrite:enable', 'bool', 'Enable Rewrite', 'radio', true, NULL, NULL, NULL, 0, 'enableRewrite'),
					new DAttr('rewrite:inherit', 'bool', 'Rewrite Inherit', 'radio', true, NULL, NULL, NULL, 0, 'rewriteInherit'),
					new DAttr('rewrite:base', 'uri', 'Rewrite Base', 'text', true, NULL, NULL, $this->_options['text_size'], 0, 'rewriteBase'),
					new DAttr('rewrite:rules', 'cust', 'Rewrite Rules', 'textarea1', true, NULL, NULL, 'rows="6" cols="60" wrap=off', 0, 'rewriteRules'),
				];
			case 'charset':
				return [
					new DAttr('addDefaultCharset', 'sel', 'Add Default Charset', 'select', true, 0, $this->_options['on_off']),
					new DAttr('defaultCharsetCustomized', 'cust', 'Customized Default Charset', 'text'),
					$this->_attrs['enableIpGeo'],
				];
			case 'uri':
				return [
					$this->_attrs['ctx_uri'],
					$this->_attrs['ctx_order']
				];
			case 'cache':
			case 'serv_cache':
				$staticoptions = ['1' => 'CSS', '2' => 'JS', '4' => 'Images', '8' => 'Fonts'];
				$part0 = [
					new DAttr('cachePolicy:checkPublicCache', 'bool', 'Check Public Cache', 'radio', true, NULL, NULL, NULL, 0, 'checkPublicCache'),
					new DAttr('cachePolicy:checkPrivateCache', 'bool', 'Check Private Cache', 'radio', true, NULL, NULL, NULL, 0, 'checkPrivateCache'),
					new DAttr('cachePolicy:respectCacheable', 'bool', 'Respect Cacheable', 'radio', true, NULL, NULL, NULL, 0, 'respectCacheable'),
					new DAttr('cachePolicy:maxCacheObjSize', 'uint', 'Max Object Size', 'text', true, '1K', '100M'),
					new DAttr('cachePolicy:maxStaleAge', 'uint', 'Cache Stale Age (seconds)', 'text', true, 0),
					new DAttr('cachePolicy:cacheKeyMod', 'cust', 'Cache Key Modifications', 'textarea', true, NULL, NULL, 'rows="3" cols="60"'),
					];
				$enable = new DAttr('cachePolicy:enableCache', 'bool', 'Publicly Cache All', 'radio', true);
				$expire = new DAttr('cachePolicy:expireInSeconds', 'uint', 'Cache Expire Time (seconds)', 'text', true, 0);
				$enablePrivate = new DAttr('cachePolicy:enablePrivateCache', 'bool', 'Privately Cache All', 'radio', true);
				$part1 = [
					new DAttr('cachePolicy:privateExpireInSeconds', 'uint', 'Private Cache Expire Time (seconds)', 'text', true, 0),
					new DAttr('cachePolicy:microCache5xx', 'bool', 'Micro Cache 5XX Response', 'radio', true),
					new DAttr('cachePolicy:cacheStaticFile', 'checkboxOr', 'Cache Static Files', 'checkboxgroup', true, NULL, $staticoptions),
					new DAttr('cachePolicy:cacheStaticVary', 'bool', 'Vary for Static Files', 'radio', true),
					new DAttr('cachePolicy:enablePostCache', 'bool', 'Enable POST cache', 'radio', true),
				];

				if ($type == 'cache') {
					return array_merge($part0, [$enable, $expire, $enablePrivate], $part1);
				} else { // server_cache
					return array_merge($part0, [$expire], $part1);
				}
		}
	}

	private function add_VH_CTXB($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Load Balancer Context Definition', 2);
		$attrs = array_merge(
			$this->get_ctx_attrs('uri'),
			array(
                new DAttr('handler', 'sel1', 'Cluster', 'select', false, array('CLS'), NULL, NULL, 0, 'extCluster'),
                $this->_attrs['note'],
                $this->_attrs['extraHeaders']),
            $this->get_ctx_attrs('auth'),
            $this->get_ctx_attrs('cache'),
            $this->get_ctx_attrs('rewrite'),
            $this->get_ctx_attrs('charset')
		);
		$this->_tblDef[$id]->setAttr($attrs, 'context', NULL, 'loadBalancerContext');
		$this->_tblDef[$id]->setRepeated('uri');
		$this->_tblDef[$id]->_defaultExtract = array('type'=>'loadbalancer');
	}

	private function add_VH_CTXR($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Redirect Context Definition', 2);

		$options = CustStatusCode::GetRedirect();

		$attrs = array_merge(
				$this->get_ctx_attrs('uri'),
                array(
                    new DAttr('externalRedirect', 'bool',  'External Redirect', 'radio', false, NULL, NULL, NULL, 0, 'externalredirect'),
                    new DAttr('statusCode', 'sel', 'Status Code', 'select', true, 0, $options, NULL, 0, 'statuscode'),
                    new DAttr('location', 'url', 'Destination URI', 'text', true, NULL, NULL, $this->_options['text_size'], 0, 'destinationuri'),
                    $this->_attrs['note'],
                    $this->_attrs['extraHeaders']),
                $this->get_ctx_attrs('auth')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'context', NULL, 'redirectContext');
		$this->_tblDef[$id]->setRepeated('uri');
		$this->_tblDef[$id]->_defaultExtract = array('type'=>'redirect');
	}

	private function add_TP_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host Template List', 1, 'TP1', $align, "form");

		$attrs = array(
			new DAttr('name', 'cust', 'Name'),
			new DAttr('listeners', 'cust', 'Mapped Listeners'),
			new DAttr('action', 'action', 'Action', NULL, false, 'TP1', 'Xd'));
		$this->_tblDef[$id]->setAttr($attrs, 'tpTop');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_TP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Template Location', 2);

		$attrs = array(
			new DAttr('name', 'name', 'Template Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'templateName'),
			$this->_attrs['tp_templateFile'],
			$this->_attrs['tp_listeners'],
			$this->_attrs['note']
			);
		$this->_tblDef[$id]->setAttr($attrs);
	}

	private function add_TP1($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host Template', 2);

		$attrs = array(
			new DAttr('name', 'name', 'Template Name', 'text', false, NULL, NULL, $this->_options['text_size'], 0, 'templateName'),
			$this->_attrs['tp_templateFile'],
			$this->_attrs['tp_listeners'],
			$this->_attrs['note']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'tpTop');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_TP_ZCONF($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf', 2);

		$attrs = array(
		    new DAttr('enabled', 'bool', 'Used by ZeroConf', 'radio', true, NULL, NULL, NULL, 0, 'tpzconf_enabled'),
            new DAttr('vhostPerDomain', 'bool', 'Separate VHost Per Domain', 'radio', true, NULL, NULL, NULL, 0, 'tpzconf_vhostPerDomain'),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'zconf', 'zconf');
	}

	private function add_TP_MEMBER_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Member Virtual Hosts', 1, 'TP_MEMBER', $align, "web");

		$attrs = array(
			$this->_attrs['tp_vhName'],
			$this->_attrs['tp_vhDomain'],
			new DAttr('action', 'action', 'Action', NULL, false, 'TP_MEMBER', 'vEdi'));

		$this->_tblDef[$id]->setAttr($attrs, 'member', 'member');
		$this->_tblDef[$id]->setRepeated('vhName');
	}

	private function add_TP_MEMBER($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Member Virtual Host', 2);

		$attrs = array(
			$this->_attrs['tp_vhName'],
			$this->_attrs['tp_vhDomain'],
			$this->_attrs['defaultCluster'],
			new DAttr('vhAliases', 'domain', 'Aliases', 'text', true, NULL, NULL, $this->_options['text_size'], 1)
			);

		$this->_tblDef[$id]->setAttr($attrs, 'member', 'member');
		$this->_tblDef[$id]->setRepeated('vhName');
	}

	private function add_TP_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Template Settings');

		$attrs = array(
			$this->_attrs['tp_vrFile']->dup('configFile', 'Config File', NULL),
			$this->_attrs['vh_maxKeepAliveReq'],
			$this->_attrs['vh_enableGzip'],
			$this->_attrs['enableIpGeo']
			);
// to do: need check path contain VH variable.
		$this->_tblDef[$id]->setAttr($attrs, 'general');
	}

	private function add_TP_GENERAL1($id)
	{
		// for file use only
		$this->_tblDef[$id] = new DTbl($id);

		$attrs = array(
			$this->_attrs['tp_vrFile']->dup('configFile', 'Config File', NULL),
			$this->_attrs['defaultCluster']
			);
// to do: need check path contain VH variable.
		$this->_tblDef[$id]->setAttr($attrs, 'general');
	}

	private function add_TP_GENERAL2($id)
	{
		// for file use only
		$this->_tblDef[$id] = new DTbl($id);
		$attrs = array(
		$this->_attrs['vh_maxKeepAliveReq'],
		$this->_attrs['vh_enableGzip'],
			$this->_attrs['enableIpGeo']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general');
	}

	private function add_TP_LOG($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('VH_LOG', $id);
		$this->_tblDef[$id]->_dattrs[1] =  $this->_attrs['tp_vrFile'];
	}

	private function add_TP_ACLOG($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('VH_ACLOG', $id);
		$this->_tblDef[$id]->_dattrs[1] =  $this->_attrs['tp_vrFile'];
	}

	private function add_TP_SEC_CONN($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Per Client Throttling Control');

		$attrs = array(
			$this->_attrs['dynReqPerSec'],
			$this->_attrs['outBandwidth'],
			$this->_attrs['inBandwidth'],
                        new DAttr('maxConnsPerIp', 'uint', 'Max Connections Per IP',  'text', true, 0),
			
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:conn', 'security');
	}

	private function add_TP_SEC_BD($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'VHost Bandwidth Throttling Control');

		$attrs = array(
            new DAttr('totalBandwidth', 'uint', 'Total Output Bandwidth (KB/sec)', 'text', true, 0)
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:conn', 'security');
	}

	private function add_TP_REALM_TOP($id)
	{
		$align = array('center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Authorization Realms', 1, 'TP_REALM_SEL', $align);

		$attrs = array(
			new DAttr('name', 'cust', 'Realm Name'),
			$this->_attrs['realm_type'],
			new DAttr('action', 'action', 'Action', NULL, false, $this->_options['tp_realmTbl'], 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_TP_REALM_SEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Realm');
		$this->_tblDef[$id]->_subTbls = $this->_options['tp_realmTbl'];

		$attrs[] = $this->_attrs['realm_type'];
 		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm', 'security:realmList:realm');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_TP_REALM_FILE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Password File Realm Definition', 2);

		$attrs = array(
			$this->_attrs['realm_name'],
			new DAttr('userDB:location', 'cust', 'User DB Location', 'text', false, 3, 'rc', $this->_options['text_size']),
			$this->_attrs['realm_udb_maxCacheSize'],
			$this->_attrs['realm_udb_cacheTimeout'],
			new DAttr('groupDB:location', 'cust', 'Group DB Location', 'text', true, 3, 'rc', $this->_options['text_size']),
			$this->_attrs['realm_gdb_maxCacheSize'],
			$this->_attrs['realm_gdb_cacheTimeout']
			);
		$this->_tblDef[$id]->setAttr($attrs, 'sec:realm');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('type'=>'file');
	}


	private function add_ADMIN_PHP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'General');

		$attrs = array(
			new DAttr('enableCoreDump', 'bool', 'Enable Core Dump', 'radio', false ),
			new DAttr('sessionTimeout', 'uint', 'Session Timeout (secs)', 'text', true, 60)
			);
		$this->_tblDef[$id]->setAttr($attrs, 'general');
	}

	private function add_ADMIN_USR_TOP($id)
	{
		$align = array('left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Web Admin Users', 1, 'ADMIN_USR_NEW', $align, "administrator");

		$attrs = array(
			new DAttr('name', 'cust', 'Admin User Name'),
			new DAttr('action', 'action', 'Action', NULL, false, 'ADMIN_USR', 'Ed')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'ADM');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_ADMIN_USR($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Web Admin User', 2);

		$attrs = array(
			new DAttr('name', 'name', 'Web Admin User Name', 'text', false, NULL, NULL, $this->_options['text_size']),
			new DAttr('oldpass', 'cust', 'Old Password', 'password', true, NULL, NULL, $this->_options['text_size'], 0, 'adminOldPass'),
			new DAttr('pass', 'cust', 'New Password', 'password', true, NULL, NULL, $this->_options['text_size']),
			new DAttr('pass1', 'cust', 'Retype Password', 'password', true, NULL, NULL, $this->_options['text_size'])
			);
		$this->_tblDef[$id]->setAttr($attrs, 'ADM', NULL, 'adminUser');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('passwd'=>'$$');
	}

	private function add_ADMIN_USR_NEW($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Web Admin User', 2);

		$attrs = array(
			new DAttr('name', 'name', 'Web Admin User Name', 'text', false, NULL, NULL, $this->_options['text_size']),
			new DAttr('pass', 'cust', 'New Password', 'password', true, NULL, NULL, $this->_options['text_size']),
			new DAttr('pass1', 'cust', 'Retype Password', 'password', true, NULL, NULL, $this->_options['text_size'])
			);
		$this->_tblDef[$id]->setAttr($attrs, 'ADM');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('passwd'=>'$$');
	}

    function add_L_TOP($id)
	{
		$align = array('center', 'center', 'center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Listeners', 1, 'L_SEL', $align, "link");

		$attrs = array(
			$this->_attrs['l_name'],
            $this->_attrs['l_type'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
            $this->_attrs['l_secure'],
			new DAttr('action', 'action', 'Action', NULL, false, 'L_GENERAL_DEL', 'Xd'),

			);
		$this->_tblDef[$id]->setAttr($attrs, 'listeners');
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_L_SEL_FILE($id)
	{
        // for file
		$this->_tblDef[$id] = new DTbl($id, 'Listener');
		$this->_tblDef[$id]->_subTbls = $this->_options['lsnTbl'];

		$attrs = array($this->_attrs['l_type']);
		$this->_tblDef[$id]->setAttr($attrs);
	}

	function add_L_SEL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New Listener - Select Type');
		$this->_tblDef[$id]->_subTbls = $this->_options['lsnTblNew'];

		$attrs = array($this->_attrs['l_type']);
		$this->_tblDef[$id]->setAttr($attrs);
		$this->_tblDef[$id]->setRepeated('name');
	}

	function add_L_GENERAL_NEW($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New HTTP Listener');

		$attrs = array(
            $this->_attrs['l_name'],
            $this->_attrs['l_addr'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
			$this->_attrs['l_binding'],
            $this->_attrs['l_secure'],
            $this->_attrs['l_zconfEnabled'],
			$this->_attrs['note'],
			);

		$this->_tblDef[$id]->setAttr($attrs, 'listeners', NULL, 'LB_listenerGeneral');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('address'=>'$$', 'type'=>'http');
	}

	function add_L_GENERAL_DEL($id)
	{
        // for delete listener from ltop
		$this->_tblDef[$id] = new DTbl($id, 'Listener Settings');

		$attrs = array(
            $this->_attrs['l_name'],
            $this->_attrs['l_type'],
            $this->_attrs['l_addr'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
			$this->_attrs['l_binding'],
			$this->_attrs['note'],
			);

		$this->_tblDef[$id]->setAttr($attrs, 'listeners');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('address'=>'$$');
	}

	function add_L_GENERAL($id)
	{
        // for file & edit
		$this->_tblDef[$id] = new DTbl($id, 'Address Settings');

		$attrs = array(
            $this->_attrs['l_name'],
            $this->_attrs['l_type_file'],
            $this->_attrs['l_addr'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
			$this->_attrs['l_binding'],
            $this->_attrs['l_secure'],
            $this->_attrs['l_zconfEnabled'],
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'LB_listenerGeneral');
		$this->_tblDef[$id]->_defaultExtract = array('address'=>'$$', 'type'=>'http');
	}

	function add_LT_GENERAL_NEW($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'New TCP Listener');

		$attrs = array(
            $this->_attrs['l_name'],
            $this->_attrs['l_addr'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
			$this->_attrs['l_binding'],
            new DAttr('loadBalancer', 'sel1', 'Mapped Cluster', 'select', false, array('CLS_L4'), NULL, NULL, 0, 'mapped_L4_clusters'),
			$this->_attrs['note'],
			);

		$this->_tblDef[$id]->setAttr($attrs, 'listeners');
		$this->_tblDef[$id]->setRepeated('name');
		$this->_tblDef[$id]->_defaultExtract = array('address'=>'$$', 'type'=>'tcp');
	}

	function add_LT_GENERAL($id)
	{
        // for file & edit
		$this->_tblDef[$id] = new DTbl($id, 'Address Settings');

		$attrs = array(
            $this->_attrs['l_name'],
            $this->_attrs['l_type_file'],
            $this->_attrs['l_addr'],
            $this->_attrs['l_ip'],
            $this->_attrs['l_port'],
			$this->_attrs['l_binding'],
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs);
		$this->_tblDef[$id]->_defaultExtract = array('address'=>'$$', 'type'=>'tcp');
	}

    function add_LT_CLMAP($id)
	{
        // for file & edit
		$this->_tblDef[$id] = new DTbl($id, 'Layer 4 Cluster Mapping');

		$attrs = array(
            new DAttr('loadBalancer', 'sel1', 'Mapped Cluster', 'select', false, array('CLS_L4'), NULL, NULL, 0, 'mapped_L4_clusters')
			);
		$this->_tblDef[$id]->setAttr($attrs);
	}

    private function add_L_VHMAP_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host Mappings', 1, 'L_VHMAP', $align, "web_link");

		$attrs = array(
			$this->_attrs['l_vhost'],
			$this->_attrs['l_domain'],
			new DAttr('action', 'action', 'Action', NULL, false, 'L_VHMAP', 'Ed'));

		$this->_tblDef[$id]->setAttr($attrs, 'vhMap', 'vhostMapList:vhostMap', 'virtualHostMapping');
		$this->_tblDef[$id]->setRepeated('vhost');
	}

	private function add_L_VHMAP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host Mappings', 2);

		$attrs = array(
			$this->_attrs['l_vhost'],
			$this->_attrs['l_domain']);
		$this->_tblDef[$id]->setAttr($attrs, 'vhMap', 'vhostMapList:vhostMap', 'virtualHostMapping');
		$this->_tblDef[$id]->setRepeated('vhost');
	}

	private function add_L_SSL_CERT($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'SSL Private Key & Certificate');

		$attrs = array(
			new DAttr('keyFile', 'cust', 'Private Key File', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('certFile', 'cust', 'Certificate File', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('certChain', 'bool', 'Chained Certificate', 'radio'),
			new DAttr('CACertPath', 'cust', 'CA Certificate Path', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('CACertFile', 'cust', 'CA Certificate File', 'text', true, NULL, '', $this->_options['text_size']),
		    );
		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'sslCert');
	}

	private function add_VH_SSL_CERT($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('L_SSL_CERT', $id,  'SSL Private Key & Certificate for Virtual Host');
		$this->_tblDef[$id]->setDataLoc('vhssl', 'vhssl');
	}

	private function add_L_SSL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'SSL Protocol');
		$sslversion = ['1'=>'SSL v3.0', '2'=>'TLS v1.0', '4'=>'TLS v1.1', '8'=>'TLS v1.2', '16'=>'TLS v1.3'];

		$attrs = [
			new DAttr('sslProtocol', 'checkboxOr', 'Protocol Version', 'checkboxgroup', TRUE, NULL, $sslversion),
			new DAttr('ciphers', 'cust', 'Ciphers', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('enableECDHE', 'bool', 'Enable ECDH Key Exchange', 'radio', true),
			new DAttr('enableDHE', 'bool', 'Enable DH Key Exchange', 'radio', true),
			new DAttr('DHParam', 'cust', 'DH Parameter', 'text', true, NULL, '', $this->_options['text_size']),
		];

		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'sslProtocolSetting');
	}

	private function add_VH_SSL_SSL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Virtual Host SSL Protocol');

		$attrs = [
			new DAttr('ciphers', 'cust', 'Ciphers', 'text', true, NULL, '', $this->_options['text_size']),
			new DAttr('enableECDHE', 'bool', 'Enable ECDH Key Exchange', 'radio', true),
			new DAttr('enableDHE', 'bool', 'Enable DH Key Exchange', 'radio', true),
			new DAttr('DHParam', 'cust', 'DH Parameter', 'text', true, NULL, '', $this->_options['text_size']),
		];

		$this->_tblDef[$id]->setAttr($attrs, 'vhssl', 'vhssl');
	}

	function add_L_SSL_FEATURE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Security & Features');

		$attrs = array(
			$this->_attrs['ssl_renegProtection'],
            $this->_attrs['sslSessionCache'],
            $this->_attrs['sslSessionTickets'],
            new DAttr('enableSpdy', 'checkboxOr', 'ALPN', 'checkboxgroup', true, NULL, array('1'=>'SPDY/2', '2'=>'SPDY/3', '4'=>'HTTP/2', '8'=>'HTTP/3', '0'=>'None')),
            new DAttr('enableQuic', 'bool', 'Open HTTP3/QUIC (UDP) Port', 'radio', true, NULL, NULL, NULL, 0, 'allowQuic'),
		    );
		$this->_tblDef[$id]->setAttr($attrs);
	}

	function add_VH_SSL_FEATURE($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Security & Features');

		$attrs = array(
			$this->_attrs['ssl_renegProtection'],
            $this->_attrs['sslSessionCache'],
            $this->_attrs['sslSessionTickets'],
            new DAttr('enableSpdy', 'checkboxOr', 'ALPN', 'checkboxgroup', true, NULL, array('1'=>'SPDY/2', '2'=>'SPDY/3', '4'=>'HTTP/2', '8'=>'HTTP/3', '0'=>'None')),
            new DAttr('enableQuic', 'bool', 'Enable HTTP3/QUIC', 'radio', true, NULL, NULL, NULL, 0, 'vhEnableQuic'),
		    );
		$this->_tblDef[$id]->setAttr($attrs, 'vhssl', 'vhssl');
	}

	function add_L_SSL_OCSP($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'OCSP Stapling');

		$attrs = array(
		    new DAttr('enableStapling', 'bool', 'Enable OCSP Stapling', 'radio', true),
		    new DAttr('ocspRespMaxAge', 'uint', 'OCSP Response Max Age (secs)', 'text', true, -1, NULL),
		    new DAttr('ocspResponder', 'httpurl', 'OCSP Responder', 'text', true, NULL, NULL, $this->_options['text_size']),
		    new DAttr('ocspCACerts', 'cust', 'OCSP CA Certificates', 'text', true, NULL, '', $this->_options['text_size']),
		    );
		$this->_tblDef[$id]->setAttr($attrs, NULL, NULL, 'sslOCSP');
	}

	function add_VH_SSL_OCSP($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('L_SSL_OCSP', $id);
		$this->_tblDef[$id]->setDataLoc('vhssl', 'vhssl');
	}

	function add_L_SSL_CLIENT_VERIFY($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Client Verification');
		$clientVerify =	new DAttr('clientVerify', 'sel', 'Client Verification', 'select', true, NULL,
			array('0'=>'none','1'=>'optional','2'=>'require','3'=>'optional_no_ca' ));
		$verifyDepth = new DAttr('verifyDepth', 'uint', 'Verify Depth', 'text', true, 0, 100);
		$crlPath = new DAttr('crlPath', 'cust', 'Client Revocation Path', 'text', true, NULL, '', $this->_options['text_size']);
		$crlFile = new DAttr('crlFile', 'cust', 'Client Revocation File', 'text', true, NULL, '', $this->_options['text_size']);

		$attrs = array($clientVerify, $verifyDepth, $crlPath, $crlFile);
		$this->_tblDef[$id]->setAttr($attrs);
	}

	function add_VH_SSL_CLIENT_VERIFY($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('L_SSL_CLIENT_VERIFY', $id);
		$this->_tblDef[$id]->setDataLoc('vhssl', 'vhssl');
	}


	private function add_ADMIN_L_TOP($id)
	{
		$align = array('center', 'center', 'center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'Listener List', 1, 'ADMIN_L_GENERAL1', $align, "link");

		$attrs = array(
			new DAttr('name', 'cust', 'Listener Name'),
			new DAttr('ip', 'cust', 'IP Address'),
			new DAttr('port', 'cust', 'Port'),
			new DAttr('secure', 'bool', 'Secure'),
			new DAttr('action', 'action', 'Action', NULL, false, 'ADMIN_L_GENERAL1', 'Xd'),

			);
		$this->_tblDef[$id]->setAttr($attrs, 'listeners');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_ADMIN_L_GENERAL($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('L_GENERAL', $id, 'Admin Listener Address Settings');
	}

	private function add_ADMIN_L_GENERAL1($id)
	{
		$this->_tblDef[$id] = $this->DupTblDef('ADMIN_L_GENERAL', $id);
		$this->_tblDef[$id]->setDataLoc('listeners');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_SERVICE_SUSPENDVH($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Suspend Vhosts');

		$attr0 = new DAttr('suspendedVhosts', 'vhname', NULL);
		$attr0->_multiInd = 1;
		$attr0->_FDE = 'YNN';
		$attrs = array( $attr0);
		$this->_tblDef[$id]->setAttr($attrs, 'service');
	}

    function add_HA_CONF_REPLICATION($id)
    {
		$this->_tblDef[$id] = new DTbl($id, 'Replication');

		$attrs = array(
			new DAttr('serverAddr', 'cust', 'Server Address', 'text', true, null, null, $this->_options['text_size'], 0, 'haRep_serverAddr'),
            new DAttr('replCluster', 'cust', 'Replication Cluster', 'textarea1', true, null, null,
			    'rows="2" cols="60" wrap=off', 0, 'haRep_replCluster'),
            new DAttr('heartBeatInterval', 'uint', 'Heart Beat Interval (secs)', 'text', true, 10, null, null, 0, 'haRep_heartBeatInterval'),
            new DAttr('heartBeatTimeout', 'uint', 'Heart Beat Timeout (secs)', 'text', true, 20, null, null, 0, 'haRep_heartBeatTimeout'),
            new DAttr('isGzipStream', 'bool', 'Is Gzip Stream', 'radio', true, null, null, null, 0, 'haRep_isGzipStream'),
            new DAttr('maxAckDiff', 'uint', 'Max Sequence Number Difference', 'text', true, 1, 65535, null, 0, 'haRep_maxAckDiff'),
            new DAttr('syncInterval', 'uint', 'Sync Interval', 'text', true, 1, 255, null, 0, 'haRep_syncInterval'),
            new DAttr('priority', 'int', 'Data Master Priority', 'text', true, -32766, 32767, null, 0, 'haRep_priority'),
            new DAttr('isFileCached', 'bool', 'Enable Replication for Cached Data', 'radio', true, null, null, null, 0, 'haRep_isFileCached'),
            new DAttr('fileCacheLstnr', 'cust', 'File Cache Server Address', 'text', true, null, null, $this->_options['text_size'], 0, 'haRep_fileCacheLstnr'),
			);
		$this->_tblDef[$id]->setAttr($attrs, 'replication', 'replication', 'HAReplication');

    }

	function add_HA_CONF_TOP($id)
	{
		$align = array('center', 'left', 'center', 'center', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'HA Interfaces', 1, 'HA_CONF', $align, "network");

		$attrs = array(
			new DAttr('id', 'cust', 'Interface ID', NULL, true, NULL, NULL, NULL, 0, 'haInterfaceId'),
			new DAttr('vip', 'cust', 'Virtual IP', NULL, true, NULL, NULL, NULL, 0, 'haVip'),
			new DAttr('vipId', 'uint', 'VIP ID', NULL, true, NULL, NULL, NULL, 0, 'haVipId'),
            new DAttr('nic', 'cust', 'NIC Name', NULL, true, NULL, NULL, NULL, 0, 'haNic'),
			new DAttr('priority', 'uint', 'Priority', NULL, true, 0, 255, NULL, 0, 'haPriority'),
            new DAttr('internal_dev', 'cust', 'Internal device', NULL, true, NULL, NULL, NULL, 0, 'haInternalDev'),
            new DAttr('unicastSrcIp', 'cust', 'Unicast Src', null, true, null, null, null, 0, 'haUniSrcVip'),
            new DAttr('unicastPeerIps', 'cust', 'Unicast Peers', null, true, null, null, null, 1, 'haUniPeerVips'),
			new DAttr('action', 'action', 'Action', NULL, true, 'HA_CONF', 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'vipconf', 'interfaces:vipconf', 'HAInterface');
		$this->_tblDef[$id]->setRepeated('id');
	}

	function add_HA_CONF($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'HA Virtual Interface Configuration');

		$attrs = array(
			new DAttr('id', 'uint', 'Interface ID', 'text', false, 1, 100, $this->_options['text_size'], 0, 'haInterfaceId'),
			new DAttr('vip', 'cust', 'Virtual IPs', 'text', false, null, null, $this->_options['text_size'], 1, 'haVip'),
			new DAttr('vipId', 'uint', 'VIP ID', 'text', false, 1, 255, $this->_options['text_size'], 0, 'haVipId'),
			new DAttr('vipToken', 'text', 'VIP Token', 'text', false, null, null, $this->_options['text_size'], 0, 'haVipToken'),
            new DAttr('nic', 'sel', 'NIC Name', 'select', true, 0, $this->_options['nic'], NULL, 0, 'haNic'),
            new DAttr('priority', 'uint', 'Priority', 'text', false, 0, 255, NULL, 0, 'haPriority'),
            new DAttr('internal_dev', 'sel', 'Internal Device', 'select', true, 0, $this->_options['nic'], NULL, 0, 'haInternalDev'),
            new DAttr('unicastSrcIp', 'cust', 'Unicast Source Virtual IP', 'text', true, null, null, $this->_options['text_size'], 0, 'haUniSrcVip'),
            new DAttr('unicastPeerIps', 'cust', 'Unicast Peer Virtual IPs', 'text', true, null, null, $this->_options['text_size'], 1, 'haUniPeerVips'),
			$this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'vipconf', 'interfaces:vipconf', 'HAInterface');
		$this->_tblDef[$id]->setRepeated('id');

	}

	function add_HA_STATUS($id)
	{
		$this->_tblDef[$id] = new DTblHAStatus($id, 'HA Status');
		$vipToken = new DAttr('vipToken', 'cust', 'VIP Token');
		$vipToken->_FDE = 'NNN';

		$attrs = array(
			new DAttr('id', 'cust', 'Interface ID', NULL, true, NULL, NULL, NULL, 0, 'haInterfaceId'),
			new DAttr('vip', 'cust', 'Configured VIP'),
			new DAttr('vipId', 'cust', 'VIP ID'),
			$vipToken,
			new DAttr('nic', 'cust', 'NIC'),
            new DAttr('priority', 'uint', 'Configured Priority', 'text', false, 0, 255, NULL, 0, 'haPriority'),
			new DAttr('internal_dev', 'cust', 'INT_DEV'),
            new DAttr('unicastSrcIp', 'cust', 'Unicast Src'),
            new DAttr('unicastPeerIps', 'cust', 'Unicast Peers'),
			new DAttr('status', 'cust', 'Configuration Status'),
            $this->_attrs['note'],
			new DAttr('sys_vip', 'cust', 'System VIP'),
			new DAttr('sys_vipId', 'cust', 'System VIP ID'),
			new DAttr('sys_nic', 'cust', 'NIC Name'),
			new DAttr('sys_priority', 'cust', 'System Priority'),
			new DAttr('sys_internal_dev', 'cust', 'Internal Device'),
            new DAttr('sys_unicastSrcIp', 'cust', 'System Unicast Src'),
            new DAttr('sys_unicastPeerIps', 'cust', 'System Unicast Peers'),
			new DAttr('sys_status', 'cust', 'System Status'),
			new DAttr('action', 'action', 'Action', NULL, false, NULL, 'yzdap')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'vipconf', 'interfaces:vipconf', 'HAStatus');
		$this->_tblDef[$id]->setRepeated('id');

	}

    function add_HA_REPLSTATUS($id)
    {
        $this->_tblDef[$id] = new DTblReplStatus($id, 'Replication Status');
        $this->_tblDef[$id]->_helpKey = 'replicationStatus';
    }

	function add_ZCONF_GENERAL($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'General');

		$attrs = array(
		    new DAttr('enabled', 'bool', 'Enable ZeroConf', 'radio', true, NULL, NULL, NULL, 0, 'zconf_enabled'),
            new DAttr('vhostPerDomain', 'bool', 'Separate VHost Per Domain', 'radio', true, NULL, NULL, NULL, 0, 'zconf_vhostPerDomain'),
            new DAttr('logPerDomain', 'checkboxOr', 'Separate Log Per Domain', 'checkboxgroup', true, NULL, 
                    array('1'=>'Access Log', '2' => 'Error Log'), NULL, 0, 'zconf_logPerDomain'),
            new DAttr('domLogDir', 'cust', 'Domain Log Directory', 'text', true, NULL, '', $this->_options['text_size'], 0, 'zconf_domLogDir'),
            new DAttr('passwdFile', 'cust', 'User Password File', 'text', true, NULL, '', $this->_options['text_size'], 0, 'zconf_passwdFile'),
            new DAttr('groupFile', 'cust', 'Group Password File', 'text', true, NULL, '', $this->_options['text_size'], 0, 'zconf_groupFile'),
            new DAttr('domCiphers', 'cust', 'Domain Ciphers', 'text', true, NULL, '', $this->_options['text_size'], 0, 'zconf_domCiphers'),
			$this->_attrs['lb_mode']->dup(NULL, NULL, 'zconf_mode'),
            $this->_attrs['lb_strategy'],
            $this->_attrs['lb_showBackend'],
            $this->_attrs['lb_forwardIpHeader'],
            $this->_attrs['lb_forwardByHeader'],
            new DAttr('zconfShmDir', 'cust', 'ZeroConf SHM Directory', 'text', true, NULL, '', $this->_options['text_size']),
		    );
		$this->_tblDef[$id]->setAttr($attrs, 'zconf', 'zeroConfig');
	}

	function add_ZCONF_WORKER($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf Worker');
		$parseFormat = "/^\/.+$/";
		$parseHelp = 'A uri path starting with "/"';

		$attrs = [
			$this->_attrs['worker_protocol'],
			new DAttr('workerMaxConns', 'uint', 'Worker Max Connections', 'text', true, 1, 2000, NULL, 0, 'zconf_workerMaxConns'),
			$this->_attrs['initTimeout'],
			$this->_attrs['retryTimeout'],
			$this->_attrs['persistConn'],
			$this->_attrs['pcKeepAliveTimeout'],
			$this->_attrs['respBuffer'],
			new DAttr('pingPath', 'parse', 'Ping Path', 'text', true, $parseFormat, $parseHelp, $this->_options['text_size']),
			$this->_attrs['pingInterval'],
			$this->_attrs['pingSmartFactor'],
		];
		$this->_tblDef[$id]->setAttr($attrs, 'zconf', 'zeroConfig');
	}

	private function add_ZCONF_AC($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'Access Control');

		$attrs = array(
			new DAttr('allow', 'subnet', 'Allowed List', 'textarea', true, NULL, NULL, 'rows="5" cols="60"',1, 'zconf_accessControl_allow'),
			new DAttr('deny', 'subnet', 'Denied List', 'textarea', true, NULL, NULL, 'rows="5" cols="60"', 1, 'zconf_accessControl_deny'));
		$this->_tblDef[$id]->setAttr($attrs, 'zconf:ac', 'zeroConfig:accessControl', 'zconf_accessControl');
		$this->_tblDef[$id]->_cols = 1;
    }

	private function add_ZCONF_L_TOP($id)
	{
		$align = array('left', 'left', 'center');
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf Listeners', 1, 'ZCONF_L', $align, 'link');

		$attrs = array(
			new DAttr('name', 'cust', 'Name', NULL, true, NULL, NULL, NULL, 0, 'zconf_listenerName'),
            new DAttr('addr', 'ipport', 'Address', 'text', false, NULL, NULL, NULL, 0, 'zconf_laddr'),
            new DAttr('action', 'action', 'Action', NULL, false, 'ZCONF_L', 'vEd')
			);
		$this->_tblDef[$id]->setAttr($attrs, 'zconf:listener', 'zeroConfig:listenerList');
		$this->_tblDef[$id]->setRepeated('name');
	}

	private function add_ZCONF_L($id)
	{
		$this->_tblDef[$id] = new DTbl($id, 'ZeroConf Listener');
        $ltype = new DAttr('type', 'cust', 'Type', 'text', false);
        $ltype->_FDE = 'YNN';
		$attrs = array(
            $ltype,
            $this->_attrs['l_name'],
            new DAttr('addr', 'ipport', 'Address', 'text', false, NULL, NULL, NULL, 0, 'zconf_laddr'),
			new DAttr('keyFile', 'cust', 'Private Key File', 'text', false, NULL, '', $this->_options['text_size']),
			new DAttr('certFile', 'cust', 'Certificate File', 'text', false, NULL, '', $this->_options['text_size']),
            new DAttr('CACertFile', 'cust', 'CA Certificate File', 'text', true, NULL, '', $this->_options['text_size']),
            $this->_attrs['note'],
			);
		$this->_tblDef[$id]->setAttr($attrs, 'zconf:listener', 'zeroConfig:listenerList:listener');
		$this->_tblDef[$id]->setRepeated('name');
        $this->_tblDef[$id]->_defaultExtract = array('type'=>'http');

	}


}

