
CURDIR=`dirname "$0"`
cd $CURDIR
CURDIR=`pwd`
LSHOME=`dirname $CURDIR`
LSHOME=`dirname $LSHOME`

display_usage()
{
    cat <<EOF
Usage: lsup.sh [-f] [-v VERSION]
  
  -f
     Force reinstall. If -f is not given and same version already installed, this upgrade command is aborted. With -f option, latest build will be downloaded and installed even within same release version.

  -v VERSION
     If VERSION is given, this command will try to install specified VERSION. Otherwise, it will get the latest version from $LSHOME/autoupdate/release.

  --help     display this help and exit

EOF
    exit 1
}

FORCED=N
VERSION=x
BUILD=
DEBUG=0
while [ "x$1" != "x" ] 
do
    if [ "x$1" = "x-d" ] ; then
        DEBUG=1
    elif [ "x$1" = "x-f" ] ; then
        FORCED=Y
    elif [ "x$1" = "--help" ] ; then
        display_usage
    elif [ "x$1" = "x-v" ] ; then
        shift
        VERSION=$1
        if [ "x$VERSION" = "x" ] ; then
            display_usage
        fi
    elif [ "x$1" = "x-b" ] ; then
        shift
        BUILD=".$1"
        if [ "x$BUILD" = "x" ] ; then
            display_usage
        fi
    else
        display_usage
    fi
    shift;
done


# detect download method
OS=`uname -s`
DLCMD=x
if [ "x$OS" = "xFreeBSD" ] ; then
    DL=`which fetch`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -o"
    fi
fi
if [ "$DLCMD" = "x" ] ; then
    DL=`which wget`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -nv -O"
    fi
fi
if [ "$DLCMD" = "x" ] ; then
    DL=`which curl`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -L -o"
    fi
fi
if [ "$DLCMD" = "x" ] ; then
    echo "[ERROR] Fail to detect proper download method"
    exit 1
fi

# check latest release
$LSHOME/bin/lslbd -U

if [ ! -f "$LSHOME/autoupdate/release" ] ; then
    echo "[ERROR] Fail to locate file $LSHOME/autoupdate/release"
    exit 1
fi

if [ ! -f "$LSHOME/autoupdate/platform" ] ; then
    echo "[ERROR] Fail to locate file $LSHOME/autoupdate/platform"
    exit 1
fi

RELEASE=`cat $LSHOME/autoupdate/release`
PLATFORM=`cat $LSHOME/autoupdate/platform`
if [ $VERSION = "x" ] ; then
    VERSION=$RELEASE
else
    RELEASE=$VERSION
fi

CURVERSION=`cat $LSHOME/VERSION`
if [ "$VERSION" = "$CURVERSION" ] && [ "$FORCED" = "N" ] ; then
    echo "[ERROR] Abort - Same version already installed. If you want to do force reinstall, please use option -f"
    exit 1
fi

FILENAME="lslb-$RELEASE-$PLATFORM.tar.gz"
if [ -e "$LSHOME/autoupdate/$FILENAME" ] ; then
    /bin/rm -f "$LSHOME/autoupdate/$FILENAME"
fi


MAJOR_VERSION=`expr $RELEASE : '\([0-9]*\)\..*'`
 
# http://www.litespeedtech.com/packages/lslb/lslb-2.0-i386-linux.tar.gz
if [ $DEBUG =  '1' ]; then
    DOWNLOAD_URL="https://www.litespeedtech.com/packages/dbg/lslb-$RELEASE-$PLATFORM-dbg.tar.gz"
else
    DOWNLOAD_URL="https://www.litespeedtech.com/packages/lslb/$FILENAME"
    echo "Query latest build of $FILENAME"
    BUILD_URL=$DOWNLOAD_URL.lastbuild
    $DLCMD "$LSHOME/autoupdate/build" $BUILD_URL

    if [ "x$BUILD" =  'x' ]; then
        BUILD=`cat "$LSHOME/autoupdate/build"`
        if [ "x$BUILD" != 'x' ]; then
            echo "Download the latest Build [$BUILD]."
            BUILD=".$BUILD"
        fi
    else
        echo "Download Build [$BUILD]."
    fi
    DOWNLOAD_URL=$DOWNLOAD_URL$BUILD
fi 



echo "$DLCMD $LSHOME/autoupdate/$FILENAME $DOWNLOAD_URL"

$DLCMD "$LSHOME/autoupdate/$FILENAME" $DOWNLOAD_URL

if [ $? != 0 ]; then
    echo "[ERROR] Failed to download $DOWNLOAD_URL"
    exit 2
fi

echo "$LSHOME/admin/misc/update.sh $VERSION $EDITION $PLATFORM"
$LSHOME/admin/misc/update.sh $VERSION $EDITION $PLATFORM

echo "Restarting LSLB"
$LSHOME/bin/lslbctrl restart
echo "Done"
