#!/bin/sh
CURDIR=`dirname "$0"`
cd $CURDIR
CURDIR=`pwd`

INST_USER=`id`
INST_USER=`expr "$INST_USER" : 'uid=.*(\(.*\)) gid=.*'`
if [ $INST_USER != "root" ]; then
	cat <<EOF
[ERROR] Only root user can install the rc script!
EOF
	exit 1
fi
INIT_DIR=""

if [ "x`uname -s`" = "xDarwin" ]; then

	STARTUP_ITEM=/System/Library/StartupItems/lslb
	if [ ! -d $STARTUP_ITEM ]; then
		mkdir $STARTUP_ITEM
	fi
	cp "$CURDIR/lslb.rc" $STARTUP_ITEM/lslb
	cat <<EOF >$STARTUP_ITEM/StartupParameters.plist
{
  Description     = "LiteSpeed load balancer";
  Provides        = ("Load Balancer");
  Requires        = ("DirectoryServices");
  Uses            = ("Disks", "NFS");
  OrderPreference = "None";
}

EOF


	exit 0
fi

if [ "x`uname -s`" = "xFreeBSD" ]; then
	if [ -d "/usr/local/etc/rc.d" ]; then
		
		cp "$CURDIR/lslb.rc" /usr/local/etc/rc.d/lslb.sh
		chmod 755 /usr/local/etc/rc.d/lslb.sh
		echo "[OK] The startup script has been successfully installed!"
		exit 0
	else
		cat <<EOF	
[ERROR] /usr/local/etc/rc.d/ does not exit in this FreeBSD system!

EOF
		exit 1
	fi
fi 

SYSTEMDDIR=""

SYSTEMBIN=`which systemctl 2>/dev/null`
if [ $? -eq 0 ] ; then
    for path in /etc/systemd/system /usr/lib/systemd/system
    do
        if [ "${SYSTEMDDIR}" = "" ] ; then
            if [ -d "$path" ]; then
                SYSTEMDDIR=$path
            fi
        fi
    done
fi



if [ "${SYSTEMDDIR}" != "" ] ; then

    cp -f ${CURDIR}/lslb.service ${SYSTEMDDIR}/lslb.service
    chmod 644 ${SYSTEMDDIR}/lslb.service
    systemctl daemon-reload
    systemctl enable lslb.service 
    if [ $? -eq 0  ]; then
        echo "[OK] lslb.service has been successfully installed!"
        exit 0
    else
        echo "[ERROR] failed to enable lslb.service in systemd!"
        exit 1
    fi
fi


if [ -f "/etc/gentoo-release" ]; then
    cp "$CURDIR/lslb.rc.gentoo" /etc/init.d/lslb
    chmod a+x /etc/init.d/lslb
    rc-update add lslb default
    exit 0
fi


for path in /etc/init.d /etc/rc.d/init.d 
do
	if [ "x$INIT_DIR" = "x" ]; then
		if [ -d "$path" ]; then
			INIT_DIR=$path
		fi
	fi
done
if [ "x$INIT_DIR" = "x" ]; then
	echo "[ERROR] failed to find the init.d directory!"
	exit 1
fi

if [ -f "$INIT_DIR/lslb" ]; then
	rm -f $INIT_DIR/lslb
fi

if [ -e /etc/debian_version ]; then
    cp "$CURDIR/lslb.rc" ${INIT_DIR}/lslb
    chmod 755 ${INIT_DIR}/lslb
    update-rc.d lslb defaults
    echo "[OK] The startup script has been successfully installed!"
    exit 0
fi


INIT_BASE_DIR=`dirname $INIT_DIR`

cp "$CURDIR/lslb.rc" $INIT_DIR/lslb
chmod 755 $INIT_DIR/lslb


if [ -d "$INIT_BASE_DIR/runlevel/default" ]; then
	ln -fs ../init.d/lslb $INIT_BASE_DIR/runlevel/default/S88lslb
	ln -fs ../init.d/lslb $INIT_BASE_DIR/runlevel/default/K12lslb
fi


if [ -d "$INIT_BASE_DIR/rc2.d" ]; then
	ln -fs ../init.d/lslb $INIT_BASE_DIR/rc2.d/S88lslb
	ln -fs ../init.d/lslb $INIT_BASE_DIR/rc2.d/K12lslb
fi

if [ -d "$INIT_BASE_DIR/rc3.d" ]; then
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc3.d/S88lslb
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc3.d/K12lslb
fi

if [ -d "$INIT_BASE_DIR/rc5.d" ]; then
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc5.d/S88lslb
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc5.d/K12lslb
fi

if [ -d "$INIT_BASE_DIR/rc0.d" ]; then
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc0.d/K12lslb
fi

if [ -d "$INIT_BASE_DIR/rc1.d" ]; then
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc1.d/K12lslb
fi

if [ -d "$INIT_BASE_DIR/rc6.d" ]; then
ln -fs ../init.d/lslb $INIT_BASE_DIR/rc6.d/K12lslb
fi

echo "[OK] The startup script has been successfully installed!"

exit 0