
#!/bin/sh


init()
{
    LSINSTALL_DIR=`pwd`
    VERSION=`cat VERSION`

    export LSINSTALL_DIR

    DIR_MOD=755
    SDIR_MOD=700
    EXEC_MOD=555
    CONF_MOD=600
    DOC_MOD=644

    INST_USER=`id`
    INST_USER=`expr "$INST_USER" : 'uid=.*(\(.*\)) gid=.*'`

    ARCH=`arch`
    SYS_NAME=`uname -s`
    if [ "x$SYS_NAME" = "xFreeBSD" ] || [ "x$SYS_NAME" = "xNetBSD" ] || [ "x$SYS_NAME" = "xDarwin" ] ; then
        PS_CMD="ps -ax"
        ID_GROUPS="id"
        TEST_BIN="/bin/test"
        ROOTGROUP="wheel"
    else
        PS_CMD="ps -ef"
        ID_GROUPS="id -a"
        TEST_BIN="/usr/bin/test"
        ROOTGROUP="root"
    fi
    SETUP_PHP=0
    SET_LOGIN=0
    ADMIN_PORT=7090
    INSTALL_TYPE="upgrade"
    SERVER_NAME=`uname -n`
}

license()
{
    more ./LICENSE
    cat <<EOF

IMPORTANT: In order to continue installation you must agree with above 
           license terms by typing "Yes" with capital "Y"! 

EOF
    SUCC=0
    TRY=1
    while [ $SUCC -eq "0" ]; do
        printf "%s" "Do you agree with above license? "  
        read YES_NO
        if [ "x$YES_NO" != "xYes" ]; then
            if [ $TRY -lt 3 ]; then
                echo "Sorry, wrong answer! Type 'Yes' with capital 'Y', try again!"
                TRY=`expr $TRY + 1`
            else
                echo "Abort installation!" 
                exit 0
            fi

        else
            SUCC=1
        fi
    done
    clear
}

readCurrentConfig()
{
    OLD_USER_CONF=`grep "<user>" "$LSLB_HOME/conf/lslbd_config.xml"`
    OLD_GROUP_CONF=`grep "<group>" "$LSLB_HOME/conf/lslbd_config.xml"`
    OLD_USER=`expr "$OLD_USER_CONF" : '.*<user>\(.*\)</user>.*'`
    OLD_GROUP=`expr "$OLD_GROUP_CONF" : '.*<group>\(.*\)</group>.*'`
    if [ "x$OLD_USER" != "x" ]; then
        WS_USER=$OLD_USER
    fi
    if [ "x$OLD_GROUP" != "x" ]; then
        WS_GROUP=$OLD_GROUP
    else
        D_GROUP=`$ID_GROUPS $WS_USER`
        WS_GROUP=`expr "$D_GROUP" : '.*gid=[0-9]*(\(.*\)) groups=.*'`
    fi
    DIR_OWN=$WS_USER:$WS_GROUP
    CONF_OWN=$WS_USER:$WS_GROUP

}




# Get destination directory
install_dir()
{

    SUCC=0
    INSTALL_TYPE="reinstall"
    SET_LOGIN=1

    if [ $INST_USER = "root" ]; then
        DEST_RECOM="/usr/local/lslb"
        if [ -f "/opt/lslb/conf/lslbd_config.xml" ]; then
            DEST_RECOM="/opt/lslb"
        fi
        WS_USER="nobody"
    else 
        cat <<EOF

As you are not the 'root' user, you may not be able to install the
load balancer into a system directory or enable chroot, the load balancer
process will run on behalf of current user - '$INST_USER'.

EOF
        WS_USER=$INST_USER
        DEST_RECOM="~/lslb"
    fi



    while [ $SUCC -eq "0" ];  do
        cat <<EOF

Please specify the destination directory. You must have permissions to 
create and manage the directory. It is recommended to install the load balancer
at /opt/lslb, /usr/local/lslb or in your home directory like '~/lslb'.

ATTENTION: The user '$WS_USER' must be able to access the destination
           directory.

EOF
        printf "%s" "Destination [$DEST_RECOM]: "
        read TMP_DEST
        echo ""
        if [ "x$TMP_DEST" = "x" ]; then
            TMP_DEST=$DEST_RECOM
        fi
        if [ `expr "$TMP_DEST" : '~'` -gt 0 ]; then 
            LSLB_HOME="$HOME`echo $TMP_DEST | sed 's/^~//' `"
        else
            LSLB_HOME=$TMP_DEST
        fi
        if [ `expr "$LSLB_HOME" : '\/'` -eq 0 ]; then
            echo "[ERROR] Must be absolute path!"
        else
            SUCC=1
        fi
        if [ ! -d "$LSLB_HOME" ]; then
            mkdir "$LSLB_HOME"
            if [ ! $? -eq 0 ]; then
                SUCC=0
            fi
        fi
        if [ -f "$LSLB_HOME/conf/lslbd_config.xml" ]; then
            cat <<EOF

Found old configuration file under destination directory $LSLB_HOME. 

To upgrade, press 'Enter', current configuration will not be changed.
To reinstall, press 'R' or 'r'.
To change directory, press 'C' or 'c'. 

EOF

            printf "%s" "Would you like to Upgrade, Reinstall or Change directory [U/r/c]? "
            read TMP_URC
            echo ""
            if [ "x$TMP_URC" = "x" ]; then
                INSTALL_TYPE="upgrade"
                SET_LOGIN=0
            else 
                if [ `expr "$TMP_URC" : '[Uu]'` -gt 0 ]; then
                    INSTALL_TYPE="upgrade"
                    SET_LOGIN=0
                else 
                    if [ `expr "$TMP_URC" : '[Rr]'` -gt 0 ]; then
                        INSTALL_TYPE="reinstall"
                        SET_LOGIN=1
                    else 
                    #if [ `expr "$TMP_URC" : '[Cc]'` -gt 0 ]; then
                        SUCC=0
                    fi
                fi
            fi

        fi

    done

    export LSLB_HOME

    if [ -f "$LSLB_HOME/conf/lslbd_config.xml" ]; then
        readCurrentConfig
    else
        INSTALL_TYPE="reinstall"
    fi


    DIR_OWN=$WS_USER:$WS_GROUP
    CONF_OWN=$WS_USER:$WS_GROUP

    chmod $DIR_MOD "$LSLB_HOME"
}


admin_login()
{
    if [ $INSTALL_TYPE = "upgrade" ]; then
        printf "%s" "Would you like to reset the login password for Web Administration Console [y/N]? "
        read TMP_URC
        echo ""
        if [ "x$TMP_URC" != "x" ]; then
            if [ `expr "$TMP_URC" : '[Yy]'` -gt 0 ]; then
                SET_LOGIN=1
            fi
        fi
    fi        

    if [ $SET_LOGIN -eq 1 ]; then

# get admin user name and password

        SUCC=0
        cat <<EOF

Please specify the user name of the administrator.
This is the user name required to log into the web administration console.

EOF

        printf "%s" "User name [admin]: "
        read ADMIN_USER
        if [ "x$ADMIN_USER" = "x" ]; then
            ADMIN_USER=admin
        fi

        cat <<EOF

Please specify the administrator's password.
This is the password required to log into the web administration console.

EOF

        while [ $SUCC -eq "0" ];  do
            printf "%s" "Password: "
            stty -echo
            read PASS_ONE
            stty echo
            echo ""
            if [ `expr "$PASS_ONE" : '.*'` -ge 6 ]; then
                printf "%s" "Retype password: "
                stty -echo
                read PASS_TWO
                stty echo
                echo ""
                if [ "x$PASS_ONE" = "x$PASS_TWO" ]; then
                    SUCC=1
                else
                    echo ""
                    echo "[ERROR] Sorry, passwords does not match. Try again!"
                    echo ""
                fi
            else
                echo ""
                echo "[ERROR] Sorry, password must be at least 6 characters!"
                echo ""
            fi
        done


# generate password file

        ENCRYPT_PASS=`"$LSINSTALL_DIR/admin/fcgi-bin/admin_php5" -q "$LSINSTALL_DIR/admin/misc/htpasswd.php" $PASS_ONE`
        echo "$ADMIN_USER:$ENCRYPT_PASS" > "$LSINSTALL_DIR/admin/conf/htpasswd"

    fi

}


getUserGroup()
{

    if [ $INST_USER = "root" ]; then
        cat <<EOF

As you are the root user, you must choose the user and group whom the load 
balancer will be running as. For security reason, you should choose a 
non-system user who does not have login shell and home directory such 
as 'nobody'.

EOF
# get user name 
        SUCC=0
        while [ $SUCC -eq "0" ]; do
            printf "%s" "User [$WS_USER]: "
            read TMP_USER
            if [ "x$TMP_USER" = "x" ]; then
                TMP_USER=$WS_USER
            fi
            USER_INFO=`id $TMP_USER 2>/dev/null`
            TST_USER=`expr "$USER_INFO" : 'uid=.*(\(.*\)) gid=.*'`
            if [ "x$TST_USER" = "x$TMP_USER" ]; then
                USER_ID=`expr "$USER_INFO" : 'uid=\(.*\)(.*) gid=.*'`
                if [ $USER_ID -gt 10 ]; then  
                    WS_USER=$TMP_USER
                    SUCC=1
                else
                    cat <<EOF

[ERROR] It is not allowed to run LiteSpeed load balancer on behalf of a 
privileged user, user id must be greater than 10. The user id of user 
'$TMP_USER' is '$USER_ID'.

EOF
                fi
            else
                cat <<EOF 

[ERROR] '$TMP_USER' is not valid user name in your system, please choose 
another user or create user '$TMP_USER' first.

EOF

            fi
        done
    fi

# get group name 
    SUCC=0
    TMP_GROUPS=`groups $WS_USER`
    TST_GROUPS=`expr "$TMP_GROUPS" : '.*:\(.*\)'`
    if [ "x$TST_GROUPS" = "x" ]; then
        TST_GROUPS=$TMP_GROUPS
    fi

    D_GROUP=`$ID_GROUPS $WS_USER`
    D_GROUP=`expr "$D_GROUP" : '.*gid=[0-9]*(\(.*\)) groups=.*'`
    echo "Please choose the group that the load balancer running as."
    echo 
    while [ $SUCC -eq "0" ];  do
        echo "User '$WS_USER' is the member of following group(s): $TST_GROUPS"
        printf "%s" "Group [$D_GROUP]: "
        read TMP_GROUP
        if [ "x$TMP_GROUP" = "x" ]; then
            TMP_GROUP=$D_GROUP
        fi
        GRP_RET=`echo $TST_GROUPS | grep -w "$TMP_GROUP"`
        if [ "x$GRP_RET" != "x" ]; then
            WS_GROUP=$TMP_GROUP
            SUCC=1
        else
            cat <<EOF

[ERROR] '$TMP_GROUP' is not valid group for user '$WS_USER', please choose
another group in the list or add user '$WS_USER' to group '$TMP_GROUP' 
first.

EOF
        fi
    done

    DIR_OWN=$WS_USER:$WS_GROUP
    CONF_OWN=$WS_USER:$WS_GROUP

    if [ $INST_USER = "root" ]; then
        if [ $SUCC -eq "1" ]; then
            chown -R "$DIR_OWN" "$LSLB_HOME"
        fi
    fi
}

test_running()
{
PIDFILE=/tmp/lslbd/lslbd.pid

RUNNING=0
if [ -f $PIDFILE ] ; then
    FPID=`cat $PIDFILE`
    if [ "x$FPID" != "x" ]; then
        kill -0 $FPID 2>/dev/null
        if [ $? -eq 0 ] ; then
            RUNNING=1
            PID=$FPID
        fi
    fi
fi
}


stopLslbd()
{
    test_running

    if [ $RUNNING -eq "1" ]; then
        cat <<EOF
LiteSpeed Load balancer is running, in order to continue installation, the 
server must be stopped.

EOF
        printf "Would you like to stop it now? [Y/n]"
        read TMP_YN
        echo ""
        if [ "x$TMP_YN" = "x" ] || [ `expr "$TMP_YN" : '[Yy]'` -gt 0 ]; then
            $LSINSTALL_DIR/bin/lslbctrl stop
            sleep 1
            test_running
            if [ $RUNNING -eq "1" ]; then
                echo "Failed to stop server, abort installation!"
                exit 1
            fi
        else
            echo "Abort installation!"
            exit 1
        fi
    fi

}


# get normal TCP port
getServerPort()
{
    cat <<EOF

Please specify the port for normal Load Balancer service.
Port 80 is the standard HTTP port, only 'root' user is allowed to use 
port 80, if you have another server running on port 80, you need to
specify another port or stop the other server before starting LiteSpeed
Load Balancer.

EOF

    SUCC=0
    DEFAULT_PORT=8090
    while [ $SUCC -eq "0" ];  do
        printf "%s" "HTTP port [$DEFAULT_PORT]: "
        read TMP_PORT
        if [ "x$TMP_PORT" = "x" ]; then
            TMP_PORT=$DEFAULT_PORT
        fi
        SUCC=1
        if [ `expr "$TMP_PORT" : '.*[^0-9]'` -gt 0 ]; then
            echo "[ERROR] Only digits is allowed, try again!"
            SUCC=0
        fi
        if  [ $SUCC -eq 1 ]; then
            if [ $INST_USER != "root" ]; then
                if [ $TMP_PORT -le 1024 ]; then
                    echo "[ERROR] Only 'root' can use port below 1024, try again!"
                    SUCC=0
                fi
            fi
        fi
        if [ $SUCC -eq 1 ]; then
            if [ `netstat -an | grep -w $TMP_PORT | grep -w LISTEN | wc -l` -gt 0 ]; then
                echo "[ERROR] Port $TMP_PORT is in use now, stop the server using this port first,"
                echo "        or choose another port."
                SUCC=0
            fi
        fi
    done

    HTTP_PORT=$TMP_PORT
}


# get administration TCP port
getAdminPort()
{
    cat <<EOF

Please specify the HTTP port for the web administration console,
which can be accessed through http://<YOUR_HOST>:<ADMIN_PORT>/

EOF

    SUCC=0
    DEFAULT_PORT=7090
    while [ $SUCC -eq "0" ];  do
        printf "%s" "Admin HTTP port [$DEFAULT_PORT]: "
        read TMP_PORT
        if [ "x$TMP_PORT" = "x" ]; then
            TMP_PORT=$DEFAULT_PORT
        fi
        SUCC=1
        if [ `expr "$TMP_PORT" : '.*[^0-9]'` -gt 0 ]; then
            echo "[ERROR] Only digits is allowed, try again!"
            SUCC=0
        fi
        if  [ $SUCC -eq 1 ]; then
            if [ $INST_USER != "root" ]; then
                if [ $TMP_PORT -le 1024 ]; then
                    echo "[ERROR] Only 'root' can use port below 1024, try again!"
                    SUCC=0
                fi
            fi
        fi
        if  [ $SUCC -eq 1 ]; then
            if [ $TMP_PORT -eq $HTTP_PORT ]; then
                echo "[ERROR] The admin HTTP port must be different from the normal HTTP port!"
                SUCC=0
            fi
        fi

        if [ $SUCC -eq 1 ]; then
            if [ `netstat -an | grep -w $TMP_PORT | grep -w LISTEN | wc -l` -gt 0 ]; then
                echo "[ERROR] Port $TMP_PORT is in use, stop the server that using this port first,"
                echo "        or choose another port."
                SUCC=0
            fi
        fi
    done

    ADMIN_PORT=$TMP_PORT
}

enablePHPHandler()
{
    cat <<EOF

You can setup a global script handler for PHP with the pre-built PHP engine
shipped with this package now. The PHP engine runs as Fast CGI which  
outperforms Apache's mod_php. 
You can always replace the pre-built PHP engine with your customized PHP 
engine.

EOF

    SUCC=0
    SETUP_PHP=1
    printf "%s" "Setup up PHP [Y/n]: "
    read TMP_YN
    if [ "x$TMP_YN" != "x" ]; then
        if [ `expr "$TMP_YN" : '[Nn]'` -gt 0 ]; then
            SETUP_PHP=0
        fi
    fi
    if [ $SETUP_PHP -eq 1 ]; then
        PHP_SUFFIX="php"
        printf "%s" "Suffix for PHP script(comma separated list) [$PHP_SUFFIX]: " 
        read TMP_SUFFIX
        if [ "x$TMP_SUFFIX" != "x" ]; then
            PHP_SUFFIX=$TMP_SUFFIX
        fi
#        PHP_PORT=5101
#        SUCC=0
#        while [ $SUCC -eq "0" ];  do
#            if [ `netstat -an | grep -w $PHP_PORT | grep -w LISTEN | wc -l` -eq 0 ]; then
#                SUCC=1
#            fi
#            PHP_PORT=`expr $PHP_PORT + 1`
#        done
    fi
}


# detect download method
detect_dlcmd()
{
DLCMD=x
if [ "x$SYS_NAME" = "xFreeBSD" ] ; then
    DL=`which fetch`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -o"
    fi
fi
if [ "$DLCMD" = "x" ] ; then
    DL=`which wget`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -nv -O"
    fi
fi
if [ "$DLCMD" = "x" ] ; then
    DL=`which curl`
    if [ $? -eq 0 ] ; then
    DLCMD="$DL -L -o"
    fi
fi
}

buildAdminSslCert()
{
    if [ ! -f "$LSLB_HOME/admin/conf/cert/admin.crt" ]; then
        HN=`hostname`
        detect_dlcmd
        if [ "x${DLCMD}" != 'xx' ]; then
            $DLCMD $LSWS_HOME/myip http://cyberpanel.sh/?ip
            MYIP=`cat $LSWS_HOME/myip`
            rm $LSWS_HOME/myip
        fi
        openssl rand -out $HOME/.rnd -hex 256
        openssl req -subj "/CN=$HN/O=webadmin/C=US/subjectAltName=DNS.1=${MYIP}/" -new -newkey rsa:2048 -sha256 -days 365 -nodes -x509 -keyout "$LSLB_HOME/admin/conf/cert/admin.key" -out "$LSLB_HOME/admin/conf/cert/admin.crt"
    fi
}


# generate configuration from template

buildConfigFiles()
{

#sed -e "s/%ADMIN_PORT%/$ADMIN_PORT/" -e "s/%PHP_FCGI_PORT%/$ADMIN_PHP_PORT/" "$LSINSTALL_DIR/admin/conf/admin_config.xml.in" > "$LSINSTALL_DIR/admin/conf/admin_config.xml"

    sed -e "s/%ADMIN_PORT%/$ADMIN_PORT/" "$LSINSTALL_DIR/admin/conf/admin_config.xml.in" > "$LSINSTALL_DIR/admin/conf/admin_config.xml"

    sed -e "s/%USER%/$WS_USER/" -e "s/%GROUP%/$WS_GROUP/" -e "s/%HTTP_PORT%/$HTTP_PORT/" -e "s/%SERVER_NAME%/$SERVER_NAME/" "$LSINSTALL_DIR/conf/lslbd_config.xml.in" > "$LSINSTALL_DIR/conf/lslbd_config.xml.tmp"

    if [ $SETUP_PHP -eq 1 ]; then
        sed -e "s/%PHP_BEGIN%//" -e "s/%PHP_END%//" -e "s/%PHP_SUFFIX%/$PHP_SUFFIX/" -e "s/%PHP_PORT%/$PHP_PORT/" "$LSINSTALL_DIR/conf/lslbd_config.xml.tmp" > "$LSINSTALL_DIR/conf/lslbd_config.xml"
    else
        sed -e "s/%PHP_BEGIN%/<!--/" -e "s/%PHP_END%/-->/" -e "s/%PHP_SUFFIX%/php/" -e "s/%PHP_PORT%/5201/" "$LSINSTALL_DIR/conf/lslbd_config.xml.tmp" > "$LSINSTALL_DIR/conf/lslbd_config.xml"
    fi

}

util_mkdir()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      if [ ! -d "$LSLB_HOME/$arg" ]; then
          mkdir "$LSLB_HOME/$arg"
      fi
      chown "$OWNER" "$LSLB_HOME/$arg"
      chmod $PERM  "$LSLB_HOME/$arg"
    done
    
}


util_cpfile()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      cp -f "$LSINSTALL_DIR/$arg" "$LSLB_HOME/$arg"
      chown "$OWNER" "$LSLB_HOME/$arg"
      chmod $PERM  "$LSLB_HOME/$arg"
    done
    
}

util_ccpfile()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      if [ ! -f "$LSLB_HOME/$arg" ]; then
          cp "$LSINSTALL_DIR/$arg" "$LSLB_HOME/$arg"
      fi
      chown "$OWNER" "$LSLB_HOME/$arg"
      chmod $PERM  "$LSLB_HOME/$arg"
    done
}


util_cpdir()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      cp -R "$LSINSTALL_DIR/$arg/"* "$LSLB_HOME/$arg/"
      chown -R "$OWNER" "$LSLB_HOME/$arg/"*
      #chmod -R $PERM  $LSLB_HOME/$arg/*
    done
}



util_cpdirv()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    VERSION=$1
    shift
    for arg
      do
      cp -R "$LSINSTALL_DIR/$arg/"* "$LSLB_HOME/$arg.$VERSION/"
      chown -R "$OWNER" "$LSLB_HOME/$arg.$VERSION"
      $TEST_BIN -L "$LSLB_HOME/$arg"
      if [ $? -eq 0 ]; then
          rm -f "$LSLB_HOME/$arg"
      fi
      FILENAME=`basename $arg`
      ln -sf "./$FILENAME.$VERSION/" "$LSLB_HOME/$arg"
              #chmod -R $PERM  $LSLB_HOME/$arg/*
    done
}

util_cpfilev()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    VERSION=$1
    shift
    for arg
      do
      cp -f "$LSINSTALL_DIR/$arg" "$LSLB_HOME/$arg.$VERSION"
      chown "$OWNER" "$LSLB_HOME/$arg.$VERSION"
      chmod $PERM  "$LSLB_HOME/$arg.$VERSION"
      $TEST_BIN -L "$LSLB_HOME/$arg"
      if [ $? -eq 0 ]; then
          rm -f "$LSLB_HOME/$arg"
      fi
      FILENAME=`basename $arg`
      ln -sf "./$FILENAME.$VERSION" "$LSLB_HOME/$arg"
    done
}

enableAdminPHPA()
{
    VER_STR=`$LSLB_HOME/admin/fcgi-bin/admin_php5 -v`

    PHP_VER=`expr "$VER_STR" : 'PHP \([^ ]*\) .*'`

    if [ ! -f $LSLB_HOME/lib/apc.so.$PHP_VER ]; then
        echo "[ERROR] Can not find $LSLB_HOME/lib/apc.so.$PHP_VER, please run install.sh again!"
    else
        ln -sf ../../lib/apc.so.$PHP_VER $LSLB_HOME/admin/fcgi-bin/apc.so
    fi
}

installation1()
{
    umask 022
    if [ $INST_USER = "root" ]; then
        SDIR_OWN="root:$ROOTGROUP"
        chown $SDIR_OWN $LSLB_HOME
    else
        SDIR_OWN=$DIR_OWN
    fi
    sed "s:%LSLB_CTRL%:$LSLB_HOME/bin/lslbctrl:" "$LSINSTALL_DIR/admin/misc/lslb.rc.in" > "$LSINSTALL_DIR/admin/misc/lslb.rc"

    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      if [ ! -f "$LSLB_HOME/$arg" ]; then
          cp "$LSINSTALL_DIR/$arg" "$LSLB_HOME/$arg"
      fi
      chown "$OWNER" "$LSLB_HOME/$arg"
      chmod $PERM  "$LSLB_HOME/$arg"
    done
}


util_cpdir()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    for arg
      do
      cp -R "$LSINSTALL_DIR/$arg/"* "$LSLB_HOME/$arg/"
      chown -R "$OWNER" "$LSLB_HOME/$arg/"*
      #chmod -R $PERM  $LSLB_HOME/$arg/*
    done
}



util_cpdirv()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    VERSION=$1
    shift
    for arg
      do
      cp -R "$LSINSTALL_DIR/$arg/"* "$LSLB_HOME/$arg.$VERSION/"
      chown -R "$OWNER" "$LSLB_HOME/$arg.$VERSION"
      $TEST_BIN -L "$LSLB_HOME/$arg"
      if [ $? -eq 0 ]; then
          rm -f "$LSLB_HOME/$arg"
      fi
      FILENAME=`basename $arg`
      ln -sf "./$FILENAME.$VERSION/" "$LSLB_HOME/$arg"
              #chmod -R $PERM  $LSLB_HOME/$arg/*
    done
}

util_cpfilev()
{
    OWNER=$1
    PERM=$2
    shift
    shift
    VERSION=$1
    shift
    for arg
      do
      cp -f "$LSINSTALL_DIR/$arg" "$LSLB_HOME/$arg.$VERSION"
      chown "$OWNER" "$LSLB_HOME/$arg.$VERSION"
      chmod $PERM  "$LSLB_HOME/$arg.$VERSION"
      $TEST_BIN -L "$LSLB_HOME/$arg"
      if [ $? -eq 0 ]; then
          rm -f "$LSLB_HOME/$arg"
      fi
      FILENAME=`basename $arg`
      ln -sf "./$FILENAME.$VERSION" "$LSLB_HOME/$arg"
    done
}

enableAdminPHPA()
{
    VER_STR=`$LSLB_HOME/admin/fcgi-bin/admin_php5 -v`

    PHP_VER=`expr "$VER_STR" : 'PHP \([^ ]*\) .*'`

    if [ ! -f $LSLB_HOME/lib/apc.so.$PHP_VER ]; then
        echo "[ERROR] Can not find $LSLB_HOME/lib/apc.so.$PHP_VER, please run install.sh again!"
    else
        ln -sf ../../lib/apc.so.$PHP_VER $LSLB_HOME/admin/fcgi-bin/apc.so
    fi
}


create_lsadm_freebsd()
{
    pw group add lsadm
    lsadm_gid=`grep "^lsadm:" /etc/group | awk -F : '{ print $3; }'`
    pw user add -g $lsadm_gid -d / -s /usr/sbin/nologin -n lsadm 
    pw usermod lsadm -G $WS_GROUP
}

create_lsadm()
{
    groupadd lsadm 
    #1>/dev/null 2>&1
    lsadm_gid=`grep "^lsadm:" /etc/group | awk -F : '{ print $3; }'`
    useradd -g $lsadm_gid -d / -r -s /sbin/nologin lsadm 
    usermod -a -G $WS_GROUP lsadm
    #1>/dev/null 2>&1
    
}

create_lsadm_solaris()
{
    groupadd lsadm 
    #1>/dev/null 2>&1
    lsadm_gid=`grep "^lsadm:" /etc/group | awk -F: '{ print $3; }'`
    useradd -g $lsadm_gid -d / -s /bin/false lsadm 
    usermod -G $WS_GROUP lsadm

    #1>/dev/null 2>&1
    
}



install_missing_packages()
{
    if [ -f /etc/redhat-release ] ; then
        output=$(cat /etc/redhat-release)
        if echo $output | grep " 9."; then
            yum -y install libxcrypt-compat libnsl
        fi
        if [ "${ARCH}" = "aarch64" ]; then
            yum -y install libatomic
        fi
    elif [ -f /etc/debian_version ] ; then
        if [ "${ARCH}" = "aarch64" ]; then
            apt -y install libatomic1
        fi
    fi
}


installation()
{
    umask 022
    if [ $INST_USER = "root" ]; then
        export PATH=/sbin:/usr/sbin:$PATH
        if [ "x$SYS_NAME" = "xLinux" ]; then
            create_lsadm
            install_missing_packages
        elif [ "x$SYS_NAME" = "xFreeBSD" ] || [ "x$SYS_NAME" = "xNetBSD" ]; then
            create_lsadm_freebsd
        elif [ "x$SYS_NAME" = "xSunOS" ]; then
            create_lsadm_solaris
        fi 
        grep "^lsadm:" /etc/passwd 1>/dev/null 2>&1
        if [ $? -eq 0 ]; then
            CONF_OWN="lsadm:lsadm"
        fi
        SDIR_OWN="root:$ROOTGROUP"
        LOGDIR_OWN="root:$WS_GROUP"
        chown $SDIR_OWN $LSLB_HOME
    else
        SDIR_OWN=$DIR_OWN
        LOGDIR_OWN=$DIR_OWN
    fi
    
    sed "s:%LSLB_CTRL%:$LSLB_HOME/bin/lslbctrl:" "$LSINSTALL_DIR/admin/misc/lslb.rc.in" > "$LSINSTALL_DIR/admin/misc/lslb.rc"
    sed "s:%LSLB_CTRL%:$LSLB_HOME/bin/lslbctrl:" "$LSINSTALL_DIR/admin/misc/lslb.rc.gentoo.in" > "$LSINSTALL_DIR/admin/misc/lslb.rc.gentoo"
    sed "s:%LSLB_CTRL%:$LSLB_HOME/bin/lslbctrl:" "$LSINSTALL_DIR/admin/misc/lslb.service.in" > "$LSINSTALL_DIR/admin/misc/lslb.service"

    if [ -d "$LSLB_HOME/admin/html.$VERSION" ]; then
        rm -rf "$LSLB_HOME/admin/html.$VERSION"
    fi

    util_mkdir "$SDIR_OWN" $DIR_MOD admin bin docs lib add-ons admin/fcgi-bin admin/html.$VERSION admin/misc
    util_mkdir "$LOGDIR_OWN" "0750" logs admin/logs
    util_mkdir "$CONF_OWN" $SDIR_MOD conf conf/cert conf/templates admin/conf admin/conf/cert admin/tmp autoupdate
    util_mkdir "$CONF_OWN" $DIR_MOD admin/htpasswds

    util_mkdir "$SDIR_OWN" $SDIR_MOD admin/cgid
    chgrp  $WS_GROUP $LSLB_HOME/admin/tmp $LSLB_HOME/admin/cgid $LSLB_HOME/admin/htpasswds
    chmod  g+x $LSLB_HOME/admin/tmp $LSLB_HOME/admin/cgid $LSLB_HOME/admin/htpasswds
    
    chown  $CONF_OWN $LSLB_HOME/admin/tmp/sess_* 1>/dev/null 2>&1
    
    buildAdminSslCert
    
    util_cpdir "$SDIR_OWN" $DOC_MOD add-ons
    util_cpfile "$SDIR_OWN" $EXEC_MOD add-ons/modsec/inspectmulti.sh
    
    util_cpfile "$SDIR_OWN" $EXEC_MOD admin/fcgi-bin/admin_php5 
    util_cpfile "$SDIR_OWN" $EXEC_MOD admin/misc/rc-inst.sh admin/misc/admpass.sh admin/misc/chroot.sh admin/misc/rc-uninst.sh admin/misc/uninstall.sh admin/misc/lslb.rc admin/misc/lslb.service admin/misc/lslb.rc.gentoo admin/misc/mgr_ver.sh admin/misc/update.sh admin/misc/ha_ctrl admin/misc/ha_trackstatus admin/misc/create_admin_keypair.sh admin/misc/cleancache.sh admin/misc/cleanlitemage.sh admin/misc/lsup.sh
    util_cpfile "$SDIR_OWN" $DOC_MOD admin/misc/gdb-bt admin/misc/htpasswd.php admin/misc/php.ini admin/misc/genjCryptionKeyPair.php

    "$LSLB_HOME/admin/fcgi-bin/admin_php5" -v 2>&1 1>/dev/null
    if [ $? -ne 0 ] && [ -e /etc/redhat-release ]; then
        yum install -y libnsl
    fi

    if [ $SET_LOGIN -eq 1 ]; then
        util_cpfile "$CONF_OWN" $CONF_MOD admin/conf/htpasswd
    else
        util_ccpfile "$CONF_OWN" $CONF_MOD admin/conf/htpasswd
    fi
    

    if [ ! -f "$LSLB_HOME/admin/htpasswds/status" ]; then
        cp "$LSLB_HOME/admin/conf/htpasswd" "$LSLB_HOME/admin/htpasswds/status"
    fi
    chown  $CONF_OWN "$LSLB_HOME/admin/htpasswds/status"
    chgrp  $WS_GROUP "$LSLB_HOME/admin/htpasswds/status"
    chmod  0640 "$LSLB_HOME/admin/htpasswds/status"
    

    if [ $INSTALL_TYPE = "upgrade" ]; then
        util_ccpfile "$CONF_OWN" $CONF_MOD admin/conf/admin_config.xml
        util_cpfile "$CONF_OWN" $CONF_MOD admin/conf/php.ini
        util_ccpfile "$CONF_OWN" $CONF_MOD conf/lslbd_config.xml conf/example.xml conf/mime.properties conf/templates/ccl.xml
        util_ccpfile "$CONF_OWN" $CONF_MOD conf/templates/ccl.xml

        $TEST_BIN ! -L "$LSLB_HOME/bin/lslbd"
        if [ $? -eq 0 ]; then
            mv -f "$LSLB_HOME/bin/lslbd" "$LSLB_HOME/bin/lslbd.old"
        fi
        $TEST_BIN ! -L "$LSLB_HOME/bin/lslbctrl"
        if [ $? -eq 0 ]; then
            mv -f "$LSLB_HOME/bin/lslbctrl" "$LSLB_HOME/bin/lslbctrl.old"
        fi
        $TEST_BIN ! -L "$LSLB_HOME/admin/html"
        if [ $? -eq 0 ]; then
            mv -f "$LSLB_HOME/admin/html" "$LSLB_HOME/admin/html.old"
        fi

    else
        util_cpfile "$CONF_OWN" $CONF_MOD admin/conf/admin_config.xml
        util_cpfile "$CONF_OWN" $CONF_MOD conf/templates/ccl.xml
        util_cpfile "$CONF_OWN" $CONF_MOD admin/conf/php.ini
        util_cpfile "$CONF_OWN" $CONF_MOD conf/lslbd_config.xml conf/example.xml conf/mime.properties
    fi

    util_cpdirv "$SDIR_OWN" $DOC_MOD $VERSION admin/html


    util_cpfile "$SDIR_OWN" $EXEC_MOD bin/lbwatch.sh
    util_cpfilev "$SDIR_OWN" $EXEC_MOD $VERSION bin/lslbctrl bin/lslbd bin/lslbd.dbg
    
    if [ -f $LSINSTALL_DIR/modules/modpagespeed.so ]; then
        util_mkdir "$SDIR_OWN" $DIR_MOD modules
        util_cpfilev "$SDIR_OWN" $EXEC_MOD $VERSION modules/modpagespeed.so
    fi

    util_mkdir "$SDIR_OWN" $DIR_MOD lsrecaptcha
    util_cpfile "$SDIR_OWN" $EXEC_MOD lsrecaptcha/_recaptcha
    util_cpfile "$SDIR_OWN" $DOC_MOD lsrecaptcha/_recaptcha.shtml

    util_cpdir "$SDIR_OWN" $DOC_MOD docs/
    util_cpfile "$SDIR_OWN" $DOC_MOD VERSION BUILD LICENSE* 
    if [ -f $LSLB_HOME/autoupdate/release ]; then
        rm $LSLB_HOME/autoupdate/release
    fi
    if [ -f $LSLB_HOME/autoupdate/download ]; then
        rm $LSLB_HOME/autoupdate/download
    fi

    if [ ! -f "$LSLB_HOME/admin/conf/jcryption_keypair" ]; then
        $LSLB_HOME/admin/misc/create_admin_keypair.sh
    fi
    chown "$CONF_OWN" "$LSLB_HOME/admin/conf/jcryption_keypair"
    chmod 0600 "$LSLB_HOME/admin/conf/jcryption_keypair"
    #enableAdminPHPA

    if [ $INST_USER = "root" ]; then
        $LSLB_HOME/admin/misc/rc-inst.sh
    fi 
    
}


setupPHPAccelerator()
{
    cat <<EOF

PHP Opcode Cache Setup

In order to maximize the performance of PHP, a pre-built PHP opcode cache
can be installed automatically. An opcode cache increases performance of
PHP scripts by caching them in compiled state, the overhead of compiling
is avoided.

Note: If an opcode cache has been installed already, you do not need to
      change it. If you built PHP binary by yourself, you need to built
      PHP opcode cache from source as well, unless the version of your
      PHP binary is same as the pre-built PHP opcode cache built for.

EOF

    printf "%s" "Would you like to change PHP opcode cache setting [y/N]? "

    read PHPACC
    echo

    if [ "x$PHPACC" = "x" ]; then
        PHPACC=n
    fi
    if [ `expr "$PHPACC" : '[Yy]'` -gt 0 ]; then
        $LSLB_HOME/admin/misc/enable_phpa.sh
    fi
}



finish()
{
    cat <<EOF
Congratulations! The LiteSpeed Load Balancer has been successfully installed.
Command line script - "$LSLB_HOME/bin/lslbctrl"
can be used to start or stop the server.

It is recommended to limit access to the web administration console. 
Right now the interface can be accessed from anywhere where this
machine can be reached over the network. Three options are available:
1. If the interface needs to be accessed only from this machine, just 
   change the listener for the interface to only listen on the loopback 
   interface - localhost(127.0.0.1). 
2. If the interface needs to be accessible from limited IP addresses or sub 
   networks, then set up access control rules for the interface accordingly.
3. If the interface has to be accessible via internet, SSL (Secure Sockets 
   Layer) should be used. Please read respective HOW-TOs on SSL configuration.

To change configurations of the interface, login and click 
"Interface Configuration" button on the main page.
The administration interface is located at http://localhost:<ADMIN_PORT>/
or http://<ip_or_Hostname_of_this_machine>:<ADMIN_PORT>/ 

EOF

    if [ $INST_USER = "root" ]; then
        if [ $INSTALL_TYPE != "upgrade" ]; then
            printf "%s" "Would you like to have LiteSpeed Load Balancer started automatically when the machine restarts [Y/n]? "
            read START_SERVER
            echo 

            if [ "x$START_SERVER" = "x" ]; then
                START_SERVER=y
            fi
            if [ `expr "$START_SERVER" : '[Yy]'` -gt 0 ]; then
                $LSLB_HOME/admin/misc/rc-inst.sh
            else
                cat <<EOF
If you want to start the server automatically later, just run 
    "$LSLB_HOME/admin/misc/rc-inst.sh"
to install the service control script.

EOF
            fi
        fi
    fi

    if [ $INSTALL_TYPE != "upgrade" ]; then
        printf "%s" "Would you like to start it right now [Y/n]? "
    else
        printf "%s" "Would you like to restart it right now [Y/n]? "
    fi
    read START_SERVER
    echo 

    if [ "x$START_SERVER" = "x" ]; then
        START_SERVER=y
    fi

    if [ `expr "$START_SERVER" : '[Yy]'` -gt 0 ]; then
        if [ $INSTALL_TYPE != "upgrade" ]; then
            "$LSLB_HOME/bin/lslbctrl" start
        else
            "$LSLB_HOME/bin/lslbctrl" restart
        fi
    else
        exit 0
    fi

    sleep 1
    RUNNING_PROCESS=`$PS_CMD | grep lslbd | grep -v grep`

    if [ "x$RUNNING_PROCESS" != "x" ]; then

        cat <<EOF

LiteSpeed Load Balancer started successfully! Have fun!

EOF
        exit 0
    else

        cat <<EOF

[ERROR] Failed to start the load balancer. For trouble shooting information,
        please refer to documents in "$LSLB_HOME/docs/".

EOF
    fi

}

