#!/bin/sh

cd `dirname "$0"`
. ./functions.sh
if [ $? != 0 ]; then
    echo [ERROR] Can not include 'functions.sh'.
    exit 1
fi

test_license()
{
	if [ -f "$LSLB_HOME/conf/license.key" ] && [ ! -f "$LSINSTALL_DIR/license.key" ]; then
		cp "$LSLB_HOME/conf/license.key" "$LSINSTALL_DIR/license.key"
	fi
	if [ -f "$LSLB_HOME/conf/serial.no" ] && [ ! -f "$LSINSTALL_DIR/serial.no" ]; then
		cp "$LSLB_HOME/conf/serial.no" "$LSINSTALL_DIR/serial.no"
	fi
	if [ -f "$LSINSTALL_DIR/license.key" ] && [ -f "$LSINSTALL_DIR/serial.no" ]; then
		echo "License key and serial number are available, testing..."
		echo
		bin/lslbd -t
		if [ $? -eq 0 ]; then
			LICENSE_OK=1
		fi
		echo
	fi

    if [ "x$LICENSE_OK" = "x" ]; then
        if [ -f "$LSINSTALL_DIR/serial.no" ]; then
            echo "Serial number is available."
            printf "Would you like to register a license key for this server? [Y/n]"
            read TMP_YN
            echo ""
            if [ "x$TMP_YN" = "x" ] || [ `expr "$TMP_YN" : '[Yy]'` -gt 0 ]; then
                echo "Contacting licensing server ..."

                echo ""
                $LSINSTALL_DIR/bin/lslbd -r
                echo ""
                
                if [ $? -eq 0 ]; then
                    echo "[OK] License key recived."
                    $LSINSTALL_DIR/bin/lslbd -t
                    if [ $? -eq 0 ]; then
                        LICENSE_OK=1
                    else
                        echo "The license key recevied does not work."
                    fi
                fi
            fi
        fi
    fi

	if [ "x$LICENSE_OK" = "x" ]; then
		if [ -f "$LSINSTALL_DIR/serial.no" ]; then
			echo "Serial number is available."
			printf "Would you like to generate a license key request? [Y/n]"
			read TMP_YN
			echo ""
			if [ "x$TMP_YN" = "x" ] || [ `expr "$TMP_YN" : '[Yy]'` -gt 0 ]; then
				$LSINSTALL_DIR/bin/lslbd -l
				if [ $? -eq 0 ]; then
					cat <<EOF

NOTE:

Please send "lic_req.bin" to register@litespeedtech.com to register your 
server. A new production license key will be sent to you via email. 
After receiving the new key please copy it to this directory and run 
the Installer again.

Please remember to set ftp to BINARY mode when ftp lic_req.bin or 
license.key to or from another machine.
 
EOF
					exit 0
				else
					exit 1
				fi
			fi
			
		fi

		if [ -f "$LSINSTALL_DIR/trial.key" ]; then
			$LSINSTALL_DIR/bin/lslbd -t
			if [ $? -ne 0 ]; then
				exit 1
			fi
		else
			cat <<EOF
[ERROR] Sorry, installation will abort without a valid license key.
 
For evaluation purpose, please obtain a trial license key from our web 
site http://www.litespeedtech.com, copy it to this directory 
and run Installer again.

If a production license has been purchased, please copy the serial number
from your confirmation email to this directory and run Installer again.

NOTE:
Please remember to set ftp to BINARY mode when you ftp trial.key from 
another machine.

EOF
			exit 1
		fi

	fi

}

configChroot()
{
	ENABLE_CHROOT=0
	CHROOT_PATH="/"
	if [ -f "$LSLB_HOME/conf/lslbd_config.xml" ]; then
		OLD_ENABLE_CHROOT_CONF=`grep "<enableChroot>" "$LSLB_HOME/conf/lslbd_config.xml"`
		OLD_CHROOT_PATH_CONF=`grep "<chrootPath>" "$LSLB_HOME/conf/lslbd_config.xml"`
		OLD_ENABLE_CHROOT=`expr "$OLD_ENABLE_CHROOT_CONF" : '.*<enableChroot>\(.*\)</enableChroot>.*'`
		OLD_CHROOT_PATH=`expr "$OLD_CHROOT_PATH_CONF" : '[^<]*<chrootPath>\([^<]*\)</chrootPath>.*'`
		if [ "x$OLD_ENABLE_CHROOT" != "x" ]; then
			ENABLE_CHROOT=$OLD_ENABLE_CHROOT
		fi
		if [ "x$OLD_CHROOT_PATH" != "x" ]; then
			CHROOT_PATH=$OLD_CHROOT_PATH
		fi
	fi
	CHANGE_CHROOT=0
	if [ $INST_USER = "root" ]; then
		CHANGE_CHROOT=1

		if [ $INSTALL_TYPE = "upgrade" ]; then
			CHANGE_CHROOT=0
			if [ $ENABLE_CHROOT -eq 1 ]; then
				cat <<EOF
Chroot is enabled with your current setup and root directory is set to 
    $CHROOT_PATH

EOF
			else
				echo "Chroot is disabled with your current setup."
				echo
			fi
			printf "%s" "Would you like to change chroot settings [y/N]? "
			read TMP_URC
			echo ""
			if [ "x$TMP_URC" != "x" ]; then
				if [ `expr "$TMP_URC" : '[Yy]'` -gt 0 ]; then
					CHANGE_CHROOT=1
				fi
			fi
		fi		

		if [ $CHANGE_CHROOT -eq 1 ]; then

			cat<<EOF

LiteSpeed Load Balancer can run in chroot environment.
It is impossible for the chrooted process and its children processes to 
access files outside the new root directory. This will provide an extra
layer of protection.

This installation script will try to setup the initial chroot environment 
automatically.

EOF

			SUCC=0
			printf "%s" "Enable chroot [y/N]: "
			read TMP_YN
			if [ `expr "x$TMP_YN" : 'x[Yy]'` -gt 1 ]; then
				ENABLE_CHROOT=1
			fi

			LSLB_HOME_LEN=`expr "$LSLB_HOME" : '.*'`
			if [ $ENABLE_CHROOT -eq 1 ]; then
				while [ $SUCC -eq 0 ]; do
					cat <<EOF

Chroot path must be absolute path and the server root 
	$LSLB_HOME
must be included in the chroot directory tree.

EOF
					printf "%s" "Chroot directory without trailing '/': "
					read TMP_CHROOT
					if [ "x$TMP_CHROOT" != "x" ]; then
						if [ $TMP_CHROOT = '/' ]; then
							echo "Set chroot directory to '/' will disable chroot."
							printf "%s" "Are you sure? [y/N]"
							read TMP_YN
							if [ `expr "x$TMP_YN" : 'x[Yy]'` -gt 1 ]; then
								ENABLE_CHROOT=0
								SUCC=1
							fi
						else
							CHROOT_LEN=`expr "$TMP_CHROOT" : '.*'`
							MATCH_LEN=`expr "$LSLB_HOME" : "$TMP_CHROOT"`
							if [ $CHROOT_LEN -ne $MATCH_LEN ]; then
								echo "Server root is not included in the chroot directory tree"
							else
								TMP_CHROOT2="$TMP_CHROOT/"
								TMP_HOME="$LSLB_HOME/"
								MATCH_LEN=`expr "$TMP_HOME" : "$TMP_CHROOT2"`
								if [ $MATCH_LEN -le $CHROOT_LEN ]; then
									echo "Server root is not included in the chroot diretory tree"
								else
									SUCC=1
									CHROOT_PATH=$TMP_CHROOT
								fi
							fi
						fi
					fi
				done
			fi
		fi
	fi
}

changeChroot()
{
	util_cpfile "$SDIR_OWN" $EXEC_MOD admin/misc/chroot.sh 

	if [ $CHANGE_CHROOT -eq 1 ]; then

		if [ $ENABLE_CHROOT -eq 1 ]; then
			$LSLB_HOME/admin/misc/chroot.sh $CHROOT_PATH
			$LSLB_HOME/admin/misc/chroot.sh $CHROOT_PATH $LSLB_HOME/bin/lslbd
			$LSLB_HOME/admin/misc/chroot.sh $CHROOT_PATH $LSLB_HOME/admin/fcgi-bin/admin_php
            #if [ -f $LSLB_HOME/fcgi-bin/php ]; then
    		#	$LSLB_HOME/admin/misc/chroot.sh $CHROOT_PATH $LSLB_HOME/fcgi-bin/php
            #fi
			#$LSLB_HOME/admin/misc/chroot.sh $CHROOT_PATH $LSLB_HOME/fcgi-bin/lsphp
            if [ `expr "x$CHROOT_PATH" : '^x/[^/]'` -gt 1 ]; then 
                egrep "$WS_USER|lsadm" /etc/passwd > $CHROOT_PATH/etc/passwd
                grep "$WS_GROUP" /etc/group > $CHROOT_PATH/etc/group
            fi
		fi	
		cp $LSLB_HOME/conf/lslbd_config.xml $LSLB_HOME/conf/lslbd_config.xml.bak
		chown "$DIR_OWN" $LSLB_HOME/conf/lslbd_config.xml.bak
		RES=`grep '</chrootPath>' $LSLB_HOME/conf/lslbd_config.xml.bak`
		if [ $? -eq 1 ]; then
			sed -e "s#</group>#</group><chrootPath>$CHROOT_PATH</chrootPath><enableChroot>$ENABLE_CHROOT</enableChroot>#" "$LSLB_HOME/conf/lslbd_config.xml.bak" > "$LSLB_HOME/conf/lslbd_config.xml" 
		else
			sed -e "s#<chrootPath>.*<\/chrootPath>#<chrootPath>$CHROOT_PATH<\/chrootPath>#" -e "s/<enableChroot>.*<\/enableChroot>/<enableChroot>$ENABLE_CHROOT<\/enableChroot>/" "$LSLB_HOME/conf/lslbd_config.xml.bak" > "$LSLB_HOME/conf/lslbd_config.xml"
		fi
	fi
}

installLicense()
{
	if [ -f ./serial.no ]; then
		cp -f ./serial.no $LSLB_HOME/conf
		chown "$DIR_OWN" $LSLB_HOME/conf/serial.no
		chmod "$CONF_MOD" $LSLB_HOME/conf/serial.no
	fi

	if [ -f ./license.key ]; then
		cp -f ./license.key $LSLB_HOME/conf
		chown "$DIR_OWN" $LSLB_HOME/conf/license.key
		chmod "$CONF_MOD" $LSLB_HOME/conf/license.key
	fi

	if [ -f ./trial.key ]; then
		cp -f ./trial.key $LSLB_HOME/conf
		chown "$DIR_OWN" $LSLB_HOME/conf/trial.key
		chmod "$CONF_MOD" $LSLB_HOME/conf/trial.key
	fi
}

LSINSTALL_DIR=`dirname "$0"`
cd $LSINSTALL_DIR

init
license
install_dir
test_license
admin_login


if [ $INSTALL_TYPE = "reinstall" ]; then

	getUserGroup
	stopLslbd
	getServerPort
	getAdminPort
	#enablePHPHandler
fi

configChroot

cat <<EOF

Installing, please wait... 

EOF

buildConfigFiles

installation


changeChroot

installLicense

#setupPHPAccelerator
finish


